@@ -132,7 +132,7 @@ public:
 };
 
 // event handler to capture keyboard events and use them to advance the technique used for rendering
-class TechniqueEventHandler : public osgGA::GUIEventHandler, public osg::NodeCallback
+class TechniqueEventHandler : public osgGA::GUIEventHandler
 {
 public:
 
@@ -142,7 +142,7 @@ public:
 
     virtual void accept(osgGA::GUIEventHandlerVisitor& v) { v.visit(*this); }
 
-    virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&);
+    virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&, osg::Object*, osg::NodeVisitor*);
     
     virtual void getUsage(osg::ApplicationUsage& usage) const;
 
@@ -157,7 +157,7 @@ protected:
         
 };
 
-bool TechniqueEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&)
+bool TechniqueEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&, osg::Object*, osg::NodeVisitor*)
 {
     switch(ea.getEventType())
     {
@@ -385,7 +385,7 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
     {
 
         osg::HeightField* grid = new osg::HeightField;
-        grid->allocateGrid(numColumns,numRows);
+        grid->allocate(numColumns,numRows);
         grid->setOrigin(origin);
         grid->setXInterval(size.x()/(float)(numColumns-1));
         grid->setYInterval(size.y()/(float)(numRows-1));
