@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -21,7 +21,8 @@ using namespace osg;
  */
 ClearNode::ClearNode():
     _requiresClear(true),
-    _clearColor(0.0f,0.0f,0.0f,1.0f)
+    _clearColor(0.0f,0.0f,0.0f,1.0f),
+    _clearMask(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
 {
     setCullingActive(false);
     StateSet* stateset = new StateSet;
