@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -259,7 +259,7 @@ void Drawable::flushDeletedVertexBufferObjects(unsigned int contextID,double /*c
 
 Drawable::Drawable()
 {
-    _bbox_computed = false;
+    _boundingBoxComputed = false;
 
     // Note, if your are defining a subclass from drawable which is
     // dynamically updated then you should set both the following to
@@ -271,20 +271,27 @@ Drawable::Drawable()
 
     _supportsVertexBufferObjects = false;
     _useVertexBufferObjects = false;
+
+    _numChildrenRequiringUpdateTraversal = 0;
+    _numChildrenRequiringEventTraversal = 0;
 }
 
 Drawable::Drawable(const Drawable& drawable,const CopyOp& copyop):
     Object(drawable,copyop),
     _parents(), // leave empty as parentList is managed by Geode
     _stateset(copyop(drawable._stateset.get())),
-    _bbox(drawable._bbox),
-    _bbox_computed(drawable._bbox_computed),
+    _initialBound(drawable._initialBound),
+    _boundingBox(drawable._boundingBox),
+    _boundingBoxComputed(drawable._boundingBoxComputed),
     _shape(copyop(drawable._shape.get())),
     _supportsDisplayList(drawable._supportsDisplayList),
     _useDisplayList(drawable._useDisplayList),
     _supportsVertexBufferObjects(drawable._supportsVertexBufferObjects),
     _useVertexBufferObjects(drawable._useVertexBufferObjects),
     _updateCallback(drawable._updateCallback),
+    _numChildrenRequiringUpdateTraversal(drawable._numChildrenRequiringUpdateTraversal),
+    _eventCallback(drawable._eventCallback),
+    _numChildrenRequiringEventTraversal(drawable._numChildrenRequiringEventTraversal),
     _cullCallback(drawable._cullCallback),
     _drawCallback(drawable._drawCallback)
 {
@@ -306,17 +313,141 @@ void Drawable::removeParent(osg::Node* node)
     if (pitr!=_parents.end()) _parents.erase(pitr);
 }
 
+
+void Drawable::setStateSet(osg::StateSet* stateset)
+{
+    // do nothing if nothing changed.
+    if (_stateset==stateset) return;
+    
+    // track whether we need to account for the need to do a update or event traversal.
+    int delta_update = 0;
+    int delta_event = 0;
+
+    // remove this node from the current statesets parent list 
+    if (_stateset.valid())
+    {
+        _stateset->removeParent(this);
+        if (_stateset->requiresUpdateTraversal()) --delta_update;
+        if (_stateset->requiresEventTraversal()) --delta_event;
+    }
+    
+    // set the stateset.
+    _stateset = stateset;
+    
+    // add this node to the new stateset to the parent list.
+    if (_stateset.valid())
+    {
+        _stateset->addParent(this);
+        if (_stateset->requiresUpdateTraversal()) ++delta_update;
+        if (_stateset->requiresEventTraversal()) ++delta_event;
+    }
+    
+
+    // only inform parents if change occurs and drawable doesn't already have an update callback
+    if (delta_update!=0 && !_updateCallback)
+    {
+        for(ParentList::iterator itr=_parents.begin();
+            itr!=_parents.end();
+            ++itr)
+        {
+            (*itr)->setNumChildrenRequiringUpdateTraversal( (*itr)->getNumChildrenRequiringUpdateTraversal()+delta_update );
+        }
+    }
+
+    // only inform parents if change occurs and drawable doesn't already have an event callback
+    if (delta_event!=0 && !_eventCallback)
+    {
+        for(ParentList::iterator itr=_parents.begin();
+            itr!=_parents.end();
+            ++itr)
+        {
+            (*itr)->setNumChildrenRequiringEventTraversal( (*itr)->getNumChildrenRequiringEventTraversal()+delta_event );
+        }
+    }
+
+
+}
+
+void Drawable::setNumChildrenRequiringUpdateTraversal(unsigned int num)
+{
+    // if no changes just return.
+    if (_numChildrenRequiringUpdateTraversal==num) return;
+
+    // note, if _updateCallback is set then the
+    // parents won't be affected by any changes to
+    // _numChildrenRequiringUpdateTraversal so no need to inform them.
+    if (!_updateCallback && !_parents.empty())
+    {
+        // need to pass on changes to parents.        
+        int delta = 0;
+        if (_numChildrenRequiringUpdateTraversal>0) --delta;
+        if (num>0) ++delta;
+        if (delta!=0)
+        {
+            // the number of callbacks has changed, need to pass this
+            // on to parents so they know whether app traversal is
+            // reqired on this subgraph.
+            for(ParentList::iterator itr =_parents.begin();
+                itr != _parents.end();
+                ++itr)
+            {    
+                (*itr)->setNumChildrenRequiringUpdateTraversal( (*itr)->getNumChildrenRequiringUpdateTraversal()+delta );
+            }
+
+        }
+    }
+    
+    // finally update this objects value.
+    _numChildrenRequiringUpdateTraversal=num;
+    
+}
+
+
+void Drawable::setNumChildrenRequiringEventTraversal(unsigned int num)
+{
+    // if no changes just return.
+    if (_numChildrenRequiringEventTraversal==num) return;
+
+    // note, if _eventCallback is set then the
+    // parents won't be affected by any changes to
+    // _numChildrenRequiringEventTraversal so no need to inform them.
+    if (!_eventCallback && !_parents.empty())
+    {
+        // need to pass on changes to parents.        
+        int delta = 0;
+        if (_numChildrenRequiringEventTraversal>0) --delta;
+        if (num>0) ++delta;
+        if (delta!=0)
+        {
+            // the number of callbacks has changed, need to pass this
+            // on to parents so they know whether app traversal is
+            // reqired on this subgraph.
+            for(ParentList::iterator itr =_parents.begin();
+                itr != _parents.end();
+                ++itr)
+            {    
+                (*itr)->setNumChildrenRequiringEventTraversal( (*itr)->getNumChildrenRequiringEventTraversal()+delta );
+            }
+
+        }
+    }
+    
+    // finally Event this objects value.
+    _numChildrenRequiringEventTraversal=num;
+    
+}
+
 osg::StateSet* Drawable::getOrCreateStateSet()
 {
-    if (!_stateset) _stateset = new StateSet;
+    if (!_stateset) setStateSet(new StateSet);
     return _stateset.get();
 }
 
 void Drawable::dirtyBound()
 {
-    if (_bbox_computed)
+    if (_boundingBoxComputed)
     {
-        _bbox_computed = false;
+        _boundingBoxComputed = false;
 
         // dirty parent bounding sphere's to ensure that all are valid.
         for(ParentList::iterator itr=_parents.begin();
@@ -360,6 +491,8 @@ void Drawable::compileGLObjects(State& state) const
 
 void Drawable::releaseGLObjects(State* state) const
 {
+    if (_stateset.valid()) _stateset->releaseGLObjects(state);
+
     if (!_useDisplayList) return;
     
     if (state)
@@ -486,7 +619,7 @@ void Drawable::setUpdateCallback(UpdateCallback* ac)
 
     _updateCallback = ac;
     
-    if (delta!=0)
+    if (delta!=0 && !(_stateset.valid() && _stateset->requiresUpdateTraversal()))
     {
         for(ParentList::iterator itr=_parents.begin();
             itr!=_parents.end();
@@ -497,7 +630,28 @@ void Drawable::setUpdateCallback(UpdateCallback* ac)
     }
 }
 
-struct ComputeBound : public Drawable::PrimitiveFunctor
+void Drawable::setEventCallback(EventCallback* ac)
+{
+    if (_eventCallback==ac) return;
+    
+    int delta = 0;
+    if (_eventCallback.valid()) --delta;
+    if (ac) ++delta;
+
+    _eventCallback = ac;
+    
+    if (delta!=0 && !(_stateset.valid() && _stateset->requiresEventTraversal()))
+    {
+        for(ParentList::iterator itr=_parents.begin();
+            itr!=_parents.end();
+            ++itr)
+        {
+            (*itr)->setNumChildrenRequiringEventTraversal( (*itr)->getNumChildrenRequiringEventTraversal()+delta );
+        }
+    }
+}
+
+struct ComputeBound : public PrimitiveFunctor
 {
         ComputeBound():_vertices(0) {}
         
@@ -571,23 +725,20 @@ struct ComputeBound : public Drawable::PrimitiveFunctor
         BoundingBox     _bb;
 };
 
-bool Drawable::computeBound() const
+BoundingBox Drawable::computeBound() const
 {
     ComputeBound cb;
 
     Drawable* non_const_this = const_cast<Drawable*>(this);
     non_const_this->accept(cb);
     
-    _bbox = cb._bb;
-    _bbox_computed = true;
-
-    return true;
+    return cb._bb;
 }
 
 void Drawable::setBound(const BoundingBox& bb) const
 {
-     _bbox = bb;
-     _bbox_computed = true;
+     _boundingBox = bb;
+     _boundingBoxComputed = true;
 }
 
 
@@ -601,7 +752,7 @@ static BufferedExtensions s_extensions;
 
 Drawable::Extensions* Drawable::getExtensions(unsigned int contextID,bool createIfNotInitalized)
 {
-    if (!s_extensions[contextID] && createIfNotInitalized) s_extensions[contextID] = new Drawable::Extensions;
+    if (!s_extensions[contextID] && createIfNotInitalized) s_extensions[contextID] = new Drawable::Extensions(contextID);
     return s_extensions[contextID].get();
 }
 
@@ -610,9 +761,9 @@ void Drawable::setExtensions(unsigned int contextID,Extensions* extensions)
     s_extensions[contextID] = extensions;
 }
 
-Drawable::Extensions::Extensions()
+Drawable::Extensions::Extensions(unsigned int contextID)
 {
-    setupGLExtenions();
+    setupGLExtenions(contextID);
 }
 
 Drawable::Extensions::Extensions(const Extensions& rhs):
@@ -690,6 +841,12 @@ void Drawable::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._glBufferData) _glBufferData = 0;
     if (!rhs._glBufferSubData) _glBufferSubData = 0;
     if (!rhs._glDeleteBuffers) _glDeleteBuffers = 0;
+    if (!rhs._glIsBuffer) _glIsBuffer = 0;
+    if (!rhs._glGetBufferSubData) _glGetBufferSubData = 0;
+    if (!rhs._glMapBuffer) _glMapBuffer = 0;
+    if (!rhs._glUnmapBuffer) _glUnmapBuffer = 0;
+    if (!rhs._glGetBufferParameteriv) _glGetBufferParameteriv = 0;
+    if (!rhs._glGetBufferPointerv) _glGetBufferPointerv = 0;
 
     if (!rhs._glGenOcclusionQueries) _glGenOcclusionQueries = 0;
     if (!rhs._glDeleteOcclusionQueries) _glDeleteOcclusionQueries = 0;
@@ -709,14 +866,14 @@ void Drawable::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._gl_get_query_objectuiv_arb) _gl_get_query_objectuiv_arb = 0;
 }
 
-void Drawable::Extensions::setupGLExtenions()
+void Drawable::Extensions::setupGLExtenions(unsigned int contextID)
 {
-    _isVertexProgramSupported = isGLExtensionSupported("GL_ARB_vertex_program");
-    _isSecondaryColorSupported = isGLExtensionSupported("GL_EXT_secondary_color");
-    _isFogCoordSupported = isGLExtensionSupported("GL_EXT_fog_coord");
-    _isMultiTexSupported = isGLExtensionSupported("GL_ARB_multitexture");
-    _isOcclusionQuerySupported = osg::isGLExtensionSupported( "GL_NV_occlusion_query" );
-    _isARBOcclusionQuerySupported = osg::isGLExtensionSupported( "GL_ARB_occlusion_query" );
+    _isVertexProgramSupported = isGLExtensionSupported(contextID,"GL_ARB_vertex_program");
+    _isSecondaryColorSupported = isGLExtensionSupported(contextID,"GL_EXT_secondary_color");
+    _isFogCoordSupported = isGLExtensionSupported(contextID,"GL_EXT_fog_coord");
+    _isMultiTexSupported = isGLExtensionSupported(contextID,"GL_ARB_multitexture");
+    _isOcclusionQuerySupported = osg::isGLExtensionSupported(contextID, "GL_NV_occlusion_query" );
+    _isARBOcclusionQuerySupported = osg::isGLExtensionSupported(contextID, "GL_ARB_occlusion_query" );
 
     _glFogCoordfv = ((FogCoordProc)osg::getGLExtensionFuncPtr("glFogCoordfv","glFogCoordfvEXT"));
     _glSecondaryColor3ubv = ((SecondaryColor3ubvProc)osg::getGLExtensionFuncPtr("glSecondaryColor3ubv","glSecondaryColor3ubvEXT"));
@@ -739,6 +896,12 @@ void Drawable::Extensions::setupGLExtenions()
     _glBufferData = ((BufferDataProc)osg::getGLExtensionFuncPtr("glBufferData","glBufferDataARB"));
     _glBufferSubData = ((BufferSubDataProc)osg::getGLExtensionFuncPtr("glBufferSubData","glBufferSubDataARB"));
     _glDeleteBuffers = ((DeleteBuffersProc)osg::getGLExtensionFuncPtr("glDeleteBuffers","glDeleteBuffersARB"));
+    _glIsBuffer = ((IsBufferProc)osg::getGLExtensionFuncPtr("glIsBuffer","glIsBufferARB"));
+    _glGetBufferSubData = ((GetBufferSubDataProc)osg::getGLExtensionFuncPtr("glGetBufferSubData","glGetBufferSubDataARB"));
+    _glMapBuffer = ((MapBufferProc)osg::getGLExtensionFuncPtr("glMapBuffer","glMapBufferARB"));
+    _glUnmapBuffer = ((UnmapBufferProc)osg::getGLExtensionFuncPtr("glUnmapBuffer","glUnmapBufferARB"));
+    _glGetBufferParameteriv = ((GetBufferParameterivProc)osg::getGLExtensionFuncPtr("glGetBufferParameteriv","glGetBufferParameterivARB"));
+    _glGetBufferPointerv = ((GetBufferPointervProc)osg::getGLExtensionFuncPtr("glGetBufferPointerv","glGetBufferPointervARB"));
 
     _glGenOcclusionQueries = ((GenOcclusionQueriesProc)osg::getGLExtensionFuncPtr("glGenOcclusionQueries","glGenOcclusionQueriesNV"));
     _glDeleteOcclusionQueries = ((DeleteOcclusionQueriesProc)osg::getGLExtensionFuncPtr("glDeleteOcclusionQueries","glDeleteOcclusionQueriesNV"));
@@ -928,64 +1091,83 @@ void Drawable::Extensions::glVertexAttrib4Nubv(unsigned int index, const GLubyte
 
 void Drawable::Extensions::glGenBuffers(GLsizei n, GLuint *buffers) const
 {
-    if (_glGenBuffers)
-    {
-        _glGenBuffers(n, buffers); 
-    }
-    else
-    {
-        notify(WARN)<<"Error: glGenBuffers not supported by OpenGL driver"<<std::endl;
-    }
+    if (_glGenBuffers) _glGenBuffers(n, buffers); 
+    else notify(WARN)<<"Error: glGenBuffers not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glBindBuffer(GLenum target, GLuint buffer) const
 {
-    if (_glBindBuffer)
-    {
-        _glBindBuffer(target, buffer); 
-    }
-    else
-    {
-        notify(WARN)<<"Error: glBindBuffer not supported by OpenGL driver"<<std::endl;
-    }
+    if (_glBindBuffer) _glBindBuffer(target, buffer); 
+    else notify(WARN)<<"Error: glBindBuffer not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glBufferData(GLenum target, GLsizeiptrARB size, const GLvoid *data, GLenum usage) const
 {
-    if (_glBufferData)
-    {
-        _glBufferData(target, size, data, usage); 
-    }
-    else
-    {
-        notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
-    }
+    if (_glBufferData) _glBufferData(target, size, data, usage); 
+    else notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glBufferSubData(GLenum target, GLintptrARB offset, GLsizeiptrARB size, const GLvoid *data) const
 {
-    if (_glBufferSubData)
-    {
-        _glBufferSubData(target, offset, size, data); 
-    }
-    else
-    {
-        notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
-    }
+    if (_glBufferSubData) _glBufferSubData(target, offset, size, data); 
+    else notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glDeleteBuffers(GLsizei n, const GLuint *buffers) const
 {
-    if (_glDeleteBuffers)
+    if (_glDeleteBuffers) _glDeleteBuffers(n, buffers); 
+    else notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
+}
+
+GLboolean Drawable::Extensions::glIsBuffer (GLuint buffer) const
+{
+    if (_glIsBuffer) return _glIsBuffer(buffer);
+    else 
     {
-        _glDeleteBuffers(n, buffers); 
-    }
-    else
-    {
-        notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
+        notify(WARN)<<"Error: glIsBuffer not supported by OpenGL driver"<<std::endl;
+        return GL_FALSE;
     }
 }
 
+void Drawable::Extensions::glGetBufferSubData (GLenum target, GLintptrARB offset, GLsizeiptrARB size, GLvoid *data) const
+{
+    if (_glGetBufferSubData) _glGetBufferSubData(target,offset,size,data);
+    else notify(WARN)<<"Error: glGetBufferSubData not supported by OpenGL driver"<<std::endl;
+}
+
+GLvoid* Drawable::Extensions::glMapBuffer (GLenum target, GLenum access) const
+{
+    if (_glMapBuffer) return _glMapBuffer(target,access);
+    else 
+    {
+        notify(WARN)<<"Error: glMapBuffer not supported by OpenGL driver"<<std::endl;
+        return 0;
+    }
+}
+
+GLboolean Drawable::Extensions::glUnmapBuffer (GLenum target) const
+{
+    if (_glUnmapBuffer) return _glUnmapBuffer(target);
+    else 
+    {
+        notify(WARN)<<"Error: glUnmapBuffer not supported by OpenGL driver"<<std::endl;
+        return GL_FALSE;
+    }
+}
+
+void Drawable::Extensions::glGetBufferParameteriv (GLenum target, GLenum pname, GLint *params) const
+{
+    if (_glGetBufferParameteriv) _glGetBufferParameteriv(target,pname,params);
+    else notify(WARN)<<"Error: glGetBufferParameteriv not supported by OpenGL driver"<<std::endl;
+}
+
+void Drawable::Extensions::glGetBufferPointerv (GLenum target, GLenum pname, GLvoid* *params) const
+{
+    if (_glGetBufferPointerv) _glGetBufferPointerv(target,pname,params);
+    else notify(WARN)<<"Error: glGetBufferPointerv not supported by OpenGL driver"<<std::endl;
+}
+
+
 void Drawable::Extensions::glGenOcclusionQueries( GLsizei n, GLuint *ids ) const
 {
     if (_glGenOcclusionQueries)
