@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -41,7 +41,7 @@ Image::Image()
     _allocationMode         = USE_NEW_DELETE;
     _data                   = (unsigned char *)0L;
 
-    _modifiedTag = 0;
+    _modifiedCount = 0;
 }
 
 Image::Image(const Image& image,const CopyOp& copyop):
@@ -53,7 +53,7 @@ Image::Image(const Image& image,const CopyOp& copyop):
     _dataType(image._dataType),
     _packing(image._packing),
     _data(0L),
-    _modifiedTag(image._modifiedTag),
+    _modifiedCount(image._modifiedCount),
     _mipmapData(image._mipmapData)
 {
     if (image._data)
@@ -81,9 +81,16 @@ void Image::deallocateData()
 
 int Image::compare(const Image& rhs) const
 {
-    if (getFileName()<rhs.getFileName()) return -1;
-    else if (getFileName()>rhs.getFileName()) return 1;
-    
+    // if at least one filename is empty, then need to test buffer
+    // pointers because images could have been created on the fly
+    // and therefore we can't rely on file names to get an accurate
+    // comparison
+    if (getFileName().empty() || rhs.getFileName().empty())
+    {
+        if (_data<rhs._data) return -1;
+        if (_data>rhs._data) return 1;
+    }
+
     // need to test against image contents here...
     COMPARE_StateAttribute_Parameter(_s)
     COMPARE_StateAttribute_Parameter(_t)
@@ -91,14 +98,15 @@ int Image::compare(const Image& rhs) const
     COMPARE_StateAttribute_Parameter(_pixelFormat)
     COMPARE_StateAttribute_Parameter(_dataType)
     COMPARE_StateAttribute_Parameter(_packing)
-    COMPARE_StateAttribute_Parameter(_modifiedTag)
+    COMPARE_StateAttribute_Parameter(_mipmapData)
+    COMPARE_StateAttribute_Parameter(_modifiedCount)
+
+    // same buffer + same parameters = same image
+    if (_data == rhs._data) return 0;
+
+    // slowest comparison at the bottom!
+    COMPARE_StateAttribute_Parameter(getFileName())    
 
-    if (_data<rhs._data) return -1;
-    if (_data>rhs._data) return 1;
-    
-    if (_mipmapData<rhs._mipmapData) return -1;
-    if (_mipmapData>rhs._mipmapData) return 1;
-    
     return 0;
 }
 
@@ -172,24 +180,37 @@ unsigned int Image::computeNumComponents(GLenum pixelFormat)
 
 unsigned int Image::computePixelSizeInBits(GLenum format,GLenum type)
 {
-    
-    if(format & 0x80F0)  /*  DXT* compressed formats */
+
+    switch(format)
     {
-        switch(format)
-        {
-            case(GL_COMPRESSED_RGB_S3TC_DXT1_EXT): return 4;
+        case(GL_COMPRESSED_RGB_S3TC_DXT1_EXT): return 4;
+        case(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT): return 4;
+        case(GL_COMPRESSED_RGBA_S3TC_DXT3_EXT): return 8;
+        case(GL_COMPRESSED_RGBA_S3TC_DXT5_EXT): return 8;
+        default: break;
+    }
 
-            case(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT): return 4;
-
-            case(GL_COMPRESSED_RGBA_S3TC_DXT3_EXT): return 8;
-
-            case(GL_COMPRESSED_RGBA_S3TC_DXT5_EXT): return 8;
-            default: 
-                {
-                    notify(WARN)<<"error format = "<<format<<std::endl;
-                    return 0;
-                }
-        }
+    // note, haven't yet added proper handling of the ARB GL_COMPRESSRED_* pathways
+    // yet, no clear size for these since its probably implementation dependent
+    // which raises the question of how to actually querry for these sizes...
+    // will need to revisit this issue, for now just report an error.
+    // this is possible a bit of mute point though as since the ARB compressed formats
+    // arn't yet used for storing images to disk, so its likely that users wont have
+    // osg::Image's for pixel formats set the ARB compressed formats, just using these
+    // compressed formats as internal texture modes.  This is very much speculation though
+    // if get the below error then its time to revist this issue :-)
+    // Robert Osfield, Jan 2005.
+    switch(format)
+    {
+        case(GL_COMPRESSED_ALPHA):
+        case(GL_COMPRESSED_LUMINANCE):
+        case(GL_COMPRESSED_LUMINANCE_ALPHA):
+        case(GL_COMPRESSED_INTENSITY):
+        case(GL_COMPRESSED_RGB):
+        case(GL_COMPRESSED_RGBA):
+            notify(WARN)<<"Image::computePixelSizeInBits(format,type) : cannot compute correct size of compressed format ("<<format<<") returning 0."<<std::endl;
+            return 0;
+        default: break;
     }
 
     switch(type)
@@ -281,14 +302,17 @@ unsigned int Image::getTotalSizeInBytesIncludingMipmaps() const
    if (t==0) t=1;
    if (r==0) r=1;
    
-   unsigned int sizeOfLastMipMap;
-   if(_pixelFormat & 0x80F0)
+   unsigned int sizeOfLastMipMap = computeRowWidthInBytes(s,_pixelFormat,_dataType,_packing)* r*t;
+   switch(_pixelFormat)
    {
-       unsigned int blockSize = (_pixelFormat == GL_COMPRESSED_RGBA_S3TC_DXT1_EXT) ? 8 : 16;
-       sizeOfLastMipMap = maximum(computeRowWidthInBytes(s,_pixelFormat,_dataType,_packing)* r*t, blockSize);
+        case(GL_COMPRESSED_RGB_S3TC_DXT1_EXT):
+        case(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT):
+           sizeOfLastMipMap = maximum(sizeOfLastMipMap, 8u); // block size of 8
+        case(GL_COMPRESSED_RGBA_S3TC_DXT3_EXT):
+        case(GL_COMPRESSED_RGBA_S3TC_DXT5_EXT):
+           sizeOfLastMipMap = maximum(sizeOfLastMipMap, 16u); // block size of 16
+        default: break;
    }
-   else
-       sizeOfLastMipMap = computeRowWidthInBytes(s,_pixelFormat,_dataType,_packing)* r*t;
 
    // notify(INFO)<<"sizeOfLastMipMap="<<sizeOfLastMipMap<<"\ts="<<s<<"\tt="<<t<<"\tr"<<r<<std::endl;                  
 
@@ -363,7 +387,7 @@ void Image::allocateImage(int s,int t,int r,
         _internalTextureFormat = 0;
     }
     
-    ++_modifiedTag;
+    ++_modifiedCount;
 }
 
 void Image::setImage(int s,int t,int r,
@@ -387,7 +411,7 @@ void Image::setImage(int s,int t,int r,
 
     _packing = packing;
         
-    ++_modifiedTag;
+    ++_modifiedCount;
 
 }
 
@@ -507,7 +531,7 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
             extensions->glGetCompressedTexImage(textureMode, i, getMipmapData(i));
         }
 
-        ++_modifiedTag;
+        ++_modifiedCount;
     
     }
     else
@@ -560,7 +584,7 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
             glGetTexImage(textureMode,i,_pixelFormat,_dataType,getMipmapData(i));
         }
 
-        ++_modifiedTag;
+        ++_modifiedCount;
     }    
 }
 
@@ -623,7 +647,7 @@ void Image::scaleImage(int s,int t,int r, GLenum newDataType)
         notify(WARN) << "Error Image::scaleImage() did not succeed : errorString = "<<gluErrorString((GLenum)status)<<std::endl;
     }
     
-    ++_modifiedTag;
+    ++_modifiedCount;
 }
 
 void Image::copySubImage(int s_offset,int t_offset,int r_offset,osg::Image* source)
@@ -713,7 +737,7 @@ void Image::flipHorizontal()
         }
     }
     
-    ++_modifiedTag;
+    ++_modifiedCount;
 }
 
 void flipImageVertical(unsigned char* top, unsigned char* bottom, unsigned int rowSize)
@@ -797,7 +821,7 @@ void Image::flipVertical()
        }
     }   
 
-    ++_modifiedTag;
+    ++_modifiedCount;
 }
 
 
@@ -819,56 +843,63 @@ void Image::ensureValidSizeForTexturing(GLint maxTextureSize)
     }
 }
 
+
+template <typename T>    
+bool _findLowerAlphaValueInRow(unsigned int num, T* data,T value, unsigned int delta)
+{
+    for(unsigned int i=0;i<num;++i)
+    {
+        if (*data<value) return true;
+        data += delta;
+    }
+    return false;
+}
+
 bool Image::isImageTranslucent() const
 {
-    
-
-    if (getPixelFormat()==GL_LUMINANCE_ALPHA)
+    unsigned int offset = 0;
+    unsigned int delta = 1;
+    switch(_pixelFormat)
     {
-        return true;
-
-        
-        // unfinished code so commenting out right now...
-//         switch(type)
-//         {
-//             case(GL_BYTE):
-//             {
-//                 int rowSizeInBytes = getRowSizeInBytes();
-//                 int imageSizeInBytes = getImageSizeInBytes();
-//                 
-//                 for (int r=0;r<_r,++r)
-//                 {
-//                     for (int t=0;t<int _t,++t)
-//                     {
-//                         char* ptr = _data+r*getRowSizeInBytes()+image*getImageSizeInBytes();
-//                         for (int s=0;s<_s,++s)
-//                         {
-// 
-//                         }
-//                     }
-//                 }
-//                 break
-//             }    
-//             case(GL_UNSIGNED_BYTE): return 8*computeNumComponents(format);
-// 
-//             case(GL_SHORT):
-//             case(GL_UNSIGNED_SHORT): return 16*computeNumComponents(format);
-// 
-//             case(GL_INT):
-//             case(GL_UNSIGNED_INT):
-//             case(GL_FLOAT): return 32*computeNumComponents(format);
-// 
-//             default: return 0;
-//         }    
-
+        case(GL_ALPHA):
+            offset = 0;
+            delta = 1;
+            break;
+        case(GL_LUMINANCE_ALPHA):
+            offset = 1;
+            delta = 2;
+            break;
+        case(GL_RGBA):
+            offset = 3;
+            delta = 4;
+            break;
+        case(GL_BGRA):
+            offset = 3;
+            delta = 4;
+            break;
+        default:
+            return false;
     }
-    else if (getPixelFormat()==GL_RGBA)
+
+    for(int ir=0;ir<r();++ir)
     {
-        return true;
+        for(int it=0;it<t();++it)
+        {
+            const unsigned char* d = data(0,it,ir);
+            switch(_dataType)
+            {
+                case(GL_BYTE):              return _findLowerAlphaValueInRow(s(), (char*)d +offset,            (char)127, delta);
+                case(GL_UNSIGNED_BYTE):     return _findLowerAlphaValueInRow(s(), (unsigned char*)d + offset,  (unsigned char)255, delta);
+                case(GL_SHORT):             return _findLowerAlphaValueInRow(s(), (short*)d + offset,          (short)32767, delta);
+                case(GL_UNSIGNED_SHORT):    return _findLowerAlphaValueInRow(s(), (unsigned short*)d + offset, (unsigned short)65535, delta);
+                case(GL_INT):               return _findLowerAlphaValueInRow(s(), (int*)d + offset,            (int)2147483647, delta);
+                case(GL_UNSIGNED_INT):      return _findLowerAlphaValueInRow(s(), (unsigned int*)d + offset,   4294967295u, delta);
+                case(GL_FLOAT):             return _findLowerAlphaValueInRow(s(), (float*)d + offset,          1.0f, delta);
+            }
+        }
     }
 
     return false;
-
 }
 
 ///////////////////////////////////////////////////////////////////////////////
