@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -16,10 +16,17 @@
 using namespace osg;
 
 ImageStream::ImageStream():
-    _status(PAUSED),
+    _status(INVALID),
     _loopingMode(LOOPING)
 {
     setDataVariance(DYNAMIC); 
+    
+#ifndef __APPLE__
+    // disabled under OSX for time being while we resolve why PBO
+    // doesn't function properly under OSX.
+    setPixelBufferObject(new PixelBufferObject(this));
+#endif
+
 }
 
 ImageStream::ImageStream(const ImageStream& image,const CopyOp& copyop):
