@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -23,7 +23,7 @@ using namespace osg;
 
 Node::Node()
 {
-    _bsphere_computed = false;
+    _boundingSphereComputed = false;
     _nodeMask = 0xffffffff;
     
     _numChildrenRequiringUpdateTraversal = 0;
@@ -38,8 +38,9 @@ Node::Node()
 
 Node::Node(const Node& node,const CopyOp& copyop):
         Object(node,copyop),
-        _bsphere(node._bsphere),
-        _bsphere_computed(node._bsphere_computed),
+        _initialBound(node._initialBound),
+        _boundingSphere(node._boundingSphere),
+        _boundingSphereComputed(node._boundingSphereComputed),
         _name(node._name),
         _parents(), // leave empty as parentList is managed by Group.
         _updateCallback(node._updateCallback),
@@ -86,9 +87,48 @@ void Node::ascend(NodeVisitor& nv)
     std::for_each(_parents.begin(),_parents.end(),NodeAcceptOp(nv));
 }
 
+void Node::setStateSet(osg::StateSet* stateset)
+{
+    // do nothing if nothing changed.
+    if (_stateset==stateset) return;
+    
+    // track whether we need to account for the need to do a update or event traversal.
+    int delta_update = 0;
+    int delta_event = 0;
+
+    // remove this node from the current statesets parent list 
+    if (_stateset.valid())
+    {
+        _stateset->removeParent(this);
+        if (_stateset->requiresUpdateTraversal()) --delta_update;
+        if (_stateset->requiresEventTraversal()) --delta_event;
+    }
+    
+    // set the stateset.
+    _stateset = stateset;
+    
+    // add this node to the new stateset to the parent list.
+    if (_stateset.valid())
+    {
+        _stateset->addParent(this);
+        if (_stateset->requiresUpdateTraversal()) ++delta_update;
+        if (_stateset->requiresEventTraversal()) ++delta_event;
+    }
+    
+    if (delta_update!=0)
+    {
+        setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()+delta_update);
+    }
+
+    if (delta_event!=0)
+    {
+        setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()+delta_event);
+    }
+}
+
 osg::StateSet* Node::getOrCreateStateSet()
 {
-    if (!_stateset) _stateset = new StateSet;
+    if (!_stateset) setStateSet(new StateSet);
     return _stateset.get();
 }
 
@@ -98,7 +138,7 @@ void Node::setUpdateCallback(NodeCallback* nc)
     // if no changes just return.
     if (_updateCallback==nc) return;
     
-    // app callback has been changed, will need to update
+    // updated callback has been changed, will need to update
     // both _updateCallback and possibly the numChildrenRequiringAppTraversal
     // if the number of callbacks changes.
 
@@ -176,7 +216,7 @@ void Node::setEventCallback(NodeCallback* nc)
     // if no changes just return.
     if (_eventCallback==nc) return;
     
-    // app callback has been changed, will need to Event
+    // event callback has been changed, will need to Event
     // both _EventCallback and possibly the numChildrenRequiringAppTraversal
     // if the number of callbacks changes.
 
@@ -365,18 +405,17 @@ bool Node::containsOccluderNodes() const
     return _numChildrenWithOccluderNodes>0 || dynamic_cast<const OccluderNode*>(this);
 }
 
-bool Node::computeBound() const
+BoundingSphere Node::computeBound() const
 {
-    _bsphere.init();
-    return false;
+    return BoundingSphere();
 }
 
 
 void Node::dirtyBound()
 {
-    if (_bsphere_computed)
+    if (_boundingSphereComputed)
     {
-        _bsphere_computed = false;
+        _boundingSphereComputed = false;
 
         // dirty parent bounding sphere's to ensure that all are valid.
         for(ParentList::iterator itr=_parents.begin();
@@ -388,3 +427,11 @@ void Node::dirtyBound()
 
     }
 }
+
+void Node::releaseGLObjects(osg::State* state) const
+{
+    if (_stateset.valid()) _stateset->releaseGLObjects(state);
+}
+
+
+
