@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -24,9 +24,9 @@ OccluderNode::OccluderNode(const OccluderNode& node,const CopyOp& copyop):
 {
 }
 
-bool OccluderNode::computeBound() const
+BoundingSphere OccluderNode::computeBound() const
 {
-    bool result = Group::computeBound();
+    BoundingSphere bsphere(Group::computeBound());
     
     if (getOccluder())
     {
@@ -40,10 +40,8 @@ bool OccluderNode::computeBound() const
         }
         if (bb.valid())
         {
-            _bsphere.expandBy(bb);
-            _bsphere_computed=true;            
-            result = true;
+            bsphere.expandBy(bb);
         }
     }
-    return result;
+    return bsphere;
 }
