@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -28,7 +28,7 @@ State::State()
     _modelView = _identity;
 
     _abortRenderingPtr = false;    
-    _reportGLErrors = false;
+    _checkGLErrors = ONCE_PER_FRAME;
 
     _currentActiveTextureUnit=0;
     _currentClientActiveTextureUnit=0;
@@ -41,6 +41,8 @@ State::State()
 
     _isVertexBufferObjectSupportResolved = false;
     _isVertexBufferObjectSupported = false;
+    
+    _lastAppliedProgramObject = 0;
 }
 
 State::~State()
@@ -49,6 +51,7 @@ State::~State()
 
 void State::reset()
 {
+
 /*
     for(ModeMap::iterator mitr=_modeMap.begin();
         mitr!=_modeMap.end();
@@ -103,13 +106,37 @@ void State::reset()
         }
     }
 
-    _drawStateStack.clear();
+    _stateStateStack.clear();
     
     _modelView = _identity;
     _projection = _identity;
     
     dirtyAllVertexArrays();
     setActiveTextureUnit(0);
+    
+    _lastAppliedProgramObject = 0;
+
+    for(AppliedProgramObjectSet::iterator apitr=_appliedProgramObjectSet.begin();
+        apitr!=_appliedProgramObjectSet.end();
+        ++apitr)
+    {
+        (*apitr)->resetAppliedUniforms();
+    }
+    
+    _appliedProgramObjectSet.clear();
+    
+    
+    // what about uniforms??? need to clear them too...
+    // go through all active Unfirom's, setting to change to force update,
+    // the idea is to leave only the global defaults left.
+    for(UniformMap::iterator uitr=_uniformMap.begin();
+        uitr!=_uniformMap.end();
+        ++uitr)
+    {
+        UniformStack& us = uitr->second;
+        us.uniformVec.clear();
+    }
+
 }
 
 void State::setInitialViewMatrix(const osg::RefMatrix* matrix)
@@ -122,7 +149,7 @@ void State::setInitialViewMatrix(const osg::RefMatrix* matrix)
 
 void State::pushStateSet(const StateSet* dstate)
 {
-    _drawStateStack.push_back(dstate);
+    _stateStateStack.push_back(dstate);
     if (dstate)
     {
 
@@ -145,22 +172,25 @@ void State::pushStateSet(const StateSet* dstate)
             pushAttributeList(getOrCreateTextureAttributeMap(unit),ds_textureAttributeList[unit]);
         }
 
+        pushUniformList(_uniformMap,dstate->getUniformList());
     }
 }
 
 void State::popAllStateSets()
 {
-    while (!_drawStateStack.empty()) popStateSet();
+    while (!_stateStateStack.empty()) popStateSet();
     
     applyProjectionMatrix(0);
     applyModelViewMatrix(0);
+    
+    _lastAppliedProgramObject = 0;
 }
 
 void State::popStateSet()
 {
-    if (_drawStateStack.empty()) return;
+    if (_stateStateStack.empty()) return;
     
-    const StateSet* dstate = _drawStateStack.back();
+    const StateSet* dstate = _stateStateStack.back();
 
     if (dstate)
     {
@@ -183,10 +213,13 @@ void State::popStateSet()
         {
             popAttributeList(getOrCreateTextureAttributeMap(unit),ds_textureAttributeList[unit]);
         }
+
+        popUniformList(_uniformMap,dstate->getUniformList());
+
     }
     
     // remove the top draw state from the stack.
-    _drawStateStack.pop_back();
+    _stateStateStack.pop_back();
 }
 
 void State::captureCurrentState(StateSet& stateset) const
@@ -229,7 +262,7 @@ void State::captureCurrentState(StateSet& stateset) const
 
 void State::apply(const StateSet* dstate)
 {
-    if (_reportGLErrors) checkGLErrors("start of State::apply(StateSet*)");
+    if (_checkGLErrors==ONCE_PER_ATTRIBUTE) checkGLErrors("start of State::apply(StateSet*)");
 
     // equivilant to:
     //pushStateSet(dstate);
@@ -261,7 +294,36 @@ void State::apply(const StateSet* dstate)
                 else if (unit<_textureAttributeMapList.size()) applyAttributeMap(_textureAttributeMapList[unit]);
             }
         }
-    
+        
+#if 1        
+        applyUniformList(_uniformMap,dstate->getUniformList());
+#else                
+        if (_lastAppliedProgramObject)
+        {
+            for(StateSetStack::iterator sitr=_stateStateStack.begin();
+                sitr!=_stateStateStack.end();
+                ++sitr)
+            {
+                const StateSet* stateset = *sitr;
+                const StateSet::UniformList& uniformList = stateset->getUniformList();
+                for(StateSet::UniformList::const_iterator itr=uniformList.begin();
+                    itr!=uniformList.end();
+                    ++itr)
+                {
+                    _lastAppliedProgramObject->apply(*(itr->second.first));
+                }
+            }
+
+            const StateSet::UniformList& uniformList = dstate->getUniformList();
+            for(StateSet::UniformList::const_iterator itr=uniformList.begin();
+                itr!=uniformList.end();
+                ++itr)
+            {
+                _lastAppliedProgramObject->apply(*(itr->second.first));
+            }
+        }
+#endif
+
     }
     else
     {
@@ -269,13 +331,13 @@ void State::apply(const StateSet* dstate)
         apply();
     }
 
-    if (_reportGLErrors) checkGLErrors("end of State::apply(StateSet*)");
+    if (_checkGLErrors==ONCE_PER_ATTRIBUTE) checkGLErrors("end of State::apply(StateSet*)");
 }
 
 void State::apply()
 {
 
-    if (_reportGLErrors) checkGLErrors("start of State::apply()");
+    if (_checkGLErrors==ONCE_PER_ATTRIBUTE) checkGLErrors("start of State::apply()");
 
     // go through all active OpenGL modes, enabling/disable where
     // appropriate.
@@ -295,7 +357,29 @@ void State::apply()
         }
     }
 
-    if (_reportGLErrors) checkGLErrors("end of State::apply()");
+#if 1        
+    applyUniformMap(_uniformMap);
+#else        
+    if (_lastAppliedProgramObject && !_stateStateStack.empty())
+    {
+        for(StateSetStack::iterator sitr=_stateStateStack.begin();
+            sitr!=_stateStateStack.end();
+            ++sitr)
+        {
+            const StateSet* stateset = *sitr;
+            const StateSet::UniformList& uniformList = stateset->getUniformList();
+            for(StateSet::UniformList::const_iterator itr=uniformList.begin();
+                itr!=uniformList.end();
+                ++itr)
+            {
+                _lastAppliedProgramObject->apply(*(itr->second.first));
+            }
+        }
+    }
+#endif
+
+
+    if (_checkGLErrors==ONCE_PER_ATTRIBUTE) checkGLErrors("end of State::apply()");
 }
 
 void State::haveAppliedMode(StateAttribute::GLMode mode,StateAttribute::GLModeValue value)
@@ -718,21 +802,21 @@ void State::disableVertexAttribPointersAboveAndIncluding( unsigned int index )
 bool State::computeSecondaryColorSupported() const
 {
     _isSecondaryColorSupportResolved = true;
-    _isSecondaryColorSupported = osg::isGLExtensionSupported("GL_EXT_secondary_color");
+    _isSecondaryColorSupported = osg::isGLExtensionSupported(_contextID,"GL_EXT_secondary_color");
     return _isSecondaryColorSupported;
 }
 
 bool State::computeFogCoordSupported() const
 {
     _isFogCoordSupportResolved = true;
-    _isFogCoordSupported = osg::isGLExtensionSupported("GL_EXT_fog_coord");
+    _isFogCoordSupported = osg::isGLExtensionSupported(_contextID,"GL_EXT_fog_coord");
     return _isFogCoordSupported;
 }
 
 bool State::computeVertexBufferObjectSupported() const
 {
     _isVertexBufferObjectSupportResolved = true;
-    _isVertexBufferObjectSupported = osg::isGLExtensionSupported("GL_ARB_vertex_buffer_object");
+    _isVertexBufferObjectSupported = osg::isGLExtensionSupported(_contextID,"GL_ARB_vertex_buffer_object");
     return _isVertexBufferObjectSupported;
 }
 
