@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -23,12 +23,14 @@
 #include <osg/PolygonMode>
 #include <osg/BlendFunc>
 #include <osg/Depth>
+#include <osg/Drawable>
+#include <osg/Node>
 
 #include <osg/TextureCubeMap>
 #include <osg/TextureRectangle>
 
 #include <set>
-
+#include <algorithm>
 
 using namespace osg;
 
@@ -46,7 +48,7 @@ class TextureGLModeSet
             _textureModeSet.insert(GL_TEXTURE_3D);
 
             _textureModeSet.insert(GL_TEXTURE_CUBE_MAP);
-	    _textureModeSet.insert(GL_TEXTURE_RECTANGLE_NV);
+        _textureModeSet.insert(GL_TEXTURE_RECTANGLE_NV);
 
             _textureModeSet.insert(GL_TEXTURE_GEN_Q);
             _textureModeSet.insert(GL_TEXTURE_GEN_R);
@@ -74,6 +76,9 @@ StateSet::StateSet()
 
     _renderingHint = DEFAULT_BIN;
     
+    _numChildrenRequiringUpdateTraversal = 0;
+    _numChildrenRequiringEventTraversal = 0;
+
     setRenderBinToInherit();
 }
 
@@ -88,7 +93,11 @@ StateSet::StateSet(const StateSet& rhs,const CopyOp& copyop):Object(rhs,copyop)
         const StateAttribute::TypeMemberPair& typemember = itr->first;
         const RefAttributePair& rap = itr->second;
         StateAttribute* attr = copyop(rap.first.get());
-        if (attr) _attributeList[typemember]=RefAttributePair(attr,rap.second);
+        if (attr)
+        {
+            _attributeList[typemember]=RefAttributePair(attr,rap.second);
+            attr->addParent(this);
+        }
     }
     
     // copy texture related modes.
@@ -110,23 +119,59 @@ StateSet::StateSet(const StateSet& rhs,const CopyOp& copyop):Object(rhs,copyop)
             const StateAttribute::TypeMemberPair& typemember = itr->first;
             const RefAttributePair& rap = itr->second;
             StateAttribute* attr = copyop(rap.first.get());
-            if (attr) lhs_attributeList[typemember]=RefAttributePair(attr,rap.second);
+            if (attr)
+            {
+                lhs_attributeList[typemember]=RefAttributePair(attr,rap.second);
+                attr->addParent(this);
+            }
+        }
+    }
+
+    // copy uniform values
+    for(UniformList::const_iterator rhs_uitr = rhs._uniformList.begin();
+        rhs_uitr != rhs._uniformList.end();
+        ++rhs_uitr)
+    {
+    const std::string& name = rhs_uitr->first;
+    const RefUniformPair& rup = rhs_uitr->second;
+    Uniform* uni = copyop(rup.first.get());
+    if (uni)
+        {
+            _uniformList[name] = RefUniformPair(uni, rup.second);
+            uni->addParent(this);
         }
     }
-    
     
     _renderingHint = rhs._renderingHint;
 
     _binMode = rhs._binMode;
     _binNum = rhs._binNum;
     _binName = rhs._binName;
+
+    _updateCallback = rhs._updateCallback;
+    _numChildrenRequiringUpdateTraversal = rhs._numChildrenRequiringUpdateTraversal;
+
+    _eventCallback = rhs._eventCallback;
+    _numChildrenRequiringEventTraversal = rhs._numChildrenRequiringEventTraversal;
+
 }
 
 StateSet::~StateSet()
 {
-    // note, all attached state attributes will be automatically
-    // unreferenced by ref_ptr<> and therefore there is no need to
-    // delete the memory manually.
+    clear();
+}
+
+void StateSet::addParent(osg::Object* object)
+{
+    // osg::notify(osg::INFO)<<"Adding parent"<<std::endl;
+
+    _parents.push_back(object);
+}
+
+void StateSet::removeParent(osg::Object* object)
+{
+    ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),object);
+    if (pitr!=_parents.end()) _parents.erase(pitr);
 }
 
 int StateSet::compare(const StateSet& rhs,bool compareAttributeContents) const
@@ -281,6 +326,25 @@ int StateSet::compare(const StateSet& rhs,bool compareAttributeContents) const
     }
     else if (rhs_mode_itr == rhs._modeList.end()) return 1;
 
+
+    // check uniforms.
+    UniformList::const_iterator lhs_uniform_itr = _uniformList.begin();
+    UniformList::const_iterator rhs_uniform_itr = rhs._uniformList.begin();
+    while (lhs_uniform_itr!=_uniformList.end() && rhs_uniform_itr!=rhs._uniformList.end())
+    {
+        if      (lhs_uniform_itr->first<rhs_uniform_itr->first) return -1;
+        else if (rhs_uniform_itr->first<lhs_uniform_itr->first) return 1;
+        if      (lhs_uniform_itr->second<rhs_uniform_itr->second) return -1;
+        else if (rhs_uniform_itr->second<lhs_uniform_itr->second) return 1;
+        ++lhs_uniform_itr;
+        ++rhs_uniform_itr;
+    }
+    if (lhs_uniform_itr==_uniformList.end())
+    {
+        if (rhs_uniform_itr!=rhs._uniformList.end()) return -1;
+    }
+    else if (rhs_uniform_itr == rhs._uniformList.end()) return 1;  
+
     return 0;
 }
 
@@ -374,12 +438,43 @@ void StateSet::clear()
 
     setRenderBinToInherit();
 
+    // remove self from as attributes parent
+    for(AttributeList::iterator itr=_attributeList.begin();
+        itr!=_attributeList.end();
+        ++itr)
+    {
+        itr->second.first->removeParent(this);
+    }
+    
     _modeList.clear();
     _attributeList.clear();
     
+    
     _textureModeList.clear();
     _textureAttributeList.clear();
-    
+
+    // remove self from as texture attributes parent
+    for(unsigned int i=0;i<_textureAttributeList.size();++i)
+    {
+        AttributeList& attributeList = _textureAttributeList[i];
+        for(AttributeList::iterator itr=attributeList.begin();
+            itr!=attributeList.end();
+            ++itr)
+        {
+            itr->second.first->removeParent(this);
+        }
+    }
+
+
+    // remove self from uniforms parent
+    for(UniformList::iterator uitr = _uniformList.begin();
+        uitr != _uniformList.end();
+        ++uitr)
+    {
+        uitr->second.first->removeParent(this);
+    }
+
+    _uniformList.clear();
 }
 
 void StateSet::merge(const StateSet& rhs)
@@ -420,13 +515,29 @@ void StateSet::merge(const StateSet& rhs)
             {
                 // override isn't on in rhs, so overrite it with incomming
                 // value.
-                lhs_aitr->second = rhs_aitr->second;
+                if (lhs_aitr->second.first!=rhs_aitr->second.first)
+                {
+                    // new attribute so need to remove self from outgoing attribute
+                    lhs_aitr->second.first->removeParent(this);
+
+                    // override isn't on in rhs, so overrite it with incomming
+                    // value.
+                    lhs_aitr->second = rhs_aitr->second;
+                    lhs_aitr->second.first->addParent(this);
+
+                }
+                else
+                {
+                    // same attribute but with override to set.
+                    lhs_aitr->second = rhs_aitr->second;
+                }
+
             }
         }
         else
         {
-            // entry doesn't exist so insert it.
-            _attributeList.insert(*rhs_aitr);
+            // entry doesn't exist so insert it, and then tell it about self by adding self as parent.
+            _attributeList.insert(*rhs_aitr).first->second.first->addParent(this);
         }
     }
 
@@ -479,17 +590,61 @@ void StateSet::merge(const StateSet& rhs)
                 {
                     // override isn't on in rhs, so overrite it with incomming
                     // value.
-                    lhs_aitr->second = rhs_aitr->second;
+                    
+                    if (lhs_aitr->second.first!=rhs_aitr->second.first)
+                    {
+                        lhs_aitr->second.first->removeParent(this);
+                        lhs_aitr->second = rhs_aitr->second;
+                        lhs_aitr->second.first->addParent(this);
+                    }
+                    else
+                    {
+                        lhs_aitr->second = rhs_aitr->second;
+                    }
                 }
             }
             else
             {
-                // entry doesn't exist so insert it.
-                lhs_attributeList.insert(*rhs_aitr);
+                // entry doesn't exist so insert it and add self as parent
+                lhs_attributeList.insert(*rhs_aitr).first->second.first->addParent(this);
             }
         }
     }
 
+    // merge the uniforms of rhs into this, 
+    // this overrides rhs if OVERRIDE defined in this.
+    for(UniformList::const_iterator rhs_uitr = rhs._uniformList.begin();
+        rhs_uitr != rhs._uniformList.end();
+        ++rhs_uitr)
+    {
+        UniformList::iterator lhs_uitr = _uniformList.find(rhs_uitr->first);
+        if (lhs_uitr!=_uniformList.end())
+        {
+            if (!(lhs_uitr->second.second & StateAttribute::OVERRIDE)) 
+            {
+                // override isn't on in rhs, so overrite it with incomming
+                // value.
+
+                if (lhs_uitr->second.first!=rhs_uitr->second.first)
+                {
+                    lhs_uitr->second.first->removeParent(this);
+                    lhs_uitr->second = rhs_uitr->second;
+                    lhs_uitr->second.first->addParent(this);
+                }
+                else
+                {
+                    lhs_uitr->second = rhs_uitr->second;
+                }
+
+            }
+        }
+        else
+        {
+            // entry doesn't exist so insert it and add self as parent
+            _uniformList.insert(*rhs_uitr).first->second.first->addParent(this);
+        }
+    }
+
     // need to merge rendering hints
     // but will need to think how best to do this first
     // RO, Nov. 2001.
@@ -597,6 +752,17 @@ void StateSet::removeAttribute(StateAttribute::Type type, unsigned int member)
     AttributeList::iterator itr = _attributeList.find(StateAttribute::TypeMemberPair(type,member));
     if (itr!=_attributeList.end())
     {
+        if (itr->second.first->getUpdateCallback()) 
+        {
+            setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()-1);
+        }
+
+        if (itr->second.first->getEventCallback()) 
+        {
+            setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()-1);
+        }
+
+        itr->second.first->removeParent(this);
         setAssociatedModes(itr->second.first.get(),StateAttribute::INHERIT);
         _attributeList.erase(itr);
     }
@@ -611,6 +777,18 @@ void StateSet::removeAttribute(StateAttribute* attribute)
     {
         if (itr->second.first != attribute) return;
         
+        if (itr->second.first->getUpdateCallback()) 
+        {
+            setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()-1);
+        }
+
+        if (itr->second.first->getEventCallback()) 
+        {
+            setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()-1);
+        }
+
+        itr->second.first->removeParent(this);
+
         setAssociatedModes(itr->second.first.get(),StateAttribute::INHERIT);
         _attributeList.erase(itr);
     }
@@ -631,6 +809,153 @@ const StateSet::RefAttributePair* StateSet::getAttributePair(StateAttribute::Typ
     return getAttributePair(_attributeList,type,member);
 }
 
+void StateSet::addUniform(Uniform* uniform, StateAttribute::OverrideValue value)
+{
+    if (uniform)
+    {
+        int delta_update = 0;
+        int delta_event = 0;
+        
+        UniformList::iterator itr=_uniformList.find(uniform->getName());
+        if (itr==_uniformList.end())
+        {
+            // new entry.
+            RefUniformPair& up = _uniformList[uniform->getName()];
+            up.first = uniform;
+            up.second = value&(StateAttribute::OVERRIDE|StateAttribute::PROTECTED);
+            
+            uniform->addParent(this);
+            
+            if (uniform->getUpdateCallback()) 
+            {
+                delta_update = 1;
+            }
+
+            if (uniform->getEventCallback()) 
+            {
+                delta_event = 1;
+            }
+        }
+        else
+        {
+            if (itr->second.first==uniform)
+            {
+                // chaning just override
+                itr->second.second = value&(StateAttribute::OVERRIDE|StateAttribute::PROTECTED);
+            }
+            else
+            {
+                itr->second.first->removeParent(this);
+                if (itr->second.first->getUpdateCallback()) --delta_update;
+                if (itr->second.first->getEventCallback()) --delta_event;
+                
+                uniform->addParent(this);
+                itr->second.first = uniform;
+                if (itr->second.first->getUpdateCallback()) ++delta_update;
+                if (itr->second.first->getEventCallback()) ++delta_event;
+
+                itr->second.second = value&(StateAttribute::OVERRIDE|StateAttribute::PROTECTED);
+            }
+        }
+
+        if (delta_update!=0) 
+        {
+            setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()+delta_update);
+        }
+
+        if (delta_event!=0) 
+        {
+            setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()+delta_event);
+        }
+
+    }
+}
+
+void StateSet::removeUniform(const std::string& name)
+{
+    UniformList::iterator itr = _uniformList.find(name);
+    if (itr!=_uniformList.end())
+    {
+        if (itr->second.first->getUpdateCallback()) 
+        {
+            setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()-1);
+        }
+
+        if (itr->second.first->getEventCallback()) 
+        {
+            setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()-1);
+        }
+
+        itr->second.first->removeParent(this);
+
+        _uniformList.erase(itr);
+    }
+}
+
+void StateSet::removeUniform(Uniform* uniform)
+{
+    if (!uniform) return;
+    
+    UniformList::iterator itr = _uniformList.find(uniform->getName());
+    if (itr!=_uniformList.end())
+    {
+        if (itr->second.first != uniform) return;
+
+        if (itr->second.first->getUpdateCallback()) 
+        {
+            setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()-1);
+        }
+
+        if (itr->second.first->getEventCallback()) 
+        {
+            setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()-1);
+        }
+
+        itr->second.first->removeParent(this);
+        _uniformList.erase(itr);
+    }
+}
+
+Uniform* StateSet::getUniform(const std::string& name)
+{
+    UniformList::iterator itr = _uniformList.find(name);
+    if (itr!=_uniformList.end()) return itr->second.first.get();
+    else return 0;
+}
+
+Uniform* StateSet::getOrCreateUniform(const std::string& name, Uniform::Type type)
+{
+    // for look for an appropriate uniform.
+    UniformList::iterator itr = _uniformList.find(name);
+    if (itr!=_uniformList.end() && 
+        itr->second.first->getType()==type)
+    {
+        return itr->second.first.get();
+    }
+
+    // no uniform found matching name so create it..
+    
+    Uniform* uniform = new Uniform(type,name);
+    addUniform(uniform);
+    
+    return uniform;
+}
+
+
+const Uniform* StateSet::getUniform(const std::string& name) const
+{
+    UniformList::const_iterator itr = _uniformList.find(name);
+    if (itr!=_uniformList.end()) return itr->second.first.get();
+    else return 0;
+}
+
+const StateSet::RefUniformPair* StateSet::getUniformPair(const std::string& name) const
+{
+    UniformList::const_iterator itr = _uniformList.find(name);
+    if (itr!=_uniformList.end()) return &(itr->second);
+    else return 0;
+}
+
 void StateSet::setTextureMode(unsigned int unit,StateAttribute::GLMode mode, StateAttribute::GLModeValue value)
 {
     if (s_textureGLModeSet.isTextureMode(mode))
@@ -740,24 +1065,49 @@ void StateSet::removeTextureAttribute(unsigned int unit,StateAttribute::Type typ
         {
             setAssociatedTextureModes(unit,itr->second.first.get(),StateAttribute::INHERIT);
         }
+
+        if (itr->second.first->getUpdateCallback()) 
+        {
+            setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()-1);
+        }
+
+        if (itr->second.first->getEventCallback()) 
+        {
+            setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()-1);
+        }
+
+
+        itr->second.first->removeParent(this);
         attributeList.erase(itr);
     }
 }
 
 void StateSet::removeTextureAttribute(unsigned int unit, StateAttribute* attribute)
 {
-	if (!attribute) return;
-	if (unit>=_textureAttributeList.size()) return;
+    if (!attribute) return;
+    if (unit>=_textureAttributeList.size()) return;
 
-	AttributeList& attributeList = _textureAttributeList[unit];
-	AttributeList::iterator itr = attributeList.find(attribute->getTypeMemberPair());
-	if (itr!=attributeList.end())
-	{
-		if (itr->second.first != attribute) return;
+    AttributeList& attributeList = _textureAttributeList[unit];
+    AttributeList::iterator itr = attributeList.find(attribute->getTypeMemberPair());
+    if (itr!=attributeList.end())
+    {
+        if (itr->second.first != attribute) return;
 
-		setAssociatedTextureModes(unit,itr->second.first.get(),StateAttribute::INHERIT);
-		attributeList.erase(itr);
-	}
+        setAssociatedTextureModes(unit,itr->second.first.get(),StateAttribute::INHERIT);
+
+        if (itr->second.first->getUpdateCallback()) 
+        {
+            setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()-1);
+        }
+
+        if (itr->second.first->getEventCallback()) 
+        {
+            setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()-1);
+        }
+
+        itr->second.first->removeParent(this);
+        attributeList.erase(itr);
+    }
 }
 
 StateAttribute* StateSet::getTextureAttribute(unsigned int unit,StateAttribute::Type type)
@@ -936,7 +1286,57 @@ void StateSet::setAttribute(AttributeList& attributeList,StateAttribute *attribu
 {
     if (attribute)
     {
-        attributeList[attribute->getTypeMemberPair()] = RefAttributePair(attribute,value&(StateAttribute::OVERRIDE|StateAttribute::PROTECTED));
+        int delta_update = 0;
+        int delta_event = 0;
+        
+        AttributeList::iterator itr=attributeList.find(attribute->getTypeMemberPair());
+        if (itr==attributeList.end())
+        {
+            // new entry.
+            attributeList[attribute->getTypeMemberPair()] = RefAttributePair(attribute,value&(StateAttribute::OVERRIDE|StateAttribute::PROTECTED));
+            attribute->addParent(this);
+            
+            if (attribute->getUpdateCallback()) 
+            {
+                delta_update = 1;
+            }
+
+            if (attribute->getEventCallback()) 
+            {
+                delta_event = 1;
+            }
+        }
+        else
+        {
+            if (itr->second.first==attribute)
+            {
+                // changing just override
+                itr->second.second = value&(StateAttribute::OVERRIDE|StateAttribute::PROTECTED);
+            }
+            else
+            {
+                itr->second.first->removeParent(this);
+                if (itr->second.first->getUpdateCallback()) --delta_update;
+                if (itr->second.first->getEventCallback()) --delta_event;
+
+                attribute->addParent(this);
+                itr->second.first = attribute;
+                if (itr->second.first->getUpdateCallback()) ++delta_update;
+                if (itr->second.first->getEventCallback()) ++delta_event;
+
+               itr->second.second = value&(StateAttribute::OVERRIDE|StateAttribute::PROTECTED);
+            }
+        }
+
+        if (delta_update!=0) 
+        {
+            setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()+delta_update);
+        }
+
+        if (delta_event!=0) 
+        {
+            setNumChildrenRequiringEventTraversal(getNumChildrenRequiringEventTraversal()+delta_event);
+        }
     }
 }
 
@@ -977,3 +1377,248 @@ const StateSet::RefAttributePair* StateSet::getAttributePair(const AttributeList
 
 
 
+void StateSet::setUpdateCallback(Callback* ac)
+{
+    //osg::notify(osg::INFO)<<"Setting StateSet callbacks"<<std::endl;
+
+    if (_updateCallback==ac) return;
+    
+    int delta = 0;
+    if (_updateCallback.valid()) --delta;
+    if (ac) ++delta;
+
+    _updateCallback = ac;
+    
+    if (delta!=0 && _numChildrenRequiringUpdateTraversal==0)
+    {
+        //osg::notify(osg::INFO)<<"Going to set StateSet parents"<<std::endl;
+
+        for(ParentList::iterator itr=_parents.begin();
+            itr!=_parents.end();
+            ++itr)
+        {
+            //osg::notify(osg::INFO)<<"Setting StateSet parent"<<std::endl;
+
+            osg::Drawable* drawable = dynamic_cast<osg::Drawable*>(*itr);
+            if (drawable) 
+            {
+                //drawable->setNumChildrenRequiringUpdateTraversal(drawable->getNumChildrenRequiringUpdateTraversal()+delta);
+            }
+            else 
+            {
+                osg::Node* node = dynamic_cast<osg::Node*>(*itr);
+                if (node) 
+                {
+                    node->setNumChildrenRequiringUpdateTraversal(node->getNumChildrenRequiringUpdateTraversal()+delta);
+                }
+            }
+        }
+    }
+}
+
+void StateSet::runUpdateCallbacks(osg::NodeVisitor* nv)
+{
+    //osg::notify(osg::INFO)<<"Running StateSet callbacks"<<std::endl;
+
+    if (_updateCallback.valid()) (*_updateCallback)(this,nv);
+
+    if (_numChildrenRequiringUpdateTraversal!=0)
+    {
+        // run attribute callbacks
+        for(AttributeList::iterator itr=_attributeList.begin();
+            itr!=_attributeList.end();
+            ++itr)
+        {
+            StateAttribute::Callback* callback = itr->second.first->getUpdateCallback();
+            if (callback) (*callback)(itr->second.first.get(),nv);
+        }
+
+
+        // run texture attribute callbacks.
+        for(unsigned int i=0;i<_textureAttributeList.size();++i)
+        {
+            AttributeList& attributeList = _textureAttributeList[i];
+            for(AttributeList::iterator itr=attributeList.begin();
+                itr!=attributeList.end();
+                ++itr)
+            {
+                StateAttribute::Callback* callback = itr->second.first->getUpdateCallback();
+                if (callback) (*callback)(itr->second.first.get(),nv);
+            }
+        }
+
+
+        // run uniform callbacks.
+        for(UniformList::iterator uitr = _uniformList.begin();
+            uitr != _uniformList.end();
+            ++uitr)
+        {
+            Uniform::Callback* callback = uitr->second.first->getUpdateCallback();
+            if (callback) (*callback)(uitr->second.first.get(),nv);
+        }
+    }
+}
+
+void StateSet::setEventCallback(Callback* ac)
+{
+    if (_eventCallback==ac) return;
+    
+    int delta = 0;
+    if (_eventCallback.valid()) --delta;
+    if (ac) ++delta;
+
+    _eventCallback = ac;
+    
+    if (delta!=0 && _numChildrenRequiringEventTraversal==0)
+    {
+        for(ParentList::iterator itr=_parents.begin();
+            itr!=_parents.end();
+            ++itr)
+        {
+            osg::Drawable* drawable = dynamic_cast<osg::Drawable*>(*itr);
+            if (drawable) 
+            {
+                //drawable->setNumChildrenRequiringUpdateTraversal(drawable->getNumChildrenRequiringUpdateTraversal()+delta);
+            }
+            else 
+            {
+                osg::Node* node = dynamic_cast<osg::Node*>(*itr);
+                if (node) 
+                {
+                    node->setNumChildrenRequiringEventTraversal(node->getNumChildrenRequiringEventTraversal()+delta);
+                }
+            }
+        }
+    }
+}
+
+void StateSet::runEventCallbacks(osg::NodeVisitor* nv)
+{
+    if (_eventCallback.valid()) (*_eventCallback)(this,nv);
+
+    if (_numChildrenRequiringEventTraversal!=0)
+    {
+        // run attribute callbacks
+        for(AttributeList::iterator itr=_attributeList.begin();
+            itr!=_attributeList.end();
+            ++itr)
+        {
+            StateAttribute::Callback* callback = itr->second.first->getEventCallback();
+            if (callback) (*callback)(itr->second.first.get(),nv);
+        }
+
+
+        // run texture attribute callbacks.
+        for(unsigned int i=0;i<_textureAttributeList.size();++i)
+        {
+            AttributeList& attributeList = _textureAttributeList[i];
+            for(AttributeList::iterator itr=attributeList.begin();
+                itr!=attributeList.end();
+                ++itr)
+            {
+                StateAttribute::Callback* callback = itr->second.first->getEventCallback();
+                if (callback) (*callback)(itr->second.first.get(),nv);
+            }
+        }
+
+
+        // run uniform callbacks.
+        for(UniformList::iterator uitr = _uniformList.begin();
+            uitr != _uniformList.end();
+            ++uitr)
+        {
+            Uniform::Callback* callback = uitr->second.first->getEventCallback();
+            if (callback) (*callback)(uitr->second.first.get(),nv);
+        }
+    }
+}
+
+void StateSet::setNumChildrenRequiringUpdateTraversal(unsigned int num)
+{
+    // if no changes just return.
+    if (_numChildrenRequiringUpdateTraversal==num) return;
+
+    // note, if _updateCallback is set then the
+    // parents won't be affected by any changes to
+    // _numChildrenRequiringUpdateTraversal so no need to inform them.
+    if (!_updateCallback && !_parents.empty())
+    {
+    
+        // need to pass on changes to parents.        
+        int delta = 0;
+        if (_numChildrenRequiringUpdateTraversal>0) --delta;
+        if (num>0) ++delta;
+        if (delta!=0)
+        {
+            // the number of callbacks has changed, need to pass this
+            // on to parents so they know whether app traversal is
+            // reqired on this subgraph.
+            for(ParentList::iterator itr =_parents.begin();
+                itr != _parents.end();
+                ++itr)
+            {    
+                osg::Drawable* drawable = dynamic_cast<osg::Drawable*>(*itr);
+                if (drawable) 
+                {
+                    drawable->setNumChildrenRequiringUpdateTraversal(drawable->getNumChildrenRequiringUpdateTraversal()+delta);
+                }
+                else 
+                {
+                    osg::Node* node = dynamic_cast<osg::Node*>(*itr);
+                    if (node) 
+                    {
+                        node->setNumChildrenRequiringUpdateTraversal(node->getNumChildrenRequiringUpdateTraversal()+delta);
+                    }
+                }
+            }
+        }
+    }
+    
+    // finally update this objects value.
+    _numChildrenRequiringUpdateTraversal=num;
+}
+
+void StateSet::setNumChildrenRequiringEventTraversal(unsigned int num)
+{
+    // if no changes just return.
+    if (_numChildrenRequiringEventTraversal==num) return;
+
+    // note, if _EventCallback is set then the
+    // parents won't be affected by any changes to
+    // _numChildrenRequiringEventTraversal so no need to inform them.
+    if (!_eventCallback && !_parents.empty())
+    {
+    
+        // need to pass on changes to parents.        
+        int delta = 0;
+        if (_numChildrenRequiringEventTraversal>0) --delta;
+        if (num>0) ++delta;
+        if (delta!=0)
+        {
+            // the number of callbacks has changed, need to pass this
+            // on to parents so they know whether app traversal is
+            // reqired on this subgraph.
+            for(ParentList::iterator itr =_parents.begin();
+                itr != _parents.end();
+                ++itr)
+            {    
+                osg::Drawable* drawable = dynamic_cast<osg::Drawable*>(*itr);
+                if (drawable) 
+                {
+                    drawable->setNumChildrenRequiringEventTraversal(drawable->getNumChildrenRequiringEventTraversal()+delta);
+                }
+                else 
+                {
+                    osg::Node* node = dynamic_cast<osg::Node*>(*itr);
+                    if (node) 
+                    {
+                        node->setNumChildrenRequiringEventTraversal(node->getNumChildrenRequiringEventTraversal()+delta);
+                    }
+                }
+            }
+        }
+    }
+    
+    // finally Event this objects value.
+    _numChildrenRequiringEventTraversal=num;
+}
