@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -12,6 +12,7 @@
 */
 #include <osg/GLExtensions>
 #include <osg/TexEnvFilter>
+#include <osg/State>
 
 using namespace osg;
 
@@ -25,11 +26,11 @@ TexEnvFilter::~TexEnvFilter()
 {
 }
 
-void TexEnvFilter::apply(State&) const
+void TexEnvFilter::apply(State& state) const
 {
     // note from RO, need to adapt to do testing per graphics context.
     static float glVersion = atof( (const char *)glGetString( GL_VERSION ) );
-    static bool s_isTexLodBias = glVersion>=1.4 || isGLExtensionSupported("GL_EXT_texture_lod_bias");
+    static bool s_isTexLodBias = glVersion>=1.4 || isGLExtensionSupported(state.getContextID(),"GL_EXT_texture_lod_bias");
 
     if (s_isTexLodBias)
     {
