@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -40,6 +40,8 @@ using namespace osg;
 #define GL_STORAGE_SHARED_APPLE           0x85BF
 #endif
 
+// #define DO_TIMING
+
 unsigned int Texture::s_numberTextureReusedLastInLastFrame = 0;
 unsigned int Texture::s_numberNewTextureInLastFrame = 0;
 unsigned int Texture::s_numberDeletedTextureInLastFrame = 0;
@@ -55,7 +57,7 @@ unsigned int Texture::getMinimumNumberOfTextureObjectsToRetainInCache()
     return s_minimumNumberOfTextureObjectsToRetainInCache;
 }
 
-class SG_EXPORT TextureObjectManager : public osg::Referenced
+class TextureObjectManager : public osg::Referenced
 {
 public:
 
@@ -335,6 +337,7 @@ Texture::Texture():
             _useHardwareMipMapGeneration(true),
             _unrefImageDataAfterApply(false),
             _clientStorageHint(false),
+            _resizeNonPowerOfTwoHint(false),
             _borderColor(0.0, 0.0, 0.0, 0.0),
             _borderWidth(0),
             _internalFormatMode(USE_IMAGE_DATA_FORMAT),
@@ -357,6 +360,7 @@ Texture::Texture(const Texture& text,const CopyOp& copyop):
             _useHardwareMipMapGeneration(text._useHardwareMipMapGeneration),
             _unrefImageDataAfterApply(text._unrefImageDataAfterApply),
             _clientStorageHint(text._clientStorageHint),
+            _resizeNonPowerOfTwoHint(text._resizeNonPowerOfTwoHint),
             _borderColor(text._borderColor),
             _borderWidth(text._borderWidth),
             _internalFormatMode(text._internalFormatMode),
@@ -398,7 +402,8 @@ int Texture::compareTexture(const Texture& rhs) const
 
     COMPARE_StateAttribute_Parameter(_unrefImageDataAfterApply)
     COMPARE_StateAttribute_Parameter(_clientStorageHint)
-    
+    COMPARE_StateAttribute_Parameter(_resizeNonPowerOfTwoHint)
+
     return 0;
 }
 
@@ -706,13 +711,23 @@ void Texture::computeRequiredTextureDimensions(State& state, const osg::Image& i
     const unsigned int contextID = state.getContextID();
     const Extensions* extensions = getExtensions(contextID,true);
 
-    int width = Image::computeNearestPowerOfTwo(image.s()-2*_borderWidth)+2*_borderWidth;
-    int height = Image::computeNearestPowerOfTwo(image.t()-2*_borderWidth)+2*_borderWidth;
+    int width,height;
+
+    if( !_resizeNonPowerOfTwoHint && extensions->isNonPowerOfTwoTextureSupported(_min_filter) )
+    {
+        width = image.s();
+        height = image.t();
+    }
+    else
+    {
+        width = Image::computeNearestPowerOfTwo(image.s()-2*_borderWidth)+2*_borderWidth;
+        height = Image::computeNearestPowerOfTwo(image.t()-2*_borderWidth)+2*_borderWidth;
+    }
 
     // cap the size to what the graphics hardware can handle.
     if (width>extensions->maxTextureSize()) width = extensions->maxTextureSize();
     if (height>extensions->maxTextureSize()) height = extensions->maxTextureSize();
-    
+
     inwidth = width;
     inheight = height;
     
@@ -759,6 +774,11 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
     if (!image || !image->data())
         return;
 
+#ifdef DO_TIMING
+    osg::Timer_t start_tick = osg::Timer::instance()->tick();
+    osg::notify(osg::NOTICE)<<"glTexImage2D pixelFormat = "<<std::hex<<image->getPixelFormat()<<std::dec<<std::endl;
+#endif
+
     // get the contextID (user defined ID of 0 upwards) for the 
     // current OpenGL context.
     const unsigned int contextID = state.getContextID();
@@ -822,8 +842,27 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
         
     }    
 
-    bool useHardwareMipMapGeneration = !image->isMipmap() && _useHardwareMipMapGeneration && generateMipMapSupported;
+    unsigned char* dataMinusOffset = 0;
+    unsigned char* dataPlusOffset = 0;
 
+    const PixelBufferObject* pbo = image->getPixelBufferObject();
+    if (pbo && pbo->isBufferObjectSupported(contextID) && !needImageRescale)
+    {
+        pbo->compileBuffer(state);
+        pbo->bindBuffer(contextID);
+        dataMinusOffset = data;
+        dataPlusOffset = reinterpret_cast<unsigned char*>(pbo->offset());
+#ifdef DO_TIMING
+        osg::notify(osg::NOTICE)<<"after PBO "<<osg::Timer::instance()->delta_m(start_tick,osg::Timer::instance()->tick())<<"ms"<<std::endl;
+#endif
+    }
+    else
+    {
+        pbo = 0;
+    }
+
+    bool useHardwareMipMapGeneration = !image->isMipmap() && _useHardwareMipMapGeneration && generateMipMapSupported;
+    
     if( _min_filter == LINEAR || _min_filter == NEAREST || useHardwareMipMapGeneration)
     {
         bool hardwareMipMapOn = false;
@@ -841,7 +880,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
                 inwidth, inheight, _borderWidth,
                 (GLenum)image->getPixelFormat(),
                 (GLenum)image->getDataType(),
-                data );
+                data -dataMinusOffset+dataPlusOffset);
 
         }
         else if (extensions->isCompressedTexImage2DSupported())
@@ -854,7 +893,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
             extensions->glCompressedTexImage2D(target, 0, _internalFormat, 
                 inwidth, inheight,0, 
                 size, 
-                data);                
+                data-dataMinusOffset+dataPlusOffset);                
         }
 
         if (hardwareMipMapOn) glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS,GL_FALSE);
@@ -886,7 +925,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
                          width, height, _borderWidth,
                         (GLenum)image->getPixelFormat(),
                         (GLenum)image->getDataType(),
-                        image->getMipmapData(k));
+                        image->getMipmapData(k)-dataMinusOffset+dataPlusOffset);
 
                     width >>= 1;
                     height >>= 1;
@@ -907,7 +946,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
                     
                     extensions->glCompressedTexImage2D(target, k, _internalFormat, 
                                                        width, height, _borderWidth, 
-                                                       size, image->getMipmapData(k));                
+                                                       size, image->getMipmapData(k)-dataMinusOffset+dataPlusOffset);                
 
                     width >>= 1;
                     height >>= 1;
@@ -924,7 +963,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
                 gluBuild2DMipmaps( target, _internalFormat,
                     inwidth,inheight,
                     (GLenum)image->getPixelFormat(), (GLenum)image->getDataType(),
-                    data );
+                    data -dataMinusOffset+dataPlusOffset);
 
                 int width  = image->s();
                 int height = image->t();
@@ -943,6 +982,17 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
 
     }
 
+    if (pbo)
+    {
+        pbo->unbindBuffer(contextID);
+    }
+#ifdef DO_TIMING
+    static double s_total_time = 0.0;
+    double delta_time = osg::Timer::instance()->delta_m(start_tick,osg::Timer::instance()->tick());
+    s_total_time += delta_time;
+    osg::notify(osg::NOTICE)<<"glTexImage2D "<<delta_time<<"ms  total "<<s_total_time<<"ms"<<std::endl;
+#endif
+
     if (needImageRescale)
     {
         // clean up the resized image.
@@ -971,6 +1021,10 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
     }
     // else image size the same as when loaded so we can go ahead and subload
     
+#ifdef DO_TIMING
+    osg::Timer_t start_tick = osg::Timer::instance()->tick();
+    osg::notify(osg::NOTICE)<<"glTexSubImage2D pixelFormat = "<<std::hex<<image->getPixelFormat()<<std::dec<<std::endl;
+#endif
    
 
     // get the contextID (user defined ID of 0 upwards) for the 
@@ -1026,6 +1080,25 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
     }    
 
     bool useHardwareMipMapGeneration = !image->isMipmap() && _useHardwareMipMapGeneration && generateMipMapSupported;
+    
+    unsigned char* dataMinusOffset=0;
+    unsigned char* dataPlusOffset=0;
+
+    const PixelBufferObject* pbo = image->getPixelBufferObject();
+    if (pbo && pbo->isBufferObjectSupported(contextID) && !needImageRescale)
+    {
+        pbo->compileBuffer(state);
+        pbo->bindBuffer(contextID);
+        dataMinusOffset = data;
+        dataPlusOffset = reinterpret_cast<unsigned char*>(pbo->offset());
+#ifdef DO_TIMING
+        osg::notify(osg::NOTICE)<<"after PBO "<<osg::Timer::instance()->delta_m(start_tick,osg::Timer::instance()->tick())<<"ms"<<std::endl;
+#endif
+    }
+    else
+    {
+        pbo = 0;
+    }
 
     if( _min_filter == LINEAR || _min_filter == NEAREST || useHardwareMipMapGeneration)
     {
@@ -1045,7 +1118,7 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
                 inwidth, inheight,
                 (GLenum)image->getPixelFormat(),
                 (GLenum)image->getDataType(),
-                data );
+                data - dataMinusOffset + dataPlusOffset);
 
         }
         else if (extensions->isCompressedTexImage2DSupported())
@@ -1058,7 +1131,7 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
                 inwidth, inheight,
                 (GLenum)image->getPixelFormat(),
                 size,
-                data );                
+                data - dataMinusOffset + dataPlusOffset );                
         }
 
         if (hardwareMipMapOn) glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS,GL_FALSE);
@@ -1087,7 +1160,7 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
                         width, height,
                         (GLenum)image->getPixelFormat(),
                         (GLenum)image->getDataType(),
-                        image->getMipmapData(k));
+                        image->getMipmapData(k) - dataMinusOffset + dataPlusOffset);
 
                     width >>= 1;
                     height >>= 1;
@@ -1112,7 +1185,7 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
                                                        width, height, 
                                                        (GLenum)image->getPixelFormat(),
                                                        size,
-                                                        image->getMipmapData(k));                
+                                                        image->getMipmapData(k) - dataMinusOffset + dataPlusOffset);                
 
                     //state.checkGLErrors("after extensions->glCompressedTexSubImage2D(");
 
@@ -1124,78 +1197,18 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
         }
         else
         {
-            if (!compressed_image)
-            {
-
-                 numMipmapLevels = 0;
-
-                int width  = inwidth;
-                int height = inheight;
-
-                glPixelStorei(GL_PACK_ALIGNMENT,image->getPacking());
-
-                unsigned int newTotalSize = osg::Image::computeRowWidthInBytes(width,image->getPixelFormat(),image->getDataType(),image->getPacking())*height;
-                unsigned char* copyData1 = new unsigned char [newTotalSize];
-                unsigned char* copyData2 = new unsigned char [newTotalSize];
-                unsigned char* tmpdata = data;
-
-                int previous_width = width;
-                int previous_height = height;
-
-                for( GLsizei k = 0 ; (width || height) ;++k)
-                {
-
-                    if (width == 0)
-                        width = 1;
-                    if (height == 0)
-                        height = 1;
-
-
-                    if (k>0)
-                    {
-                        if (data!=copyData1)
-                        {
-                            gluScaleImage(image->getPixelFormat(),
-                                          previous_width,previous_height,image->getDataType(),data,
-                                          width,height,image->getDataType(),copyData1);
-                            data = copyData1;
-                        }
-                        else
-                        {
-                            gluScaleImage(image->getPixelFormat(),
-                                          previous_width,previous_height,image->getDataType(),data,
-                                          width,height,image->getDataType(),copyData2);
-                            data = copyData2;
-                        }                                  
-                    }
-
-                    glTexSubImage2D( target, k, 
-                        0, 0,
-                        width, height,
-                        (GLenum)image->getPixelFormat(),
-                        (GLenum)image->getDataType(),
-                        data);
-
-                    previous_width = width;
-                    previous_height = height;
-
-                    width >>= 1;
-                    height >>= 1;
-                }
-
-                delete [] copyData1;
-                delete [] copyData2;
-                
-                // restore the data paramters so it can be deleted if it was allocated locally via needImageRescale
-                data = tmpdata;
-            }
-            else 
-            {
-                notify(WARN)<<"Warning:: Compressed image cannot be mip mapped"<<std::endl;
-            }
+            notify(WARN)<<"Warning:: cannot subload mip mapped texture from non mipmapped image."<<std::endl;
         }
     }
     
+    if (pbo)
+    {
+        pbo->unbindBuffer(contextID);
+    }
+#ifdef DO_TIMING
+    osg::notify(osg::NOTICE)<<"glTexSubImage2D "<<osg::Timer::instance()->delta_m(start_tick,osg::Timer::instance()->tick())<<"ms"<<std::endl;
+#endif
+
     if (needImageRescale)
     {
         // clean up the resized image.
@@ -1237,7 +1250,7 @@ static BufferedExtensions s_extensions;
 
 Texture::Extensions* Texture::getExtensions(unsigned int contextID,bool createIfNotInitalized)
 {
-    if (!s_extensions[contextID] && createIfNotInitalized) s_extensions[contextID] = new Extensions;
+    if (!s_extensions[contextID] && createIfNotInitalized) s_extensions[contextID] = new Extensions(contextID);
     return s_extensions[contextID].get();
 }
 
@@ -1246,9 +1259,9 @@ void Texture::setExtensions(unsigned int contextID,Extensions* extensions)
     s_extensions[contextID] = extensions;
 }
 
-Texture::Extensions::Extensions()
+Texture::Extensions::Extensions(unsigned int contextID)
 {
-    setupGLExtensions();
+    setupGLExtensions(contextID);
 }
 
 Texture::Extensions::Extensions(const Extensions& rhs):
@@ -1271,6 +1284,9 @@ Texture::Extensions::Extensions(const Extensions& rhs):
     _isShadowAmbientSupported = rhs._isShadowAmbientSupported;
 
     _isClientStorageSupported = rhs._isClientStorageSupported;
+
+    _isNonPowerOfTwoTextureMipMappedSupported = rhs._isNonPowerOfTwoTextureMipMappedSupported;
+    _isNonPowerOfTwoTextureNonMipMappedSupported = rhs._isNonPowerOfTwoTextureNonMipMappedSupported;
 }
 
 void Texture::Extensions::lowestCommonDenominator(const Extensions& rhs)
@@ -1298,33 +1314,49 @@ void Texture::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._isShadowAmbientSupported) _isShadowAmbientSupported = false;
     
     if (!rhs._isClientStorageSupported) _isClientStorageSupported = false;
+
+    if (!rhs._isNonPowerOfTwoTextureMipMappedSupported) _isNonPowerOfTwoTextureMipMappedSupported = false;
+    if (!rhs._isNonPowerOfTwoTextureNonMipMappedSupported) _isNonPowerOfTwoTextureNonMipMappedSupported = false;
 }
 
-void Texture::Extensions::setupGLExtensions()
+void Texture::Extensions::setupGLExtensions(unsigned int contextID)
 {
     float glVersion = atof( (const char *)glGetString( GL_VERSION ) );
+    const char* renderer = (const char*) glGetString(GL_RENDERER);
+    std::string rendererString(renderer ? renderer : "");
     
     _isMultiTexturingSupported = ( glVersion >= 1.3 ) ||
-                                 isGLExtensionSupported("GL_ARB_multitexture") ||
-                                 isGLExtensionSupported("GL_EXT_multitexture");
-    _isTextureFilterAnisotropicSupported = isGLExtensionSupported("GL_EXT_texture_filter_anisotropic");
+                                 isGLExtensionSupported(contextID,"GL_ARB_multitexture") ||
+                                 isGLExtensionSupported(contextID,"GL_EXT_multitexture");
+    _isTextureFilterAnisotropicSupported = isGLExtensionSupported(contextID,"GL_EXT_texture_filter_anisotropic");
     _isTextureCompressionARBSupported = ( glVersion >= 1.3 ) ||
-                                        isGLExtensionSupported("GL_ARB_texture_compression");
-    _isTextureCompressionS3TCSupported = isGLExtensionSupported("GL_EXT_texture_compression_s3tc");
+                                        isGLExtensionSupported(contextID,"GL_ARB_texture_compression");
+    _isTextureCompressionS3TCSupported = isGLExtensionSupported(contextID,"GL_EXT_texture_compression_s3tc");
     _isTextureMirroredRepeatSupported = ( glVersion >= 1.4 ) || 
-                                    isGLExtensionSupported("GL_IBM_texture_mirrored_repeat") ||
-                                    isGLExtensionSupported("GL_ARB_texture_mirrored_repeat");
+                                    isGLExtensionSupported(contextID,"GL_IBM_texture_mirrored_repeat") ||
+                                    isGLExtensionSupported(contextID,"GL_ARB_texture_mirrored_repeat");
     _isTextureEdgeClampSupported = ( glVersion >= 1.2 ) ||
-                                    isGLExtensionSupported("GL_EXT_texture_edge_clamp") || 
-                                    isGLExtensionSupported("GL_SGIS_texture_edge_clamp");
+                                    isGLExtensionSupported(contextID,"GL_EXT_texture_edge_clamp") || 
+                                    isGLExtensionSupported(contextID,"GL_SGIS_texture_edge_clamp");
     _isTextureBorderClampSupported = ( glVersion >= 1.3 ) ||
-                                    isGLExtensionSupported("GL_ARB_texture_border_clamp");
+                                    isGLExtensionSupported(contextID,"GL_ARB_texture_border_clamp");
     _isGenerateMipMapSupported = (strncmp((const char*)glGetString(GL_VERSION),"1.4",3)>=0) ||
-                                  isGLExtensionSupported("GL_SGIS_generate_mipmap");
-    _isShadowSupported = isGLExtensionSupported("GL_ARB_shadow");
-    _isShadowAmbientSupported = isGLExtensionSupported("GL_ARB_shadow_ambient");
+                                  isGLExtensionSupported(contextID,"GL_SGIS_generate_mipmap");
+    _isShadowSupported = isGLExtensionSupported(contextID,"GL_ARB_shadow");
+    _isShadowAmbientSupported = isGLExtensionSupported(contextID,"GL_ARB_shadow_ambient");
 
-    _isClientStorageSupported = isGLExtensionSupported("GL_APPLE_client_storage");
+    _isClientStorageSupported = isGLExtensionSupported(contextID,"GL_APPLE_client_storage");
+
+    _isNonPowerOfTwoTextureNonMipMappedSupported = ( glVersion >= 2.0 ) ||
+                                       isGLExtensionSupported(contextID,"GL_ARB_texture_non_power_of_two");
+
+    _isNonPowerOfTwoTextureMipMappedSupported = _isNonPowerOfTwoTextureNonMipMappedSupported;
+    
+    if (rendererString.find("Radeon")!=std::string::npos || rendererString.find("RADEON")!=std::string::npos)
+    {
+        _isNonPowerOfTwoTextureMipMappedSupported = false;
+        osg::notify(osg::INFO)<<"Disabling _isNonPowerOfTwoTextureMipMappedSupported for ATI hardware."<<std::endl;
+    }
 
     glGetIntegerv(GL_MAX_TEXTURE_SIZE,&_maxTextureSize);
 
