@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -80,7 +80,7 @@ void Texture1D::setImage(Image* image)
     dirtyTextureObject();
 
     _image = image;
-    _modifiedTag.setAllElementsTo(0);
+    _modifiedCount.setAllElementsTo(0);
 
 }
 
@@ -105,12 +105,12 @@ void Texture1D::apply(State& state) const
         {
             _subloadCallback->subload(*this,state);
         }
-        else if (_image.valid() && getModifiedTag(contextID) != _image->getModifiedTag())
+        else if (_image.valid() && getModifiedCount(contextID) != _image->getModifiedCount())
         {
             applyTexImage1D(GL_TEXTURE_1D,_image.get(),state, _textureWidth, _numMipmapLevels);
 
-            // update the modified tag to show that it is upto date.
-            getModifiedTag(contextID) = _image->getModifiedTag();
+            // update the modified count to show that it is upto date.
+            getModifiedCount(contextID) = _image->getModifiedCount();
         }
 
     }
@@ -149,8 +149,8 @@ void Texture1D::apply(State& state) const
 
         textureObject->setAllocated(_numMipmapLevels,_internalFormat,_textureWidth,1,1,0);
 
-        // update the modified tag to show that it is upto date.
-        getModifiedTag(contextID) = _image->getModifiedTag();
+        // update the modified count to show that it is upto date.
+        getModifiedCount(contextID) = _image->getModifiedCount();
     
         if (_unrefImageDataAfterApply && areAllTextureObjectsLoaded() && _image->getDataVariance()==STATIC)
         {
@@ -188,7 +188,12 @@ void Texture1D::applyTexImage1D(GLenum target, Image* image, State& state, GLsiz
     // select the internalFormat required for the texture.
     bool compressed = isCompressedInternalFormat(_internalFormat);
     
-    image->ensureValidSizeForTexturing(extensions->maxTextureSize());
+    //Rescale if resize hint is set or NPOT not supported or dimension exceeds max size
+    if( _resizeNonPowerOfTwoHint || !extensions->isNonPowerOfTwoTextureSupported(_min_filter) || inwidth > extensions->maxTextureSize() )
+    {
+        // this is not thread safe... should really create local image data and rescale to that as per Texture2D.
+        image->ensureValidSizeForTexturing(extensions->maxTextureSize());
+    }
 
     glPixelStorei(GL_UNPACK_ALIGNMENT,image->getPacking());
 
@@ -324,7 +329,7 @@ void Texture1D::copyTexImage1D(State& state, int x, int y, int width)
     textureObject->setAllocated(_numMipmapLevels,_internalFormat,_textureWidth,1,1,0);
 
     // inform state that this texture is the current one bound.
-    state.haveAppliedAttribute(this);
+    state.haveAppliedTextureAttribute(state.getActiveTextureUnit(), this);
 }
 
 void Texture1D::copyTexSubImage1D(State& state, int xoffset, int x, int y, int width)
@@ -344,7 +349,7 @@ void Texture1D::copyTexSubImage1D(State& state, int xoffset, int x, int y, int w
         glCopyTexSubImage1D( GL_TEXTURE_1D, 0, xoffset, x, y, width);
 
         // inform state that this texture is the current one bound.
-        state.haveAppliedAttribute(this);
+        state.haveAppliedTextureAttribute(state.getActiveTextureUnit(), this);
 
     }
     else
