@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -88,7 +88,7 @@ void Texture3D::setImage(Image* image)
     // delete old texture objects.
     dirtyTextureObject();
 
-    _modifiedTag.setAllElementsTo(0);
+    _modifiedCount.setAllElementsTo(0);
 
     _image = image;
 }
@@ -97,10 +97,22 @@ void Texture3D::computeRequiredTextureDimensions(State& state, const osg::Image&
 {
     const unsigned int contextID = state.getContextID();
     const Extensions* extensions = getExtensions(contextID,true);
+    const Texture::Extensions* texExtensions = Texture::getExtensions(contextID,true);
 
-    int width = Image::computeNearestPowerOfTwo(image.s()-2*_borderWidth)+2*_borderWidth;
-    int height = Image::computeNearestPowerOfTwo(image.t()-2*_borderWidth)+2*_borderWidth;
-    int depth = Image::computeNearestPowerOfTwo(image.r()-2*_borderWidth)+2*_borderWidth;
+    int width,height,depth;
+
+    if( !_resizeNonPowerOfTwoHint && texExtensions->isNonPowerOfTwoTextureSupported(_min_filter) )
+    {
+        width = image.s();
+        height = image.t();
+        depth = image.r();
+    }
+    else
+    {
+        width = Image::computeNearestPowerOfTwo(image.s()-2*_borderWidth)+2*_borderWidth;
+        height = Image::computeNearestPowerOfTwo(image.t()-2*_borderWidth)+2*_borderWidth;
+        depth = Image::computeNearestPowerOfTwo(image.r()-2*_borderWidth)+2*_borderWidth;
+    }
 
     // cap the size to what the graphics hardware can handle.
     if (width>extensions->maxTexture3DSize()) width = extensions->maxTexture3DSize();
@@ -170,12 +182,12 @@ void Texture3D::apply(State& state) const
         {
             _subloadCallback->subload(*this,state);
         }
-        else if (_image.get() && getModifiedTag(contextID) != _image->getModifiedTag())
+        else if (_image.get() && getModifiedCount(contextID) != _image->getModifiedCount())
         {
             applyTexImage3D(GL_TEXTURE_3D,_image.get(),state, _textureWidth, _textureHeight, _textureDepth,_numMipmapLevels);
 
-            // update the modified tag to show that it is upto date.
-            getModifiedTag(contextID) = _image->getModifiedTag();
+            // update the modified count to show that it is upto date.
+            getModifiedCount(contextID) = _image->getModifiedCount();
         }
 
     }
@@ -219,8 +231,8 @@ void Texture3D::apply(State& state) const
 
         textureObject->setAllocated(_numMipmapLevels,_internalFormat,_textureWidth,_textureHeight,_textureDepth,0);
 
-        // update the modified tag to show that it is upto date.
-        getModifiedTag(contextID) = _image->getModifiedTag();
+        // update the modified count to show that it is upto date.
+        getModifiedCount(contextID) = _image->getModifiedCount();
 
         if (_unrefImageDataAfterApply && areAllTextureObjectsLoaded() && _image->getDataVariance()==STATIC)
         {
@@ -250,6 +262,7 @@ void Texture3D::applyTexImage3D(GLenum target, Image* image, State& state, GLsiz
     // current OpenGL context.
     const unsigned int contextID = state.getContextID();
     const Extensions* extensions = getExtensions(contextID,true);    
+    const Texture::Extensions* texExtensions = Texture::getExtensions(contextID,true);
 
     // compute the internal texture format, this set the _internalFormat to an appropriate value.
     computeInternalFormat();
@@ -264,7 +277,12 @@ void Texture3D::applyTexImage3D(GLenum target, Image* image, State& state, GLsiz
         //return;
     }    
     
-    image->ensureValidSizeForTexturing(extensions->maxTexture3DSize());
+    //Rescale if resize hint is set or NPOT not supported or dimensions exceed max size
+    if( _resizeNonPowerOfTwoHint || !texExtensions->isNonPowerOfTwoTextureSupported(_min_filter)
+        || inwidth > extensions->maxTexture3DSize()
+        || inheight > extensions->maxTexture3DSize()
+        || indepth > extensions->maxTexture3DSize() )
+        image->ensureValidSizeForTexturing(extensions->maxTexture3DSize());
 
     glPixelStorei(GL_UNPACK_ALIGNMENT,image->getPacking());
 
@@ -369,7 +387,7 @@ void Texture3D::copyTexSubImage3D(State& state, int xoffset, int yoffset, int zo
         //glBindTexture( GL_TEXTURE_3D, handle );
 
         // inform state that this texture is the current one bound.
-        state.haveAppliedAttribute(this);
+        state.haveAppliedTextureAttribute(state.getActiveTextureUnit(), this);
 
     }
     else
@@ -383,7 +401,7 @@ static BufferedExtensions s_extensions;
 
 Texture3D::Extensions* Texture3D::getExtensions(unsigned int contextID,bool createIfNotInitalized)
 {
-    if (!s_extensions[contextID] && createIfNotInitalized) s_extensions[contextID] = new Extensions;
+    if (!s_extensions[contextID] && createIfNotInitalized) s_extensions[contextID] = new Extensions(contextID);
     return s_extensions[contextID].get();
 }
 
@@ -396,9 +414,9 @@ void Texture3D::setExtensions(unsigned int contextID,Extensions* extensions)
 #define GL_MAX_3D_TEXTURE_SIZE 0x8073
 #endif
 
-Texture3D::Extensions::Extensions()
+Texture3D::Extensions::Extensions(unsigned int contextID)
 {
-    setupGLExtenions();
+    setupGLExtenions(contextID);
 }
 
 Texture3D::Extensions::Extensions(const Extensions& rhs):
@@ -428,9 +446,9 @@ void Texture3D::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._gluBuild3DMipmaps)                    _gluBuild3DMipmaps = 0;
 }
 
-void Texture3D::Extensions::setupGLExtenions()
+void Texture3D::Extensions::setupGLExtenions(unsigned int contextID)
 {
-    _isTexture3DFast = isGLExtensionSupported("GL_EXT_texture3D");
+    _isTexture3DFast = isGLExtensionSupported(contextID,"GL_EXT_texture3D");
 
     if (_isTexture3DFast) _isTexture3DSupported = true;
     else _isTexture3DSupported = strncmp((const char*)glGetString(GL_VERSION),"1.2",3)>=0;
