@@ -8,6 +8,8 @@
 
 #include <OpenThreads/ScopedLock>
 
+#include <algorithm>
+
 #ifdef WIN32
 #include <windows.h>
 #else
@@ -33,8 +35,8 @@ DatabasePager::DatabasePager()
     _frameBlock = new Block;
     _databasePagerThreadBlock = new Block;
 
-    _threadPriorityDuringFrame = PRIORITY_MIN;
-    _threadPriorityOutwithFrame = PRIORITY_MIN;
+    _threadPriorityDuringFrame = THREAD_PRIORITY_MIN;
+    _threadPriorityOutwithFrame = THREAD_PRIORITY_MIN;
 
     _changeAutoUnRef = true;
     _valueAutoUnRef = true;
@@ -50,6 +52,10 @@ DatabasePager::DatabasePager()
     
     _expiryDelay = 10;
 
+    _targetFrameRate = 100.0;
+    _minimumTimeAvailableForGLCompileAndDeletePerFrame = 0.001; // 1ms.
+    _maximumNumOfObjectsToCompilePerFrame = 8;
+
     // make sure a SharedStateManager exists.
     //osgDB::Registry::instance()->getOrCreateSharedStateManager();
     
@@ -799,6 +805,7 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
             {
                 std::sort(_dataToCompileList.begin(),_dataToCompileList.end(),SortFileRequestFunctor());
 
+                // prune all the old entries.
                 DatabaseRequestList::iterator litr;
                 int i=0;
                 for(litr = _dataToCompileList.begin();
@@ -824,8 +831,9 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
                     _dataToCompileList.erase(litr,_dataToCompileList.end());
                 }
 
-
-                databaseRequest = _dataToCompileList.front();
+                // advance to the next entry to compile if one is available.
+                databaseRequest = _dataToCompileList.empty() ? 0 : _dataToCompileList.front();
+            
             }
         };
 
@@ -834,7 +842,7 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
 
         // while there are valid databaseRequest's in the to compile list and there is
         // sufficient time left compile each databaseRequest's stateset and drawables.
-        while (databaseRequest.valid() && elapsedTime<availableTime && numObjectsCompiled<maxNumObjectsToCompile) 
+        while (databaseRequest.valid() && elapsedTime<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame) 
         {
             DataToCompileMap& dcm = databaseRequest->_dataToCompileMap;
             DataToCompile& dtc = dcm[state.getContextID()];
