@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -171,6 +171,9 @@ std::string osgDB::findFileInPath(const std::string& filename, const FilePathLis
     if (filename.empty()) 
         return filename;
 
+    if (!isFileNameNativeStyle(filename)) 
+        return findFileInPath(convertFileNameToNativeStyle(filename), filepath, caseSensitivity);
+
 
     for(FilePathList::const_iterator itr=filepath.begin();
         itr!=filepath.end();
