@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -150,6 +150,7 @@ Registry::Registry()
     addFileExtensionAlias("ivz",   "gz");
     addFileExtensionAlias("ozg",   "gz");
     
+
 #if defined(DARWIN_QUICKTIME)
     addFileExtensionAlias("jpg",  "qt");
     addFileExtensionAlias("jpe",  "qt");
@@ -158,16 +159,29 @@ Registry::Registry()
     addFileExtensionAlias("tiff", "qt");
     addFileExtensionAlias("gif",  "qt");
     addFileExtensionAlias("png",  "qt");
+    addFileExtensionAlias("psd",  "qt");
+    addFileExtensionAlias("tga",  "qt");
     addFileExtensionAlias("mov",  "qt");
+    addFileExtensionAlias("avi",  "qt");
     addFileExtensionAlias("mpg",  "qt");
     addFileExtensionAlias("mpv",  "qt");
     addFileExtensionAlias("dv",   "qt");
+    addFileExtensionAlias("mp4",   "qt");
 #else
     addFileExtensionAlias("jpg",  "jpeg");
     addFileExtensionAlias("jpe",  "jpeg");
     addFileExtensionAlias("tif",  "tiff");
-    addFileExtensionAlias("mpg",  "mpeg");
-    addFileExtensionAlias("mpv",  "mpeg");
+
+    // really need to decide this at runtime...
+    #if defined(USE_XINE)
+        addFileExtensionAlias("mov",  "xine");
+        addFileExtensionAlias("mpg",  "xine");
+        addFileExtensionAlias("mpv",  "xine");
+        addFileExtensionAlias("dv",   "xine");
+        addFileExtensionAlias("avi",  "xine");
+        addFileExtensionAlias("wmv",  "xine");
+    #endif
+
 #endif
 
 
@@ -231,6 +245,102 @@ static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENT
 static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_FILE_PATH <path>[;path]..","Paths for locating datafiles");
 #endif
 
+#if defined(__APPLE__)
+
+//Executable packages should be able to load file resources from inside the packages.
+//These resources should be stored in YourProgram.app/Contents/Resources, and so
+//should the path list for the data files should include that path by default.
+#include <CoreServices/CoreServices.h>
+
+void Registry::initDataFilePathList()
+{
+  
+  FilePathList filepath;
+  
+  //
+  // set up data file paths
+  //
+  char *ptr;
+  
+  if( (ptr = getenv( "OSG_FILE_PATH" )) )
+  {
+    //notify(DEBUG_INFO) << "OSG_FILE_PATH("<<ptr<<")"<<std::endl;
+    convertStringPathIntoFilePathList(ptr, filepath);
+  }
+  else if( (ptr = getenv( "OSGFILEPATH" )) )
+  {
+    //notify(DEBUG_INFO) << "OSGFILEPATH("<<ptr<<")"<<std::endl;
+    convertStringPathIntoFilePathList(ptr, filepath);
+  }
+  
+
+  char* buffer = NULL;
+  char* bundlePathBuffer = NULL;
+  CFURLRef  url;
+  CFStringRef pathString;
+  CFBundleRef myBundle;
+  CFStringRef bundlePathString;
+  CFIndex strLen; 
+  CFIndex maxBufferSize;
+  
+  myBundle = CFBundleGetMainBundle();
+  if (myBundle != NULL) {
+    // Get the URL to the bundle
+    url = CFBundleCopyBundleURL( myBundle );
+    
+    // Convert the URL to a CFString that looks like a Unix file path
+    bundlePathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
+    
+    // Convert the CFString to a C string
+    strLen = CFStringGetLength( bundlePathString );
+    maxBufferSize = CFStringGetMaximumSizeForEncoding( strLen, kCFStringEncodingUTF8 );
+    bundlePathBuffer = (char*) malloc(maxBufferSize);
+    CFStringGetCString( bundlePathString, bundlePathBuffer, maxBufferSize, kCFStringEncodingUTF8 );
+    
+    CFRelease( url );
+
+    // Now find the resources folder
+    url = CFBundleCopyResourcesDirectoryURL( myBundle );
+    //pathString = CFURLCopyPath( url );
+    // Convert the URL to a CFString that looks like a Unix file path
+    pathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
+    
+    // Convert the CFString to a C string
+    strLen = CFStringGetLength( pathString );
+    maxBufferSize = CFStringGetMaximumSizeForEncoding( strLen, kCFStringEncodingUTF8 );
+    buffer = (char*) malloc(maxBufferSize);
+    CFStringGetCString( pathString, buffer, maxBufferSize, kCFStringEncodingUTF8 );
+    
+    // Combine the string and copy it into the FilePath list
+    filepath.push_back( std::string(bundlePathBuffer) 
+                         + std::string("/")
+                         + std::string(buffer)
+                         );
+    
+    CFRelease( pathString );
+    CFRelease( bundlePathString );
+    CFRelease( url );
+    
+    if (bundlePathBuffer != NULL) free(bundlePathBuffer);
+    if (buffer != NULL) free(buffer);
+    
+    pathString = NULL;
+    bundlePathString = NULL;
+    url = NULL;
+  }
+  else
+  {
+    osg::notify( osg::DEBUG_INFO ) << "Couldn't find the Application Bundle" << std::endl;
+  }
+  
+  setDataFilePathList(filepath);
+
+  //osg::notify(INFO)<<"Data FilePathList"<<std::endl;
+  //PrintFilePathList(osg::notify(INFO),getDataFilePathList());
+}
+
+#else
+
 void Registry::initDataFilePathList()
 {
     //
@@ -252,6 +362,8 @@ void Registry::initDataFilePathList()
     //PrintFilePathList(osg::notify(INFO),getDataFilePathList());
 }
 
+#endif
+
 void Registry::setDataFilePathList(const std::string& paths)
 {
     _dataFilePath.clear(); 
@@ -363,6 +475,11 @@ void Registry::addDotOsgWrapper(DotOsgWrapper* wrapper)
             _stateAttrWrapperMap[name] = wrapper;
             _stateAttrWrapperMap[compositeName] = wrapper;
         }
+        if (dynamic_cast<const Uniform*>(proto))
+        {
+            _uniformWrapperMap[name] = wrapper;
+            _uniformWrapperMap[compositeName] = wrapper;
+        }
         if (dynamic_cast<const Node*>(proto))
         {
             _nodeWrapperMap[name] = wrapper;
@@ -402,6 +519,7 @@ void Registry::removeDotOsgWrapper(DotOsgWrapper* wrapper)
     eraseWrapper(_classNameWrapperMap,wrapper);
     eraseWrapper(_imageWrapperMap,wrapper);
     eraseWrapper(_drawableWrapperMap,wrapper);
+    eraseWrapper(_uniformWrapperMap,wrapper);
     eraseWrapper(_stateAttrWrapperMap,wrapper);
     eraseWrapper(_nodeWrapperMap,wrapper);
 }
@@ -450,26 +568,32 @@ std::string Registry::createLibraryNameForExtension(const std::string& ext)
     ExtensionAliasMap::iterator itr=_extAliasMap.find(ext);
     if (itr!=_extAliasMap.end()) return createLibraryNameForExtension(itr->second);
 
+#ifdef OSG_JAVA_BUILD
+	static std::string prepend = "java";
+#else
+	static std::string prepend = "";
+#endif
+
 #if defined(WIN32)
     // !! recheck evolving Cygwin DLL extension naming protocols !! NHV
     #ifdef __CYGWIN__
-        return "cygosgdb_"+ext+".dll";
+        return "cyg"+prepend+"osgdb_"+ext+".dll";
     #elif defined(__MINGW32__)
-        return "libosgdb_"+ext+".dll";
+        return "lib"+prepend+"osgdb_"+ext+".dll";
     #else
         #ifdef _DEBUG
-            return "osgdb_"+ext+"d.dll";
+            return prepend+"osgdb_"+ext+"d.dll";
         #else
-            return "osgdb_"+ext+".dll";
+            return prepend+"osgdb_"+ext+".dll";
         #endif
     #endif
 #elif macintosh
-    return "osgdb_"+ext;
+    return prepend+"osgdb_"+ext;
 #elif defined(__hpux__)
     // why don't we use PLUGIN_EXT from the makefiles here?
-    return "osgdb_"+ext+".sl";
+    return prepend+"osgdb_"+ext+".sl";
 #else
-    return "osgdb_"+ext+".so";
+    return prepend+"osgdb_"+ext+".so";
 #endif
 
 }
@@ -953,6 +1077,27 @@ StateAttribute* Registry::readStateAttribute(Input& fr)
     return dynamic_cast<StateAttribute*>(readObject(_stateAttrWrapperMap,fr));
 }
 
+//
+// read drawable from input iterator.
+//
+Uniform* Registry::readUniform(Input& fr)
+{
+
+    if (fr[0].matchWord("Use"))
+    {
+        if (fr[1].isString())
+        {
+            Uniform* attribute = dynamic_cast<Uniform*>(fr.getObjectForUniqueID(fr[1].getStr()));
+            if (attribute) fr+=2;
+            return attribute;
+        }
+        else return NULL;
+
+    }
+
+    return dynamic_cast<Uniform*>(readObject(_uniformWrapperMap,fr));
+}
+
 //
 // read node from input iterator.
 //
@@ -1548,6 +1693,13 @@ void Registry::addEntryToObjectCache(const std::string& filename, osg::Object* o
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_objectCacheMutex);
     _objectCache[filename]=ObjectTimeStampPair(object,timestamp);
 }
+osg::Object* Registry::getFromObjectCache(const std::string& fileName)
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_objectCacheMutex);
+    ObjectCache::iterator itr = _objectCache.find(fileName);
+    if (itr!=_objectCache.end()) return itr->second.first.get();
+    else return 0;
+}
 
 void Registry::updateTimeStampOfObjectsInCacheWithExtenalReferences(double currentTime)
 {
@@ -1569,8 +1721,6 @@ void Registry::updateTimeStampOfObjectsInCacheWithExtenalReferences(double curre
 
 void Registry::removeExpiredObjectsInCache(double expiryTime)
 {
-    return;
-
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_objectCacheMutex);
 
     typedef std::vector<std::string> ObjectsToRemove;
@@ -1637,6 +1787,19 @@ void Registry::clearArchiveCache()
     _archiveCache.clear();
 }
 
+void Registry::releaseGLObjects(osg::State* state)
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_objectCacheMutex);
+
+    for(ObjectCache::iterator itr = _objectCache.begin();
+        itr != _objectCache.end();
+        ++itr)
+    {
+        osg::Object* object = itr->second.first.get();
+        object->releaseGLObjects(state);
+    }
+}
+
 DatabasePager* Registry::getOrCreateDatabasePager()
 {
     if (!_databasePager) _databasePager = new DatabasePager;
