@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -30,11 +30,11 @@ public:
 
     virtual std::string getFileName() const { return _filename; }
 
-    virtual void setSize(unsigned int width, unsigned int height);
+    virtual void setFontResolution(unsigned int width, unsigned int height);
 
     virtual osgText::Font::Glyph* getGlyph(unsigned int charcode);
     
-	virtual osg::Vec2 getKerning(unsigned int leftcharcode,unsigned int rightcharcode, osgText::KerningType _kerningType);
+    virtual osg::Vec2 getKerning(unsigned int leftcharcode,unsigned int rightcharcode, osgText::KerningType _kerningType);
     
     virtual bool hasVertical() const;
 
