@@ -9,7 +9,7 @@
  *
  *    HISTORY:        Created 11.03.2003
  *                    Updated for texture1D by Don Burns, 27.1.2004
- *					  Updated for light model - Stan Blinov at 25 august 7512 from World Creation (7.09.2004)
+ *                      Updated for light model - Stan Blinov at 25 august 7512 from World Creation (7.09.2004)
  *
  *
  *    Copyright 2003 VR-C
@@ -23,6 +23,7 @@
 #include "CullFace.h"
 #include "ClipPlane.h"
 #include "PolygonOffset.h"
+//#include "PolygonMode.h"
 #include "ShadeModel.h"
 #include "Point.h"
 #include "LineWidth.h"
@@ -37,6 +38,9 @@
 #include "FragmentProgram.h"
 #include "VertexProgram.h"
 #include "LightModel.h"
+#include "ProxyNode.h"
+#include "FrontFace.h"
+#include "Program.h"
 
 #include "Group.h"
 #include "MatrixTransform.h"
@@ -55,26 +59,46 @@
 #include "OccluderNode.h"
 #include "Impostor.h"
 #include "CoordinateSystemNode.h"
+#include "Uniform.h"
+#include "Shader.h"
 
 #include "LightPointNode.h"
 #include "MultiSwitch.h"
 #include "VisibilityGroup.h"
 
+#include "MultiTextureControl.h"
+
 #include "Geometry.h"
 #include "ShapeDrawable.h"
 #include "Shape.h"
 
+#include "Text.h"
+
 #include <osg/Endian>
 #include <osg/Notify>
+#include <osg/io_utils>
 #include <osgDB/ReadFile>
 
 
 using namespace ive;
 using namespace std;
 
+void DataInputStream::setOptions(const osgDB::ReaderWriter::Options* options) 
+{ 
+    _options = options; 
+
+    if (_options.get())
+    {
+        setLoadExternalReferenceFiles(_options->getOptionString().find("noLoadExternalReferenceFiles")==std::string::npos);
+        osg::notify(osg::DEBUG_INFO) << "ive::DataInputStream.setLoadExternalReferenceFiles()=" << getLoadExternalReferenceFiles() << std::endl;
+    }
+}
+
 DataInputStream::DataInputStream(std::istream* istream)
 {
     unsigned int endianType ;
+
+    _loadExternalReferenceFiles = false;
     
     _verboseOutput = false;
 
@@ -313,6 +337,39 @@ osg::Vec4 DataInputStream::readVec4(){
     
     return v;
 }
+osg::Vec2d DataInputStream::readVec2d()
+{
+    osg::Vec2d v;
+    v.x()=readDouble();
+    v.y()=readDouble();
+
+    if (_verboseOutput) std::cout<<"read/writeVec2d() ["<<v<<"]"<<std::endl;
+    
+    return v;
+}
+
+osg::Vec3d DataInputStream::readVec3d(){
+    osg::Vec3d v;
+    v.x()=readDouble();
+    v.y()=readDouble();
+    v.z()=readDouble();
+
+    if (_verboseOutput) std::cout<<"read/writeVec3d() ["<<v<<"]"<<std::endl;
+
+    return v;
+}
+
+osg::Vec4d DataInputStream::readVec4d(){
+    osg::Vec4d v;
+    v.x()=readDouble();
+    v.y()=readDouble();
+    v.z()=readDouble();
+    v.w()=readDouble();
+
+    if (_verboseOutput) std::cout<<"read/writeVec4d() ["<<v<<"]"<<std::endl;
+    
+    return v;
+}
 
 osg::Plane DataInputStream::readPlane(){
     osg::Plane v;
@@ -555,9 +612,29 @@ osg::Vec4Array* DataInputStream::readVec4Array(){
     return a;
 }
 
-osg::Matrix DataInputStream::readMatrix()
+osg::Matrixf DataInputStream::readMatrixf()
 {
-    osg::Matrix mat;
+    osg::Matrixf mat;
+    for(int r=0;r<4;r++)
+    {
+        for(int c=0;c<4;c++)
+        {
+            mat(r,c) = readFloat();
+        }
+    }
+
+    if (_istream->rdstate() & _istream->failbit)
+        throw Exception("DataInputStream::readMatrix(): Failed to read Matrix array.");
+
+    if (_verboseOutput) std::cout<<"read/writeMatrix() ["<<mat<<"]"<<std::endl;
+    
+
+    return mat;
+}
+
+osg::Matrixd DataInputStream::readMatrixd()
+{
+    osg::Matrixd mat;
     for(int r=0;r<4;r++)
     {
         for(int c=0;c<4;c++)
@@ -579,12 +656,12 @@ osg::Image* DataInputStream::readImage(std::string filename)
 {
     // If image is already read and in list 
     // then just return pointer to this.
-    ImageMap::iterator mitr=_imageMap.find(filename);
-    if (mitr!=_imageMap.end()) mitr->second.get();
+    ImageMap::iterator mitr=_imageMap.find(filename);    
+    if (mitr!=_imageMap.end()) return mitr->second.get();
         
     // Image is not in list. 
     // Read it from disk, 
-    osg::Image* image = osgDB::readImageFile(filename.c_str());
+    osg::Image* image = osgDB::readImageFile(filename.c_str(),_options.get());
         
     // add it to the imageList,
     _imageMap[filename] = image;
@@ -657,6 +734,10 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::PolygonOffset();
         ((ive::PolygonOffset*)(attribute))->read(this);
     }
+/*    else if(attributeID == IVEPOLYGONMODE){
+        attribute = new osg::PolygonMode();
+        ((ive::PolygonMode*)(attribute))->read(this);
+    }*/
     else if(attributeID == IVESHADEMODEL){
         attribute = new osg::ShadeModel();
         ((ive::ShadeModel*)(attribute))->read(this);
@@ -709,12 +790,18 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::VertexProgram();
         ((ive::VertexProgram*)(attribute))->read(this);
     }
-
     else if(attributeID == IVELIGHTMODEL){
         attribute = new osg::LightModel();
         ((ive::LightModel*)(attribute))->read(this);
     }
-
+    else if(attributeID == IVEFRONTFACE){
+        attribute = new osg::FrontFace();
+        ((ive::FrontFace*)(attribute))->read(this);
+    }
+    else if(attributeID == IVEPROGRAM){
+        attribute = new osg::Program();
+        ((ive::Program*)(attribute))->read(this);
+    }
     else{
         throw Exception("Unknown StateAttribute in StateSet::read()");
     }
@@ -728,6 +815,55 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
     return attribute;
 }
 
+osg::Uniform* DataInputStream::readUniform()
+{
+    // Read uniforms unique ID.
+    int id = readInt();
+    // See if uniform is already in the list.
+    UniformMap::iterator itr= _uniformMap.find(id);
+    if (itr!=_uniformMap.end()) return itr->second.get();
+
+    // Uniform is not in list.
+    // Create a new uniform,
+    osg::Uniform* uniform = new osg::Uniform();
+
+    // read its properties from stream
+    ((ive::Uniform*)(uniform))->read(this);
+        
+    // and add it to the uniform map,
+    _uniformMap[id] = uniform;
+        
+
+    if (_verboseOutput) std::cout<<"read/writeUniform() ["<<id<<"]"<<std::endl;
+    
+    return uniform;
+}
+
+
+osg::Shader* DataInputStream::readShader()
+{
+    // Read shaders unique ID.
+    int id = readInt();
+    // See if shader is already in the list.
+    ShaderMap::iterator itr= _shaderMap.find(id);
+    if (itr!=_shaderMap.end()) return itr->second.get();
+
+    // Shader is not in list.
+    // Create a new shader,
+    osg::Shader* shader = new osg::Shader();
+
+    // read its properties from stream
+    ((ive::Shader*)(shader))->read(this);
+        
+    // and add it to the shader map,
+    _shaderMap[id] = shader;
+        
+
+    if (_verboseOutput) std::cout<<"read/writeShader() ["<<id<<"]"<<std::endl;
+    
+    return shader;
+}
+
 osg::Drawable* DataInputStream::readDrawable()
 {
     // Read stateattributes unique ID.
@@ -751,6 +887,10 @@ osg::Drawable* DataInputStream::readDrawable()
         drawable = new osg::ShapeDrawable();                
         ((ShapeDrawable*)(drawable))->read(this);
     }
+    else if(drawableTypeID == IVETEXT){
+        drawable = new osgText::Text();
+        ((Text*)(drawable))->read(this);
+    }    
     else
         throw Exception("Unknown drawable drawableTypeIDentification in Geode::read()");
 
@@ -887,7 +1027,7 @@ osg::Node* DataInputStream::readNode()
         ((ive::MultiSwitch*)(node))->read(this);
     }
     else if(nodeTypeID== IVEIMPOSTOR){
-        node = new osg::Impostor();
+        node = new osgSim::Impostor();
         ((ive::Impostor*)(node))->read(this);
     }
     else if(nodeTypeID== IVEOCCLUDERNODE){
@@ -898,6 +1038,10 @@ osg::Node* DataInputStream::readNode()
         node = new osgSim::VisibilityGroup();
         ((ive::VisibilityGroup*)(node))->read(this);
     }
+    else if(nodeTypeID== IVEPROXYNODE){
+        node = new osg::ProxyNode();
+        ((ive::ProxyNode*)(node))->read(this);
+    }
     else if(nodeTypeID== IVEGROUP){
         node = new osg::Group();
         ((ive::Group*)(node))->read(this);
@@ -914,6 +1058,10 @@ osg::Node* DataInputStream::readNode()
         node = new osgSim::LightPointNode();
         ((ive::LightPointNode*)(node))->read(this);
     }
+    else if(nodeTypeID== IVEMULTITEXTURECONTROL){
+        node = new osgFX::MultiTextureControl();
+        ((ive::MultiTextureControl*)(node))->read(this);
+    }
     else{
         throw Exception("Unknown node identification in DataInputStream::readNode()");
     }
