@@ -9,7 +9,7 @@
  *
  *    HISTORY:        Created 11.03.2003
  *                    Updated for 1D textures - Don Burns 27.1.2004
- *					  Updated for light model - Stan Blinov at 25 august 7512 from World Creation (7.09.2004)
+ *                      Updated for light model - Stan Blinov at 25 august 7512 from World Creation (7.09.2004)
  *
  *    Copyright 2003 VR-C
  **********************************************************************/
@@ -24,6 +24,7 @@
 #include "CullFace.h"
 #include "ClipPlane.h"
 #include "PolygonOffset.h"
+//#include "PolygonMode.h"
 #include "ShadeModel.h"
 #include "Point.h"
 #include "LineWidth.h"
@@ -38,7 +39,11 @@
 #include "FragmentProgram.h"
 #include "VertexProgram.h"
 #include "LightModel.h"
-
+#include "ProxyNode.h"
+#include "FrontFace.h"
+#include "Program.h"
+#include "Uniform.h"
+#include "Shader.h"
 
 #include "Group.h"
 #include "MatrixTransform.h"
@@ -62,18 +67,52 @@
 #include "MultiSwitch.h"
 #include "VisibilityGroup.h"
 
+#include "MultiTextureControl.h"
+
 #include "Geometry.h"
 #include "ShapeDrawable.h"
 
 #include "Shape.h"
 
+#include "Text.h"
+
+#include <osg/Notify>
+#include <osg/io_utils>
+
 using namespace ive;
 
+
+void DataOutputStream::setOptions(const osgDB::ReaderWriter::Options* options) 
+{ 
+    _options = options; 
+
+    if (_options.get())
+    {
+        setIncludeImageData(_options->getOptionString().find("noTexturesInIVEFile")==std::string::npos);
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeImageData()=" << getIncludeImageData() << std::endl;
+
+        setIncludeExternalReferences(_options->getOptionString().find("inlineExternalReferencesInIVEFile")!=std::string::npos);
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeExternalReferences()=" << getIncludeExternalReferences() << std::endl;
+
+        setWriteExternalReferenceFiles(_options->getOptionString().find("noWriteExternalReferenceFiles")==std::string::npos);
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setWriteExternalReferenceFiles()=" << getWriteExternalReferenceFiles() << std::endl;
+
+        setUseOriginalExternalReferences(_options->getOptionString().find("useOriginalExternalReferences")!=std::string::npos);
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setUseOriginalExternalReferences()=" << getUseOriginalExternalReferences() << std::endl;
+    }
+}
+
 DataOutputStream::DataOutputStream(std::ostream * ostream)
 {
     _verboseOutput = false;
 
     _includeImageData= true;
+
+    _includeExternalReferences     = false;
+    _writeExternalReferenceFiles   = false;
+    _useOriginalExternalReferences = true;
+    
+    
     _ostream = ostream;
     if(!_ostream)
         throw Exception("DataOutputStream::DataOutputStream(): null pointer exception in argument.");
@@ -182,6 +221,30 @@ void DataOutputStream::writeVec4(const osg::Vec4& v){
     if (_verboseOutput) std::cout<<"read/writeVec4() ["<<v<<"]"<<std::endl;
 }
 
+void DataOutputStream::writeVec2d(const osg::Vec2d& v){
+    writeDouble(v.x());
+    writeDouble(v.y());
+    
+    if (_verboseOutput) std::cout<<"read/writeVec2() ["<<v<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec3d(const osg::Vec3d& v){
+    writeDouble(v.x());
+    writeDouble(v.y());
+    writeDouble(v.z());
+    
+    if (_verboseOutput) std::cout<<"read/writeVec3d() ["<<v<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec4d(const osg::Vec4d& v){
+    writeDouble(v.x());
+    writeDouble(v.y());
+    writeDouble(v.z());
+    writeDouble(v.w());
+    
+    if (_verboseOutput) std::cout<<"read/writeVec4d() ["<<v<<"]"<<std::endl;
+}
+
 void DataOutputStream::writePlane(const osg::Plane& v)
 {
     writeFloat(v[0]);
@@ -366,7 +429,20 @@ void DataOutputStream::writeVec4Array(const osg::Vec4Array* a)
     if (_verboseOutput) std::cout<<"read/writeVec4Array() ["<<size<<"]"<<std::endl;
 }
 
-void DataOutputStream::writeMatrix(const osg::Matrix& mat)
+void DataOutputStream::writeMatrixf(const osg::Matrixf& mat)
+{
+    for(int r=0;r<4;r++)
+    {
+        for(int c=0;c<4;c++)
+        {
+            writeFloat(mat(r,c));
+        }
+    }
+    
+    if (_verboseOutput) std::cout<<"read/writeMatrix() ["<<mat<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeMatrixd(const osg::Matrixd& mat)
 {
     for(int r=0;r<4;r++)
     {
@@ -407,7 +483,6 @@ void DataOutputStream::writeStateSet(const osg::StateSet* stateset)
         if (_verboseOutput) std::cout<<"read/writeStateSet() ["<<id<<"]"<<std::endl;
 
     }
-    
 }
 
 void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
@@ -453,6 +528,10 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::PolygonOffset*>(attribute)){
             ((ive::PolygonOffset*)(attribute))->write(this);
         }
+/*        // This is a PolygonMode
+        else if(dynamic_cast<const osg::PolygonMode*>(attribute)){
+            ((ive::PolygonMode*)(attribute))->write(this);
+        }*/
         else if(dynamic_cast<const osg::ShadeModel*>(attribute)){
             ((ive::ShadeModel*)(attribute))->write(this);
         }
@@ -505,11 +584,18 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::VertexProgram*>(attribute)){
             ((ive::VertexProgram*)(attribute))->write(this);
         }
-
         // This is a LightModel
         else if(dynamic_cast<const osg::LightModel*>(attribute)){
             ((ive::LightModel*)(attribute))->write(this);
         }
+        // This is a FrontFace
+        else if(dynamic_cast<const osg::FrontFace*>(attribute)){
+            ((ive::FrontFace*)(attribute))->write(this);
+        }
+        // This is a FrontFace
+        else if(dynamic_cast<const osg::Program*>(attribute)){
+            ((ive::Program*)(attribute))->write(this);
+        }
 
         else{
             std::string className = attribute->className();
@@ -519,6 +605,64 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
     }
 }
 
+void DataOutputStream::writeUniform(const osg::Uniform* uniform)
+{
+    UniformMap::iterator itr = _uniformMap.find(uniform);
+    if (itr!=_uniformMap.end())
+    {
+        // Id already exists so just write ID.
+        writeInt(itr->second);
+
+        if (_verboseOutput) std::cout<<"read/writeUniform() ["<<itr->second<<"]"<<std::endl;
+    }
+    else
+    {
+        // id doesn't exist so create a new ID and
+        // register the uniform.
+
+        int id = _uniformMap.size();
+        _uniformMap[uniform] = id;
+
+        // write the id.
+        writeInt(id);
+
+        // write the stateset.
+        ((ive::Uniform*)(uniform))->write(this);
+
+        if (_verboseOutput) std::cout<<"read/writeUniform() ["<<id<<"]"<<std::endl;
+
+    }
+}
+
+void DataOutputStream::writeShader(const osg::Shader* shader)
+{
+    ShaderMap::iterator itr = _shaderMap.find(shader);
+    if (itr!=_shaderMap.end())
+    {
+        // Id already exists so just write ID.
+        writeInt(itr->second);
+
+        if (_verboseOutput) std::cout<<"read/writeShader() ["<<itr->second<<"]"<<std::endl;
+    }
+    else
+    {
+        // id doesn't exist so create a new ID and
+        // register the shader.
+
+        int id = _shaderMap.size();
+        _shaderMap[shader] = id;
+
+        // write the id.
+        writeInt(id);
+
+        // write the stateset.
+        ((ive::Shader*)(shader))->write(this);
+
+        if (_verboseOutput) std::cout<<"read/writeShader() ["<<id<<"]"<<std::endl;
+
+    }
+}
+
 void DataOutputStream::writeDrawable(const osg::Drawable* drawable)
 {
     DrawableMap::iterator itr = _drawableMap.find(drawable);
@@ -544,6 +688,8 @@ void DataOutputStream::writeDrawable(const osg::Drawable* drawable)
             ((ive::Geometry*)(drawable))->write(this);
         else if(dynamic_cast<const osg::ShapeDrawable*>(drawable))
             ((ive::ShapeDrawable*)(drawable))->write(this);
+        else if(dynamic_cast<const osgText::Text*>(drawable))
+            ((ive::Text*)(drawable))->write(this);
         else
         {
             throw Exception("Unknown drawable in DataOutputStream::writeDrawable()");
@@ -637,7 +783,7 @@ void DataOutputStream::writeNode(const osg::Node* node)
         else if(dynamic_cast<const osg::Sequence*>(node)){
             ((ive::Sequence*)(node))->write(this);
         }
-        else if(dynamic_cast<const osg::Impostor*>(node)){
+        else if(dynamic_cast<const osgSim::Impostor*>(node)){
             ((ive::Impostor*)(node))->write(this);
         }
         else if(dynamic_cast<const osg::PagedLOD*>(node)){
@@ -664,6 +810,12 @@ void DataOutputStream::writeNode(const osg::Node* node)
         else if(dynamic_cast<const osgSim::VisibilityGroup*>(node)){
             ((ive::VisibilityGroup*)(node))->write(this);
         }
+        else if(dynamic_cast<const osg::ProxyNode*>(node)){
+            ((ive::ProxyNode*)(node))->write(this);
+        }
+        else if(dynamic_cast<const osgFX::MultiTextureControl*>(node)){
+            ((ive::MultiTextureControl*)(node))->write(this);
+        }
         else if(dynamic_cast<const osg::Group*>(node)){
             ((ive::Group*)(node))->write(this);
         }
