@@ -64,7 +64,7 @@
 #include <sstream>
 #include <string>
 
-#if defined(__APPLE__) 
+#if (defined(__APPLE__)&&(__GNUC__<4)) || (defined(WIN32)&&!defined(__CYGWIN__)) || defined (__sgi) || defined (__hpux) 
 typedef int socklen_t;
 #endif
 
@@ -77,8 +77,9 @@ extern "C" {
 #	include <errno.h>
 }
 #else
-#	define socklen_t int
-
+#   if (_MSC_VER >= 1400)
+#      include <errno.h>
+#   endif
 #	define EWOULDBLOCK			WSAEWOULDBLOCK
 #	define EINPROGRESS			WSAEINPROGRESS
 #	define EALREADY					WSAEALREADY
@@ -129,9 +130,6 @@ extern "C" {
 #include <sys/filio.h>
 #endif
 
-#ifdef __sgi
-#define socklen_t int
-#endif
 
 #ifndef BUFSIZ
 #  define BUFSIZ 1024
@@ -205,7 +203,7 @@ bool sockerr::op () const
   case EHOSTDOWN:
   case EHOSTUNREACH:
   case ENOTEMPTY:
-#	if !defined(__linux__) && !defined(__sun)// LN
+#	if !defined(__linux__) && !defined(__sun) && !defined(__hpux)// LN
   case EPROCLIM:
 #	endif
   case EUSERS:
@@ -505,9 +503,9 @@ void sockbuf::listen (int num)
 
 sockbuf::sockdesc sockbuf::accept (sockAddr& sa)
 {
-  int len = sa.size ();
+  socklen_t len = sa.size ();
   int soc = -1;
-  if ((soc = ::accept (rep->sock, sa.addr (), (socklen_t*) // LN
+  if ((soc = ::accept (rep->sock, sa.addr (), 
                        &len)) == -1)
     throw sockerr (errno, "sockbuf::sockdesc", sockname.c_str());
   return sockdesc (soc);
@@ -947,7 +945,7 @@ void sockbuf::closeonexec (bool set) const
 // if set is true, set close on exec flag
 // else clear close on exec flag
 {
-#ifndef __sgi
+#if !defined( __sgi) && !defined(__hpux)
   if (set) {
     if (::ioctl (rep->sock, FIOCLEX, 0) == -1)
       throw sockerr (errno, "sockbuf::closeonexec", sockname.c_str());
