@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -208,7 +208,20 @@ OsgCameraGroup::~OsgCameraGroup()
 
 void OsgCameraGroup::_init()
 {
-    _thread_model = ThreadPerCamera;
+    const char* str = getenv("OSG_CAMERA_THREADING");
+    if (str)
+    {
+        if (strcmp(str,"SingleThreaded")==0) _thread_model = SingleThreaded;
+        else if (strcmp(str,"ThreadPerCamera")==0) _thread_model = ThreadPerCamera;
+    }
+    
+    str = getenv("OSG_SHARE_GRAPHICS_CONTEXTS");
+    if (str)
+    {
+        if (strcmp(str,"ON")==0) Producer::RenderSurface::shareAllGLContexts(true);
+        else if (strcmp(str,"OFF")==0) Producer::RenderSurface::shareAllGLContexts(false);
+    }
+    
 
     _scene_data = NULL;
     _global_stateset = NULL;
@@ -448,7 +461,11 @@ bool OsgCameraGroup::realize()
         if (_renderSurfaceStateMap.count(rs)==0)
         {
             _renderSurfaceStateMap[rs] = sv->getState();
-            sv->getState()->setContextID(contextID++);
+            sv->getState()->setContextID(contextID);
+            
+            // if we aren't share OpenGL objects between graphics contexts then need to increment the contextID
+            // to ensure that the OSG generates seperate objects for each graphics context. 
+            if (!Producer::RenderSurface::allGLContextsAreShared()) ++contextID;
         }
         else
         {
@@ -627,3 +644,23 @@ void OsgCameraGroup::frame()
      CameraGroup::frame();
 }
 
+void OsgCameraGroup::cleanup_frame()
+{
+    // first relase all GL objects and switch on the flush of deleted objects
+    // in the next frame.
+    for(SceneHandlerList::iterator itr = _shvec.begin();
+        itr != _shvec.end();
+        ++itr)
+    {
+        (*itr)->getSceneView()->releaseAllGLObjects();
+        (*itr)->setFlushOfAllDeletedGLObjectsOnNextFrame(true);
+    }
+    
+    // make sure that the registry all flushes all its texture objects.
+    osgDB::Registry::instance()->releaseGLObjects();
+    
+    // then run the frame to do the actuall OpenGL clean up.
+    frame();
+}
+
+
