@@ -1,24 +1,35 @@
 #include <osg/LightSource>
 #include <osg/ApplicationUsage>
+#include <osg/AlphaFunc>
+#include <osg/io_utils>
 
 #include <osgUtil/UpdateVisitor>
 
 #include <osgDB/Registry>
 
+#include <osgGA/EventVisitor>
 #include <osgGA/AnimationPathManipulator>
 #include <osgGA/TrackballManipulator>
 #include <osgGA/FlightManipulator>
 #include <osgGA/DriveManipulator>
 #include <osgGA/TerrainManipulator>
+#include <osgGA/UFOManipulator>
 #include <osgGA/StateSetManipulator>
 
 #include <osgProducer/Viewer>
 #include <osgProducer/ViewerEventHandler>
 
+#include <stdio.h>
+
 using namespace Producer;
 using namespace osgProducer;
 using namespace osg;
 
+
+#ifdef __APPLE__
+#define SINGLE_THREAD_KEYBOARDMOUSE
+#endif
+
 //////////////////////////////////////////////////////////////////////////////
 //
 // Picking intersection visitor.
@@ -121,6 +132,13 @@ public:
         inverseMVPW.invert(projm);
         osg::Vec3 near_point = osg::Vec3(x,y,-1.0f)*inverseMVPW;
         osg::Vec3 far_point = osg::Vec3(x,y,1.0f)*inverseMVPW;
+        
+        if (!near_point.valid() || !far_point.valid())
+        {
+            osg::notify(osg::INFO)<<"Warning: PickVisitor caught invalid nea/far poiints."<<std::endl;
+            return _PIVsegHitList;
+        }
+        
         setxy(x,y);    
         getHits(node,near_point,far_point);
         return _PIVsegHitList;
@@ -194,11 +212,27 @@ public:
             if (csn)
             {
                 osg::Vec3 local_position = position*osg::computeWorldToLocal(tmpPath);
-                osg::notify(osg::INFO)<<"local postion "<<local_position<<std::endl;
-            
-                osg::notify(osg::INFO)<<"csn->computeLocalCoordinateFrame(position)* osg::computeLocalToWorld(tmpPath)"<<std::endl;
-
+                
+                // get the coordinate frame in world coords.
                 coordinateFrame = csn->computeLocalCoordinateFrame(local_position)* osg::computeLocalToWorld(tmpPath);
+
+                // keep the position of the coordinate frame to reapply after rescale.
+                osg::Vec3d pos = coordinateFrame.getTrans();
+
+                // compensate for any scaling, so that the coordinate frame is a unit size
+                osg::Vec3d x(1.0,0.0,0.0);
+                osg::Vec3d y(0.0,1.0,0.0);
+                osg::Vec3d z(0.0,0.0,1.0);
+                x = osg::Matrixd::transform3x3(x,coordinateFrame);
+                y = osg::Matrixd::transform3x3(y,coordinateFrame);
+                z = osg::Matrixd::transform3x3(z,coordinateFrame);
+                coordinateFrame.preMult(osg::Matrixd::scale(1.0/x.length(),1.0/y.length(),1.0/z.length()));
+
+                // reapply the position.
+                coordinateFrame.setTrans(pos);
+
+                osg::notify(osg::INFO)<<"csn->computeLocalCoordinateFrame(position)* osg::computeLocalToWorld(tmpPath)"<<coordinateFrame<<std::endl;
+
             }
             else
             {
@@ -301,6 +335,7 @@ Viewer::Viewer(osg::ArgumentParser& arguments):
         arguments.getApplicationUsage()->addCommandLineOption("-p <filename>","Specify camera path file to animate the camera through the loaded scene");
         arguments.getApplicationUsage()->addCommandLineOption("--run-till-frame-number <integer>","Specify the number of frame to run");
         arguments.getApplicationUsage()->addCommandLineOption("--run-till-elapsed-time","Specify the about of time to run");
+        arguments.getApplicationUsage()->addCommandLineOption("--clear-color <float>,<float>,<float>[,<float>]","Specify the clear color as RGB or RGBA");
     }
 
     osg::DisplaySettings::instance()->readCommandLine(arguments);
@@ -335,6 +370,16 @@ Viewer::Viewer(osg::ArgumentParser& arguments):
         setWriteImageWhenDone(true);
         setWriteImageFileName(filename);
     }
+
+    std::string colorStr;
+    while (arguments.read("--clear-color",colorStr))
+    {
+        float r, g, b;
+        float a = 1.0f;
+        int cnt = sscanf( colorStr.c_str(), "%f,%f,%f,%f", &r, &g, &b, &a );
+        if( cnt==3 || cnt==4 ) setClearColor( osg::Vec4(r,g,b,a) );
+        else osg::notify(osg::WARN)<<"Invalid clear color \""<<colorStr<<"\""<<std::endl;
+    }
 }
 
 Viewer::~Viewer()
@@ -482,10 +527,15 @@ void Viewer::setUpViewer(unsigned int options)
     
     _updateVisitor->setFrameStamp(_frameStamp.get());
 
+    if (!_eventVisitor) _eventVisitor = new osgGA::EventVisitor;
+    _eventVisitor->setActionAdapter(this);
+
+
     if (options&TRACKBALL_MANIPULATOR) addCameraManipulator(new osgGA::TrackballManipulator);
     if (options&FLIGHT_MANIPULATOR) addCameraManipulator(new osgGA::FlightManipulator);
     if (options&DRIVE_MANIPULATOR) addCameraManipulator(new osgGA::DriveManipulator);
     if (options&TERRAIN_MANIPULATOR) addCameraManipulator(new osgGA::TerrainManipulator);
+    if (options&UFO_MANIPULATOR) addCameraManipulator(new osgGA::UFOManipulator);
     
     if (options&STATE_MANIPULATOR)
     {
@@ -565,9 +615,10 @@ bool Viewer::realize()
 
     OsgCameraGroup::realize();
 
+#ifndef SINGLE_THREAD_KEYBOARDMOUSE
     // kick start the keyboard mouse if needed.
     if (_kbm.valid() && !_kbm->isRunning()) _kbm->startThread();
-
+#endif
     // by default set up the DatabasePager.
     {    
         osgDB::DatabasePager* databasePager = osgDB::Registry::instance()->getOrCreateDatabasePager();
@@ -623,6 +674,10 @@ bool Viewer::realize()
 
 void Viewer::update()
 {
+#ifdef SINGLE_THREAD_KEYBOARDMOUSE
+    if (_kbm.valid() && !_kbm->isRunning()) _kbm->update(*(_kbm->getCallback()));
+#endif
+
     // get the event since the last frame.
     osgProducer::KeyboardMouseCallback::EventQueue queue;
     if (_kbmcb.valid()) _kbmcb->getEventQueue(queue);
@@ -632,6 +687,11 @@ void Viewer::update()
     frame_event->adaptFrame(_frameStamp->getReferenceTime());
     queue.push_back(frame_event);
 
+    if (_eventVisitor.valid())
+    {
+        _eventVisitor->setTraversalNumber(_frameStamp->getFrameNumber());
+    }
+
     // dispatch the events in order of arrival.
     for(osgProducer::KeyboardMouseCallback::EventQueue::iterator event_itr=queue.begin();
         event_itr!=queue.end();
@@ -642,10 +702,17 @@ void Viewer::update()
             handler_itr!=_eventHandlerList.end() && !handled;
             ++handler_itr)
         {   
-            handled = (*handler_itr)->handle(*(*event_itr),*this);
+            handled = (*handler_itr)->handle(*(*event_itr),*this,0,0);
         }
+        if (_eventVisitor.valid())
+        {
+            _eventVisitor->reset();
+            _eventVisitor->addEvent(event_itr->get());
+            getTopMostSceneData()->accept(*_eventVisitor);
+        }
+        
     }
-    
+
     if (osgDB::Registry::instance()->getDatabasePager())
     {
         // update the scene graph by remove expired subgraphs and merge newly loaded subgraphs
@@ -661,6 +728,7 @@ void Viewer::update()
         getTopMostSceneData()->accept(*_updateVisitor);
     }
     
+
     // update the main producer camera
     if (_keyswitchManipulator.valid() && _keyswitchManipulator->getCurrentMatrixManipulator()) 
     {
@@ -836,7 +904,7 @@ bool Viewer::computeIntersections(float x,float y,unsigned int cameraNum,osg::No
 
 bool Viewer::computeIntersections(float x,float y,unsigned int cameraNum,osgUtil::IntersectVisitor::HitList& hits,osg::Node::NodeMask traversalMask)
 {
-    return computeIntersections(x,y,cameraNum,getSceneData(),hits,traversalMask);
+    return computeIntersections(x,y,cameraNum,getTopMostSceneData(),hits,traversalMask);
 }
 
 bool Viewer::computeIntersections(float x,float y,osg::Node *node,osgUtil::IntersectVisitor::HitList& hits,osg::Node::NodeMask traversalMask)
@@ -852,7 +920,7 @@ bool Viewer::computeIntersections(float x,float y,osg::Node *node,osgUtil::Inter
 
 bool Viewer::computeIntersections(float x,float y,osgUtil::IntersectVisitor::HitList& hits,osg::Node::NodeMask traversalMask)
 {
-    return computeIntersections(x,y,getSceneData(),hits,traversalMask);
+    return computeIntersections(x,y,getTopMostSceneData(),hits,traversalMask);
 }
 
 void Viewer::selectCameraManipulator(unsigned int no)
@@ -860,6 +928,55 @@ void Viewer::selectCameraManipulator(unsigned int no)
     if (_keyswitchManipulator.valid()) _keyswitchManipulator->selectMatrixManipulator(no);
 }
 
+void Viewer::getCameraManipulatorNameList( std::list<std::string> &nameList )
+{
+    osgGA::KeySwitchMatrixManipulator *ksm = getKeySwitchMatrixManipulator();
+    osgGA::KeySwitchMatrixManipulator::KeyManipMap &kmmap = ksm->getKeyManipMap();
+    osgGA::KeySwitchMatrixManipulator::KeyManipMap::iterator p;
+    for( p = kmmap.begin(); p != kmmap.end(); p++ )
+    {
+        osgGA::KeySwitchMatrixManipulator::NamedManipulator  nm = (*p).second;
+        nameList.push_back( nm.first );
+    }
+}
+
+bool Viewer::selectCameraManipulatorByName( const std::string &name )
+{
+    unsigned int num = 0xFFFF;
+    osgGA::KeySwitchMatrixManipulator *ksm = getKeySwitchMatrixManipulator();
+    osgGA::KeySwitchMatrixManipulator::KeyManipMap &kmmap = ksm->getKeyManipMap();
+    osgGA::KeySwitchMatrixManipulator::KeyManipMap::iterator p;
+    for( p = kmmap.begin(); p != kmmap.end(); p++ )
+    {
+        int key = (*p).first;
+        osgGA::KeySwitchMatrixManipulator::NamedManipulator  nm = (*p).second;
+        if( nm.first == name )
+            num = key - '1';
+    }
+
+    if( num == 0xFFFF )
+        return false;
+
+    selectCameraManipulator(num);
+    return true;
+}
+
+osgGA::MatrixManipulator *Viewer::getCameraManipulatorByName( const std::string &name )
+{
+    osgGA::KeySwitchMatrixManipulator *ksm = getKeySwitchMatrixManipulator();
+    osgGA::KeySwitchMatrixManipulator::KeyManipMap &kmmap = ksm->getKeyManipMap();
+    osgGA::KeySwitchMatrixManipulator::KeyManipMap::iterator p;
+    for( p = kmmap.begin(); p != kmmap.end(); p++ )
+    {
+        osgGA::KeySwitchMatrixManipulator::NamedManipulator  nm = (*p).second;
+        if( nm.first == name )
+            return nm.second.get();
+    }
+    return 0L;
+}
+
+
+
 void Viewer::requestRedraw()
 {
     //osg::notify(osg::INFO)<<"Viewer::requestRedraw() called"<<std::endl;
