@@ -768,6 +768,18 @@ void ViewerEventHandler::setWriteImageFileName(const std::string& filename)
     }
 }
 
+void ViewerEventHandler::setFrameStatsMode(FrameStatsMode mode)
+{ 
+    _frameStatsMode = mode; 
+    if (_frameStatsMode==NO_STATS)
+    {
+        _cg->setInstrumentationMode(false);
+    }
+    else
+    {
+        _cg->setInstrumentationMode(true);
+    }    
+}
 
 bool ViewerEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& aa)
 {
@@ -780,15 +792,8 @@ bool ViewerEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActio
         {
             case 's' :
             {
-                _frameStatsMode = (FrameStatsMode)((_frameStatsMode+1)%3);
-                if (_frameStatsMode==NO_STATS)
-                {
-                    _cg->setInstrumentationMode(false);
-                }
-                else
-                {
-                    _cg->setInstrumentationMode(true);
-                }
+                FrameStatsMode newFrameStatsMode = (FrameStatsMode)((_frameStatsMode+1)%3);
+                setFrameStatsMode(newFrameStatsMode);
                 return true;
             }
             case 'v' :
@@ -807,12 +812,19 @@ bool ViewerEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActio
                 Producer::CameraConfig* cfg = _cg->getCameraConfig();
 
                 bool shouldBeFullScreen = false;
+                bool flag = true;
                 for( unsigned int i = 0; i < cfg->getNumberOfCameras(); ++i )
                 {
                     Producer::Camera *cam = cfg->getCamera(i);
                     Producer::RenderSurface* rs = cam->getRenderSurface();
+                    if( rs->getDrawableType() != Producer::RenderSurface::DrawableType_Window )
+                        continue;
                   
-                    if(i==0) shouldBeFullScreen =! rs->isFullScreen();    // Remember the initial state of the first render surface
+                    if( flag )
+                    {
+                        shouldBeFullScreen =! rs->isFullScreen();    // Remember the initial state of the first render surface
+                        flag = false;
+                    }
 
                     if ( shouldBeFullScreen!=rs->isFullScreen() )        // If the current render surface hasn't been modified already
                     {
