@@ -3,6 +3,8 @@
 #include <osg/CullFace>
 #include <osg/LineWidth>
 
+#include <algorithm>
+
 using namespace osgSim;
 
 // Define the collection of nested classes, all Drawables, which make
@@ -33,9 +35,9 @@ public:
 
     void drawImplementation(osg::State& state) const;
 
-protected:
+    virtual osg::BoundingBox computeBound() const;
 
-    virtual bool computeBound() const;
+protected:
 
 private:
 
@@ -47,10 +49,11 @@ void SphereSegment::Surface::drawImplementation(osg::State& state) const
     _ss->Surface_drawImplementation(state);
 }
 
-bool SphereSegment::Surface::computeBound() const
+osg:: BoundingBox SphereSegment::Surface::computeBound() const
 {
-    _bbox_computed = _ss->Surface_computeBound(_bbox);
-    return _bbox_computed;
+    osg:: BoundingBox bbox;
+    _ss->Surface_computeBound(bbox);
+    return bbox;
 }
 
 
@@ -91,7 +94,7 @@ protected:
     }
 
 
-    virtual bool computeBound() const;
+    virtual osg::BoundingBox computeBound() const;
 
 private:
 
@@ -103,10 +106,11 @@ void SphereSegment::EdgeLine::drawImplementation(osg::State& state) const
     _ss->EdgeLine_drawImplementation(state);
 }
 
-bool SphereSegment::EdgeLine::computeBound() const
+osg::BoundingBox SphereSegment::EdgeLine::computeBound() const
 {
-    _bbox_computed = _ss->EdgeLine_computeBound(_bbox);
-    return _bbox_computed;
+    osg::BoundingBox bbox;
+    _ss->EdgeLine_computeBound(bbox);
+    return bbox;
 }
 
 
@@ -139,7 +143,7 @@ public:
 
 protected:
 
-    virtual bool computeBound() const;
+    virtual osg::BoundingBox computeBound() const;
 
 private:
     SphereSegment* _ss;
@@ -153,10 +157,11 @@ void SphereSegment::Side::drawImplementation(osg::State& state) const
     _ss->Side_drawImplementation(state, _planeOrientation, _BoundaryAngle);
 }
 
-bool SphereSegment::Side::computeBound() const
+osg::BoundingBox SphereSegment::Side::computeBound() const
 {
-    _bbox_computed = _ss->Side_computeBound(_bbox, _planeOrientation, _BoundaryAngle);
-    return _bbox_computed;
+    osg::BoundingBox bbox;
+    _ss->Side_computeBound(bbox, _planeOrientation, _BoundaryAngle);
+    return bbox;
 }
 
 
@@ -197,7 +202,7 @@ protected:
 	//getOrCreateStateSet()->setAttributeAndModes(new osg::LineWidth(2.0),osg::StateAttribute::OFF);
     }
     
-    virtual bool computeBound() const;
+    virtual osg::BoundingBox computeBound() const;
 
 private:
     SphereSegment* _ss;
@@ -209,10 +214,11 @@ void SphereSegment::Spoke::drawImplementation(osg::State& state) const
     _ss->Spoke_drawImplementation(state, _azAngle, _elevAngle);
 }
 
-bool SphereSegment::Spoke::computeBound() const
+osg::BoundingBox SphereSegment::Spoke::computeBound() const
 {
-    _bbox_computed = _ss->Spoke_computeBound(_bbox, _azAngle, _elevAngle);
-    return _bbox_computed;
+    osg::BoundingBox bbox;
+    _ss->Spoke_computeBound(bbox, _azAngle, _elevAngle);
+    return bbox;
 }
 
 SphereSegment::SphereSegment(const osg::Vec3& centre, float radius, const osg::Vec3& vec, float azRange,
@@ -343,40 +349,24 @@ void SphereSegment::init()
     addDrawable(new Spoke(this,MAX,MAX));
 }
 
-namespace
-{
-
-struct DirtyDisplayList
-{
-    void operator()(osg::ref_ptr<osg::Drawable>& dptr)
-    {
-        dptr->dirtyDisplayList();
-    }
-};
-
-}
-
 void SphereSegment::dirtyAllDrawableDisplayLists()
 {
-    std::for_each(_drawables.begin(), _drawables.end(), DirtyDisplayList());
-}
-
-namespace
-{
-
-struct DirtyBound
-{
-    void operator()(osg::ref_ptr<osg::Drawable>& dptr)
+    for(DrawableList::iterator itr = _drawables.begin();
+        itr != _drawables.end();
+        ++itr)
     {
-        dptr->dirtyBound();
+        (*itr)->dirtyDisplayList();
     }
-};
-
 }
 
 void SphereSegment::dirtyAllDrawableBounds()
 {
-    std::for_each(_drawables.begin(), _drawables.end(), DirtyBound());
+    for(DrawableList::iterator itr = _drawables.begin();
+        itr != _drawables.end();
+        ++itr)
+    {
+        (*itr)->dirtyBound();
+    }
 }
 
 void SphereSegment::Surface_drawImplementation(osg::State& /* state */) const
@@ -892,8 +882,6 @@ void SphereSegment::setDrawMask(DrawMask dm)
     dirtyBound();
 }
 
-namespace{
-
 struct ActivateTransparencyOnType
 {
     ActivateTransparencyOnType(const std::type_info& t): _t(t) {}
@@ -933,8 +921,6 @@ struct DeactivateTransparencyOnType
     const std::type_info&  _t;
 };
 
-}
-
 void SphereSegment::setSurfaceColor(const osg::Vec4& c)
 {
     _surfaceColor=c;
