@@ -1,5 +1,5 @@
-
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+;
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -86,8 +86,8 @@ Font::Font(FontImplementation* implementation):
     _width(16),
     _height(16),
     _margin(2),
-    _textureWidthHint(256),
-    _textureHeightHint(256),
+    _textureWidthHint(512),
+    _textureHeightHint(512),
     _minFilterHint(osg::Texture::LINEAR_MIPMAP_LINEAR),
     _magFilterHint(osg::Texture::LINEAR)
 {
@@ -123,17 +123,17 @@ std::string Font::getFileName() const
     return "";
 }
 
-void Font::setSize(unsigned int width, unsigned int height)
+void Font::setFontResolution(unsigned int width, unsigned int height)
 {
-    if (_implementation.valid()) _implementation->setSize(width, height);
+    if (_implementation.valid()) _implementation->setFontResolution(width, height);
 }
 
-unsigned int Font::getWidth() const
+unsigned int Font::getFontWidth() const
 {
     return _width;
 }
 
-unsigned int Font::getHeight() const
+unsigned int Font::getFontHeight() const
 {
     return _height;
 }
@@ -202,6 +202,16 @@ Font::Glyph* Font::getGlyph(unsigned int charcode)
     else return 0;
 }
 
+void Font::releaseGLObjects(osg::State* state) const
+{
+  for(GlyphTextureList::const_iterator itr=_glyphTextureList.begin();
+    itr!=_glyphTextureList.end();
+    ++itr)
+  {
+    (*itr)->releaseGLObjects(state);
+  }
+}
+
 osg::Vec2 Font::getKerning(unsigned int leftcharcode,unsigned int rightcharcode, KerningType kerningType)
 {
     if (_implementation.valid()) return _implementation->getKerning(leftcharcode,rightcharcode,kerningType);
@@ -247,11 +257,11 @@ void Font::addGlyph(unsigned int width, unsigned int height, unsigned int charco
 
         glyphTexture = new GlyphTexture;
         
-        //std::cout<<"    Creating new GlyphTexture "<<glyphTexture<<"& StateSet "<<stateset<<std::endl;
-
         static int numberOfTexturesAllocated = 0;
         ++numberOfTexturesAllocated;
-        //std::cout << "    " << this<< "  numberOfTexturesAllocated "<<numberOfTexturesAllocated<<std::endl;
+
+        osg::notify(osg::INFO)<<"    Creating new GlyphTexture "<<glyphTexture<<"& StateSet "<<stateset<<std::endl;
+        osg::notify(osg::INFO)<< "   Font " << this<< ", numberOfTexturesAllocated "<<numberOfTexturesAllocated<<std::endl;
 
         // reserve enough space for the glyphs.
         glyphTexture->setGlyphImageMargin(_margin);
