@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -17,16 +17,17 @@
 
 #include <osg/Math>
 #include <osg/GL>
+#include <osg/Notify>
 #include <osgUtil/CullVisitor>
 #include <osgDB/ReadFile>
 
 #include "DefaultFont.h"
 
+using namespace osg;
 using namespace osgText;
 
 //#define TREES_CODE_FOR_MAKING_SPACES_EDITABLE
 
-
 Text::Text():
     _fontWidth(32),
     _fontHeight(32),
@@ -221,25 +222,31 @@ void Text::setDrawMode(unsigned int mode)
 }
 
 
-bool Text::computeBound() const
+osg::BoundingBox Text::computeBound() const
 {
-    _bbox.init();
+    osg::BoundingBox  bbox;
 
     if (_textBB.valid())
     {
     
         for(unsigned int i=0;i<_autoTransformCache.size();++i)
         {
-            osg::Matrix& matrix = _autoTransformCache[i]._matrix;
-            _bbox.expandBy(osg::Vec3(_textBB.xMin(),_textBB.yMin(),_textBB.zMin())*matrix);
-            _bbox.expandBy(osg::Vec3(_textBB.xMax(),_textBB.yMin(),_textBB.zMin())*matrix);
-            _bbox.expandBy(osg::Vec3(_textBB.xMax(),_textBB.yMax(),_textBB.zMin())*matrix);
-            _bbox.expandBy(osg::Vec3(_textBB.xMin(),_textBB.yMax(),_textBB.zMin())*matrix);
+            if (_autoTransformCache[i]._traversalNumber<0 && (_characterSizeMode!=OBJECT_COORDS || _autoRotateToScreen))
+            {
+                // _autoTransformCache is not valid so don't take it into accoumt when compute bounding volume.
+            }
+            else
+            {            
+                osg::Matrix& matrix = _autoTransformCache[i]._matrix;
+                bbox.expandBy(osg::Vec3(_textBB.xMin(),_textBB.yMin(),_textBB.zMin())*matrix);
+                bbox.expandBy(osg::Vec3(_textBB.xMax(),_textBB.yMin(),_textBB.zMin())*matrix);
+                bbox.expandBy(osg::Vec3(_textBB.xMax(),_textBB.yMax(),_textBB.zMin())*matrix);
+                bbox.expandBy(osg::Vec3(_textBB.xMin(),_textBB.yMax(),_textBB.zMin())*matrix);
+            }
         }
     }
     
-    _bbox_computed = true;
-    return true;
+    return bbox;
 }
 
 Font* Text::getActiveFont()
@@ -257,10 +264,9 @@ String::iterator Text::computeLastCharacterOnLine(osg::Vec2 cursor, String::iter
     Font* activefont = getActiveFont();
     if (!activefont) return last;
 
-    float hr = _characterHeight/(float)activefont->getHeight();
+    float hr = _characterHeight/(float)activefont->getFontHeight();
     float wr = hr/_characterAspectRatio;
 
-    bool horizontal = _layout!=VERTICAL;
     bool kerning = true;
     unsigned int previous_charcode = 0;
 
@@ -310,11 +316,6 @@ String::iterator Text::computeLastCharacterOnLine(osg::Vec2 cursor, String::iter
                     break; // no kerning when vertical.
                 }
             }        
-
-            osg::Vec2 bearing(horizontal?glyph->getHorizontalBearing():glyph->getVerticalBearing());
-            cursor.x() += bearing.x() * wr;
-            cursor.y() += bearing.y() * hr;
-
             // check to see if we are still within line if not move to next line.
             switch(_layout)
             {
@@ -370,9 +371,9 @@ void Text::computeGlyphRepresentation()
     bool horizontal = _layout!=VERTICAL;
     bool kerning = true;
 
-    activefont->setSize(_fontWidth,_fontHeight);
+    activefont->setFontResolution(_fontWidth,_fontHeight);
     
-    float hr = _characterHeight/(float)activefont->getHeight();
+    float hr = _characterHeight/(float)activefont->getFontHeight();
     float wr = hr/_characterAspectRatio;
 
     std::set<unsigned int> deliminatorSet;
@@ -385,6 +386,8 @@ void Text::computeGlyphRepresentation()
     deliminatorSet.insert(':');
     deliminatorSet.insert('.');
 
+    unsigned int lineNumber = 0;
+
     for(String::iterator itr=_text.begin();
         itr!=_text.end();
         )
@@ -468,6 +471,7 @@ void Text::computeGlyphRepresentation()
                     GlyphQuads& glyphquad = _textureGlyphQuadMap[glyph->getTexture()->getStateSet()];
 
                     glyphquad._glyphs.push_back(glyph);
+                    glyphquad._lineNumbers.push_back(lineNumber);
 
                     // set up the coords of the quad
                     glyphquad._coords.push_back(local+osg::Vec2(0.0f,height));
@@ -533,17 +537,18 @@ void Text::computeGlyphRepresentation()
           }
           break;
         }
+        
+        ++lineNumber;
 
     }
 
+    // compute the bounding box
     _textBB.init();
-
     for(TextureGlyphQuadMap::const_iterator titr=_textureGlyphQuadMap.begin();
         titr!=_textureGlyphQuadMap.end();
         ++titr)
     {
         const GlyphQuads& glyphquad = titr->second;
-        
         for(GlyphQuads::Coords2::const_iterator citr = glyphquad._coords.begin();
             citr != glyphquad._coords.end();
             ++citr)
@@ -552,11 +557,127 @@ void Text::computeGlyphRepresentation()
         }
     }
 
-    if (!_textureGlyphQuadMap.empty()) 
+    if (lineNumber>1)
     {
-        setStateSet(const_cast<osg::StateSet*>((*_textureGlyphQuadMap.begin()).first.get()));
+        // account for line justification    
+        typedef std::vector<osg::Vec2> LineDimensions;
+        LineDimensions minLineCoords(lineNumber, osg::Vec2(FLT_MAX,FLT_MAX));
+        LineDimensions maxLineCoords(lineNumber, osg::Vec2(-FLT_MAX,-FLT_MAX));
+
+        // osg::notify(osg::NOTICE)<<"lineNumber="<<lineNumber<<std::endl;
+
+        TextureGlyphQuadMap::iterator titr;
+        for(titr=_textureGlyphQuadMap.begin();
+            titr!=_textureGlyphQuadMap.end();
+            ++titr)
+        {
+            const GlyphQuads& glyphquad = titr->second;
+            const GlyphQuads::Coords2& coords = glyphquad._coords;
+
+            unsigned int coordIndex = 0;
+            for(GlyphQuads::LineNumbers::const_iterator litr = glyphquad._lineNumbers.begin();
+                litr != glyphquad._lineNumbers.end();
+                ++litr)
+            {
+                unsigned int line = *litr;
+                osg::Vec2& minLineCoord = minLineCoords[line];
+                osg::Vec2& maxLineCoord = maxLineCoords[line];
+                for(unsigned int ci=0;ci<4;++ci)
+                {
+                    const osg::Vec2& coord = coords[coordIndex++];
+                    if (coord.x()<minLineCoord.x()) minLineCoord.x()=coord.x();
+                    if (coord.y()<minLineCoord.y()) minLineCoord.y()=coord.y();
+                    if (coord.x()>maxLineCoord.x()) maxLineCoord.x()=coord.x();
+                    if (coord.y()>maxLineCoord.y()) maxLineCoord.y()=coord.y();
+                }
+            }
+        }
+        
+        // osg::notify(osg::NOTICE)<<"Text dimensions min="<<_textBB.xMin()<<" "<<_textBB.yMin()<<"  max="<<_textBB.xMax()<<" "<<_textBB.yMax()<<std::endl;
+        typedef std::vector<osg::Vec2> LineOffsets;
+        LineOffsets lineOffsets(lineNumber,osg::Vec2(0.0f,0.0f));
+        for(unsigned int li=0;li<lineNumber;++li)
+        {
+            osg::Vec2& minCoord = minLineCoords[li];
+            osg::Vec2& maxCoord = maxLineCoords[li];
+            osg::Vec2& lineOffset = lineOffsets[li];
+            if (horizontal)
+            {
+                switch(_alignment)
+                {
+                case LEFT_BASE_LINE:
+                case LEFT_TOP:
+                case LEFT_CENTER:
+                case LEFT_BOTTOM: lineOffset.x() = _textBB.xMin() - minCoord.x(); break;
+
+                case CENTER_BASE_LINE:
+                case CENTER_TOP:
+                case CENTER_CENTER:
+                case CENTER_BOTTOM: lineOffset.x() = (_textBB.xMin()-minCoord.x()+_textBB.xMax()-maxCoord.x())*0.5f; break;
+
+                case RIGHT_BASE_LINE:
+                case RIGHT_TOP:
+                case RIGHT_CENTER:
+                case RIGHT_BOTTOM: lineOffset.x() = _textBB.xMax() - maxCoord.x(); break;
+                }
+            }
+            else
+            {
+                switch(_alignment)
+                {
+                case LEFT_TOP: 
+                case CENTER_TOP:
+                case RIGHT_TOP: lineOffset.y() = _textBB.yMax() - maxCoord.y(); break;
+
+                case LEFT_CENTER:
+                case CENTER_CENTER:
+                case RIGHT_CENTER: lineOffset.y() = (_textBB.yMin()-minCoord.y()+_textBB.yMax()-maxCoord.y())*0.5f; break;
+
+                case LEFT_BASE_LINE:
+                case CENTER_BASE_LINE:
+                case RIGHT_BASE_LINE: break;
+
+                case LEFT_BOTTOM: 
+                case CENTER_BOTTOM:
+                case RIGHT_BOTTOM: lineOffset.y() = _textBB.yMin() - minCoord.y(); break;
+                }
+            }
+            // osg::notify(osg::NOTICE)<<"  mincoords = "<<minLineCoords[li]<<" max = "<<maxLineCoords[li]<<std::endl;
+            // osg::notify(osg::NOTICE)<<"  offset = "<<lineOffset<<std::endl;
+        }
+
+        // shift lines
+        for(titr=_textureGlyphQuadMap.begin();
+            titr!=_textureGlyphQuadMap.end();
+            ++titr)
+        {
+            GlyphQuads& glyphquad = titr->second;
+            GlyphQuads::Coords2& coords = glyphquad._coords;
+
+            unsigned int coordIndex = 0;
+            for(GlyphQuads::LineNumbers::iterator litr = glyphquad._lineNumbers.begin();
+                litr != glyphquad._lineNumbers.end();
+                ++litr)
+            {
+                unsigned int line = *litr;
+                osg::Vec2& lineOffset = lineOffsets[line];
+                if (lineOffset.x()!=0.0f || lineOffset.y()!=0.0f)
+                {
+                    for(unsigned int ci=0;ci<4;++ci)
+                    {
+                        coords[coordIndex++] += lineOffset;
+                    }
+                }
+                else
+                {
+                    coordIndex += 4;
+                }
+            }
+        }
     }
 
+    setStateSet(const_cast<osg::StateSet*>((*_textureGlyphQuadMap.begin()).first.get()));
+
     computePositions();
 }
 
@@ -643,21 +764,27 @@ void Text::computePositions(unsigned int contextID) const
 
             float pixelSizeVector_w = M(3,2)*P23 + M(3,3)*P33;
 
-            float pixelSize = (_characterHeight*sqrtf(scale_00.length2()+scale_10.length2()))/(pixelSizeVector_w*0.701f);
+            float pixelSizeVert=(_characterHeight*sqrtf(scale_10.length2()))/(pixelSizeVector_w*0.701f);
+            float pixelSizeHori=(_characterHeight/_characterAspectRatio*sqrtf(scale_00.length2()))/(pixelSizeVector_w*0.701f);
 
             // avoid nasty math by preventing a divide by zero
-            if(pixelSize == 0.0f)
-            {
-                pixelSize = 1.0f;
-            }
+            if (pixelSizeVert == 0.0f)
+               pixelSizeVert= 1.0f;
+            if (pixelSizeHori == 0.0f)
+               pixelSizeHori= 1.0f;
+
             if (_characterSizeMode==SCREEN_COORDS)
             {
-                float scale_font = _characterHeight/pixelSize;
-                matrix.postMult(osg::Matrix::scale(scale_font, scale_font,1.0f));
+                float scale_font_vert=_characterHeight/pixelSizeVert;
+                float scale_font_hori=_characterHeight/_characterAspectRatio/pixelSizeHori;
+
+                if (P10<0)
+                   scale_font_vert=-scale_font_vert;
+                matrix.postMult(osg::Matrix::scale(scale_font_hori, scale_font_vert,1.0f));
             }
-            else if (pixelSize>_fontHeight)
+            else if (pixelSizeVert>_fontHeight)
             {
-                float scale_font = _fontHeight/pixelSize;
+                float scale_font = _fontHeight/pixelSizeVert;
                 matrix.postMult(osg::Matrix::scale(scale_font, scale_font,1.0f));
             }
 
@@ -866,7 +993,7 @@ void Text::accept(osg::Drawable::ConstAttributeFunctor& af) const
     }
 }
 
-void Text::accept(osg::Drawable::PrimitiveFunctor& pf) const
+void Text::accept(osg::PrimitiveFunctor& pf) const
 {
     for(TextureGlyphQuadMap::const_iterator titr=_textureGlyphQuadMap.begin();
         titr!=_textureGlyphQuadMap.end();
@@ -880,3 +1007,12 @@ void Text::accept(osg::Drawable::PrimitiveFunctor& pf) const
     }
     
 }
+
+/** If State is non-zero, this function releases OpenGL objects for
+  * the specified graphics context. Otherwise, releases OpenGL objexts
+  * for all graphics contexts. */
+void Text::releaseGLObjects(osg::State* state) const
+{
+    Drawable::releaseGLObjects(state);
+    if (_font.valid()) _font->releaseGLObjects(state);
+}
