@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -17,7 +17,6 @@
 #include <osg/MatrixTransform>
 #include <osg/PositionAttitudeTransform>
 #include <osg/LOD>
-#include <osg/Impostor>
 #include <osg/Billboard>
 #include <osg/Geometry>
 #include <osg/Notify>
@@ -25,6 +24,8 @@
 #include <osg/Sequence>
 #include <osg/Switch>
 #include <osg/Texture>
+#include <osg/PagedLOD>
+#include <osg/ProxyNode>
 
 #include <osgUtil/TransformAttributeFunctor>
 #include <osgUtil/TriStripVisitor>
@@ -37,27 +38,12 @@
 using namespace osgUtil;
 
 
-////////////////////////////////////////////////////////////////////////////
-// Overall Optimization function.
-////////////////////////////////////////////////////////////////////////////
-
-static bool isNodeEmpty(const osg::Node& node)
-{
-    if (node.getUserData()) return false;
-    if (node.getUpdateCallback()) return false;
-    if (node.getCullCallback()) return false;
-    if (node.getNumDescriptions()>0) return false;
-    if (node.getStateSet()) return false;
-    if (node.getNodeMask()!=0xffffffff) return false;
-    return true;
-}
 
 void Optimizer::reset()
 {
-    _permissibleOptimizationsMap.clear();
 }
 
-static osg::ApplicationUsageProxy Optimizer_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_OPTIMIZER \"<type> [<type>]\"","OFF | DEFAULT | FLATTEN_STATIC_TRANSFORMS | REMOVE_REDUNDANT_NODES | COMBINE_ADJACENT_LODS | SHARE_DUPLICATE_STATE | MERGE_GEOMETRY | SPATIALIZE_GROUPS  | COPY_SHARED_NODES  | TRISTRIP_GEOMETRY | OPTIMIZE_TEXTURE_SETTINGS");
+static osg::ApplicationUsageProxy Optimizer_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_OPTIMIZER \"<type> [<type>]\"","OFF | DEFAULT | FLATTEN_STATIC_TRANSFORMS | REMOVE_REDUNDANT_NODES | COMBINE_ADJACENT_LODS | SHARE_DUPLICATE_STATE | MERGE_GEOMETRY | SPATIALIZE_GROUPS  | COPY_SHARED_NODES  | TRISTRIP_GEOMETRY | OPTIMIZE_TEXTURE_SETTINGS | REMOVE_LOADED_PROXY_NODES");
 
 void Optimizer::optimize(osg::Node* node)
 {
@@ -80,6 +66,9 @@ void Optimizer::optimize(osg::Node* node)
         if(str.find("~REMOVE_REDUNDANT_NODES")!=std::string::npos) options ^= REMOVE_REDUNDANT_NODES;
         else if(str.find("REMOVE_REDUNDANT_NODES")!=std::string::npos) options |= REMOVE_REDUNDANT_NODES;
 
+        if(str.find("~REMOVE_LOADED_PROXY_NODES")!=std::string::npos) options ^= REMOVE_LOADED_PROXY_NODES;
+        else if(str.find("REMOVE_LOADED_PROXY_NODES")!=std::string::npos) options |= REMOVE_LOADED_PROXY_NODES;
+
         if(str.find("~COMBINE_ADJACENT_LODS")!=std::string::npos) options ^= COMBINE_ADJACENT_LODS;
         else if(str.find("COMBINE_ADJACENT_LODS")!=std::string::npos) options |= COMBINE_ADJACENT_LODS;
 
@@ -126,6 +115,16 @@ void Optimizer::optimize(osg::Node* node, unsigned int options)
         node->accept(tsv);        
     }
     
+    if (options & REMOVE_LOADED_PROXY_NODES)
+    {
+        osg::notify(osg::INFO)<<"Optimizer::optimize() doing REMOVE_LOADED_PROXY_NODES"<<std::endl;
+
+        RemoveLoadedProxyNodesVisitor rlpnv(this);
+        node->accept(rlpnv);
+        rlpnv.removeRedundantNodes();
+
+    }
+
     if (options & COMBINE_ADJACENT_LODS)
     {
         osg::notify(osg::INFO)<<"Optimizer::optimize() doing COMBINE_ADJACENT_LODS"<<std::endl;
@@ -201,14 +200,6 @@ void Optimizer::optimize(osg::Node* node, unsigned int options)
 
     }
     
-    if (options & SPATIALIZE_GROUPS)
-    {
-        osg::notify(osg::INFO)<<"Optimizer::optimize() doing SPATIALIZE_GROUPS"<<std::endl;
-
-        SpatializeGroupsVisitor sv(this);
-        node->accept(sv);
-        sv.divide();
-    }
 
     if (options & CHECK_GEOMETRY)
     {
@@ -234,6 +225,15 @@ void Optimizer::optimize(osg::Node* node, unsigned int options)
         node->accept(tsv);
         tsv.stripify();
     }
+
+    if (options & SPATIALIZE_GROUPS)
+    {
+        osg::notify(osg::INFO)<<"Optimizer::optimize() doing SPATIALIZE_GROUPS"<<std::endl;
+
+        SpatializeGroupsVisitor sv(this);
+        node->accept(sv);
+        sv.divide();
+    }
     
 }
 
@@ -259,9 +259,10 @@ void Optimizer::TesselateVisitor::apply(osg::Geode& geode)
 // Optimize State Visitor
 ////////////////////////////////////////////////////////////////////////////
 
-struct LessAttributeFunctor
+template<typename T>
+struct LessDerefFunctor 
 {
-    bool operator () (const osg::StateAttribute* lhs,const osg::StateAttribute* rhs) const
+    bool operator () (const T* lhs,const T* rhs) const
     {
         return (*lhs<*rhs); 
     }
@@ -344,12 +345,17 @@ void Optimizer::StateVisitor::optimize()
         typedef std::pair<osg::StateSet*,unsigned int>      StateSetUnitPair;
         typedef std::set<StateSetUnitPair>                  StateSetList;
         typedef std::map<osg::StateAttribute*,StateSetList> AttributeToStateSetMap;
+        AttributeToStateSetMap attributeToStateSetMap;
+
+        // create map from uniforms to stateset when contain them.
+        typedef std::set<osg::StateSet*>                    StateSetSet;
+        typedef std::map<osg::Uniform*,StateSetSet>         UniformToStateSetMap;
         
         const unsigned int NON_TEXTURE_ATTRIBUTE = 0xffffffff;
         
-        AttributeToStateSetMap _attributeToStateSetMap;
+        UniformToStateSetMap  uniformToStateSetMap;
 
-        // NOTE will need to track state attribute override value too.
+        // NOTE - TODO will need to track state attribute override value too.
 
         for(StateSetMap::iterator sitr=_statesets.begin();
             sitr!=_statesets.end();
@@ -362,7 +368,7 @@ void Optimizer::StateVisitor::optimize()
             {
                 if (aitr->second.first->getDataVariance()==osg::Object::STATIC)
                 {
-                    _attributeToStateSetMap[aitr->second.first.get()].insert(StateSetUnitPair(sitr->first,NON_TEXTURE_ATTRIBUTE));
+                    attributeToStateSetMap[aitr->second.first.get()].insert(StateSetUnitPair(sitr->first,NON_TEXTURE_ATTRIBUTE));
                 }
             }
 
@@ -377,92 +383,152 @@ void Optimizer::StateVisitor::optimize()
                 {
                     if (aitr->second.first->getDataVariance()==osg::Object::STATIC)
                     {
-                        _attributeToStateSetMap[aitr->second.first.get()].insert(StateSetUnitPair(sitr->first,unit));
+                        attributeToStateSetMap[aitr->second.first.get()].insert(StateSetUnitPair(sitr->first,unit));
                     }
                 }
             }
 
-        }
 
-        if (_attributeToStateSetMap.size()<2)
-        {
-            osg::notify(osg::INFO) << "Too few state attributes to optimize."<< std::endl;
-            return;
-        }
-
-        // create unique set of state attribute pointers.
-        typedef std::vector<osg::StateAttribute*> AttributeList;
-        AttributeList _attributeList;
-
-        for(AttributeToStateSetMap::iterator aitr=_attributeToStateSetMap.begin();
-            aitr!=_attributeToStateSetMap.end();
-            ++aitr)
-        {
-            _attributeList.push_back(aitr->first);
-        }
-
-        // sort the attributes so that equal attributes sit along side each
-        // other.
-        std::sort(_attributeList.begin(),_attributeList.end(),LessAttributeFunctor());
-
-
-        osg::notify(osg::INFO) << "state attribute list"<< std::endl;
-        for(AttributeList::iterator aaitr = _attributeList.begin();
-            aaitr!=_attributeList.end();
-            ++aaitr)
-        {
-            osg::notify(osg::INFO) << "    "<<*aaitr << "  "<<(*aaitr)->className()<< std::endl;
-        }
-
-        osg::notify(osg::INFO) << "searching for duplicate attributes"<< std::endl;
-        // find the duplicates.
-        AttributeList::iterator first_unique = _attributeList.begin();
-        AttributeList::iterator current = first_unique; ++current;
-        for(; current!=_attributeList.end();++current)
-        {
-            if (**current==**first_unique)
+            osg::StateSet::UniformList& uniforms = sitr->first->getUniformList();
+            for(osg::StateSet::UniformList::iterator uitr= uniforms.begin();
+                uitr!=uniforms.end();
+                ++uitr)
             {
-                osg::notify(osg::INFO) << "    found duplicate "<<(*current)->className()<<"  first="<<*first_unique<<"  current="<<*current<< std::endl;
-                StateSetList& statesetlist = _attributeToStateSetMap[*current];
-                for(StateSetList::iterator sitr=statesetlist.begin();
-                    sitr!=statesetlist.end();
-                    ++sitr)
+                if (uitr->second.first->getDataVariance()==osg::Object::STATIC)
                 {
-                    osg::notify(osg::INFO) << "       replace duplicate "<<*current<<" with "<<*first_unique<< std::endl;
-                    osg::StateSet* stateset = sitr->first;
-                    unsigned int unit = sitr->second;
-                    if (unit==NON_TEXTURE_ATTRIBUTE) stateset->setAttribute(*first_unique);
-                    else stateset->setTextureAttribute(unit,*first_unique);
+                    uniformToStateSetMap[uitr->second.first.get()].insert(sitr->first);
                 }
             }
-            else first_unique = current;
+
+        }
+
+        if (attributeToStateSetMap.size()>=2)
+        {
+            // create unique set of state attribute pointers.
+            typedef std::vector<osg::StateAttribute*> AttributeList;
+            AttributeList attributeList;
+
+            for(AttributeToStateSetMap::iterator aitr=attributeToStateSetMap.begin();
+                aitr!=attributeToStateSetMap.end();
+                ++aitr)
+            {
+                attributeList.push_back(aitr->first);
+            }
+
+            // sort the attributes so that equal attributes sit along side each
+            // other.
+            std::sort(attributeList.begin(),attributeList.end(),LessDerefFunctor<osg::StateAttribute>());
+
+            osg::notify(osg::INFO) << "state attribute list"<< std::endl;
+            for(AttributeList::iterator aaitr = attributeList.begin();
+                aaitr!=attributeList.end();
+                ++aaitr)
+            {
+                osg::notify(osg::INFO) << "    "<<*aaitr << "  "<<(*aaitr)->className()<< std::endl;
+            }
+
+            osg::notify(osg::INFO) << "searching for duplicate attributes"<< std::endl;
+            // find the duplicates.
+            AttributeList::iterator first_unique = attributeList.begin();
+            AttributeList::iterator current = first_unique;
+            ++current;
+            for(; current!=attributeList.end();++current)
+            {
+                if (**current==**first_unique)
+                {
+                    osg::notify(osg::INFO) << "    found duplicate "<<(*current)->className()<<"  first="<<*first_unique<<"  current="<<*current<< std::endl;
+                    StateSetList& statesetlist = attributeToStateSetMap[*current];
+                    for(StateSetList::iterator sitr=statesetlist.begin();
+                        sitr!=statesetlist.end();
+                        ++sitr)
+                    {
+                        osg::notify(osg::INFO) << "       replace duplicate "<<*current<<" with "<<*first_unique<< std::endl;
+                        osg::StateSet* stateset = sitr->first;
+                        unsigned int unit = sitr->second;
+                        if (unit==NON_TEXTURE_ATTRIBUTE) stateset->setAttribute(*first_unique);
+                        else stateset->setTextureAttribute(unit,*first_unique);
+                    }
+                }
+                else first_unique = current;
+            }
         }
         
+                
+        if (uniformToStateSetMap.size()>=2)
+        {
+            // create unique set of uniform pointers.
+            typedef std::vector<osg::Uniform*> UniformList;
+            UniformList uniformList;
+
+            for(UniformToStateSetMap::iterator aitr=uniformToStateSetMap.begin();
+                aitr!=uniformToStateSetMap.end();
+                ++aitr)
+            {
+                uniformList.push_back(aitr->first);
+            }
+
+            // sort the uniforms so that equal uniforms sit along side each
+            // other.
+            std::sort(uniformList.begin(),uniformList.end(),LessDerefFunctor<osg::Uniform>());
+
+            osg::notify(osg::INFO) << "state uniform list"<< std::endl;
+            for(UniformList::iterator uuitr = uniformList.begin();
+                uuitr!=uniformList.end();
+                ++uuitr)
+            {
+                osg::notify(osg::INFO) << "    "<<*uuitr << "  "<<(*uuitr)->getName()<< std::endl;
+            }
+
+            osg::notify(osg::INFO) << "searching for duplicate uniforms"<< std::endl;
+            // find the duplicates.
+            UniformList::iterator first_unique_uniform = uniformList.begin();
+            UniformList::iterator current_uniform = first_unique_uniform; 
+            ++current_uniform;
+            for(; current_uniform!=uniformList.end();++current_uniform)
+            {
+                if ((**current_uniform)==(**first_unique_uniform))
+                {
+                    osg::notify(osg::INFO) << "    found duplicate uniform "<<(*current_uniform)->getName()<<"  first_unique_uniform="<<*first_unique_uniform<<"  current_uniform="<<*current_uniform<< std::endl;
+                    StateSetSet& statesetset = uniformToStateSetMap[*current_uniform];
+                    for(StateSetSet::iterator sitr=statesetset.begin();
+                        sitr!=statesetset.end();
+                        ++sitr)
+                    {
+                        osg::notify(osg::INFO) << "       replace duplicate "<<*current_uniform<<" with "<<*first_unique_uniform<< std::endl;
+                        osg::StateSet* stateset = *sitr;
+                        stateset->addUniform(*first_unique_uniform);
+                    }
+                }
+                else first_unique_uniform = current_uniform;
+            }
+        }
+
     }
+    
     // duplicate state attributes removed.
     // now need to look at duplicate state sets.
-    
+    if (_statesets.size()>=2)
     {
         // create the list of stateset's.
         typedef std::vector<osg::StateSet*> StateSetSortList;
-        StateSetSortList _statesetSortList;
+        StateSetSortList statesetSortList;
         for(StateSetMap::iterator ssitr=_statesets.begin();
             ssitr!=_statesets.end();
             ++ssitr)
         {
-            _statesetSortList.push_back(ssitr->first);
+            statesetSortList.push_back(ssitr->first);
         }
 
 
         // sort the StateSet's so that equal StateSet's sit along side each
         // other.
-        std::sort(_statesetSortList.begin(),_statesetSortList.end(),LessStateSetFunctor());
+        std::sort(statesetSortList.begin(),statesetSortList.end(),LessDerefFunctor<osg::StateSet>());
 
         osg::notify(osg::INFO) << "searching for duplicate attributes"<< std::endl;
         // find the duplicates.
-        StateSetSortList::iterator first_unique = _statesetSortList.begin();
+        StateSetSortList::iterator first_unique = statesetSortList.begin();
         StateSetSortList::iterator current = first_unique; ++current;
-        for(; current!=_statesetSortList.end();++current)
+        for(; current!=statesetSortList.end();++current)
         {
             if (**current==**first_unique)
             {
@@ -499,15 +565,17 @@ void Optimizer::StateVisitor::optimize()
 // Flatten static transforms
 ////////////////////////////////////////////////////////////////////////////
 
-class CollectLowestTransformsVisitor : public osg::NodeVisitor
+class CollectLowestTransformsVisitor : public BaseOptimizerVisitor
 {
     public:
 
 
         CollectLowestTransformsVisitor(Optimizer* optimizer=0):
-                    osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_PARENTS),
-                    _transformFunctor(osg::Matrix()),
-                    _optimizer(optimizer) {}
+                    BaseOptimizerVisitor(optimizer,Optimizer::FLATTEN_STATIC_TRANSFORMS),
+                    _transformFunctor(osg::Matrix())
+        {
+            setTraversalMode(osg::NodeVisitor::TRAVERSE_PARENTS);
+        }
 
         virtual void apply(osg::Node& node)
         {
@@ -585,16 +653,30 @@ class CollectLowestTransformsVisitor : public osg::NodeVisitor
         void disableTransform(osg::Transform* transform);
         bool removeTransforms(osg::Node* nodeWeCannotRemove);
 
-        inline bool isOperationPermissibleForObject(const osg::Object* object)
+        inline bool isOperationPermissibleForObject(const osg::Object* object) const
+        {
+            const osg::Drawable* drawable = dynamic_cast<const osg::Drawable*>(object);
+            if (drawable) return isOperationPermissibleForObject(drawable);
+            
+            const osg::Node* node = dynamic_cast<const osg::Node*>(object);
+            if (node) return isOperationPermissibleForObject(node);
+
+            return true;
+        }
+
+        inline bool isOperationPermissibleForObject(const osg::Drawable* drawable) const
         {
             // disable if cannot apply transform functor.
-            const osg::Drawable* drawable = dynamic_cast<const osg::Drawable*>(object);
             if (drawable && !drawable->supports(_transformFunctor)) return false;
+            return BaseOptimizerVisitor::isOperationPermissibleForObject(drawable);
+        }
             
+        inline bool isOperationPermissibleForObject(const osg::Node* node) const
+        {
             // disable if object is a light point node.
-            if (strcmp(object->className(),"LightPointNode")==0) return false;
-
-            return _optimizer ? _optimizer->isOperationPermissibleForObject(object,Optimizer::FLATTEN_STATIC_TRANSFORMS) :  true; 
+            if (strcmp(node->className(),"LightPointNode")==0) return false;
+            if (dynamic_cast<const osg::ProxyNode*>(node)) return false;
+            return BaseOptimizerVisitor::isOperationPermissibleForObject(node);
         }
 
     protected:        
@@ -625,14 +707,14 @@ class CollectLowestTransformsVisitor : public osg::NodeVisitor
                 if (transform)
                 {
                     if (transform->getDataVariance()==osg::Transform::DYNAMIC) _moreThanOneMatrixRequired=true;
-                    else if (transform->getReferenceFrame()==osg::Transform::RELATIVE_RF) _moreThanOneMatrixRequired=true;
+                    else if (transform->getReferenceFrame()==osg::Transform::ABSOLUTE_RF) _moreThanOneMatrixRequired=true;
                     else
                     {
                         if (_transformSet.empty()) transform->computeLocalToWorldMatrix(_firstMatrix,0);
                         else
                         {
                             osg::Matrix matrix;
-                            transform->computeLocalToWorldMatrix(_firstMatrix,0);
+                            transform->computeLocalToWorldMatrix(matrix,0);
                             if (_firstMatrix!=matrix) _moreThanOneMatrixRequired=true;
                         }
                     }
@@ -641,7 +723,7 @@ class CollectLowestTransformsVisitor : public osg::NodeVisitor
                 {
                     if (!_transformSet.empty()) 
                     {
-                        if (_firstMatrix!=osg::Matrix::identity()) _moreThanOneMatrixRequired=true;
+                        if (_firstMatrix!=_identity_matrix) _moreThanOneMatrixRequired=true;
                     }
 
                 }
@@ -652,6 +734,7 @@ class CollectLowestTransformsVisitor : public osg::NodeVisitor
             bool            _moreThanOneMatrixRequired;
             osg::Matrix     _firstMatrix;
             TransformSet    _transformSet;
+            osg::Matrix     _identity_matrix; 
 
         };    
 
@@ -679,7 +762,6 @@ class CollectLowestTransformsVisitor : public osg::NodeVisitor
         void doTransform(osg::Object* obj,osg::Matrix& matrix);
         
         osgUtil::TransformAttributeFunctor _transformFunctor;
-        Optimizer*      _optimizer;
         TransformMap    _transformMap;
         ObjectMap       _objectMap;
         ObjectList      _currentObjectList;
@@ -798,7 +880,6 @@ void CollectLowestTransformsVisitor::disableTransform(osg::Transform* transform)
 
 void CollectLowestTransformsVisitor::setUpMaps()
 {
-
     // create the TransformMap from the ObjectMap
     ObjectMap::iterator oitr;
     for(oitr=_objectMap.begin();
@@ -815,7 +896,6 @@ void CollectLowestTransformsVisitor::setUpMaps()
             _transformMap[*titr].add(object);
         }
     }
-    
 
     // disable all the objects which have more than one matrix associated 
     // with them, and then disable all transforms which have an object associated 
@@ -837,6 +917,7 @@ void CollectLowestTransformsVisitor::setUpMaps()
             }
         }
     }
+
 }
 
 bool CollectLowestTransformsVisitor::removeTransforms(osg::Node* nodeWeCannotRemove)
@@ -864,7 +945,6 @@ bool CollectLowestTransformsVisitor::removeTransforms(osg::Node* nodeWeCannotRem
     {
         if (titr->second._canBeApplied)
         {
-        
             if (titr->first!=nodeWeCannotRemove)
             {
                 transformRemoved = true;
@@ -873,6 +953,7 @@ bool CollectLowestTransformsVisitor::removeTransforms(osg::Node* nodeWeCannotRem
                 osg::ref_ptr<osg::Group>     group = new osg::Group;
                 group->setName( transform->getName() );
                 group->setDataVariance(osg::Object::STATIC);
+                group->setNodeMask(transform->getNodeMask());
                 for(unsigned int i=0;i<transform->getNumChildren();++i)
                 {
                     for(unsigned int j=0;j<transform->getNumParents();++j)
@@ -925,6 +1006,15 @@ void Optimizer::FlattenStaticTransformsVisitor::apply(osg::Node& node)
     traverse(node);
 }
 
+
+void Optimizer::FlattenStaticTransformsVisitor::apply(osg::ProxyNode& node)
+{
+    _excludedNodeSet.insert(&node);
+
+    traverse(node);
+}
+
+
 void Optimizer::FlattenStaticTransformsVisitor::apply(osg::Geode& geode)
 {
     if (!_transformStack.empty())
@@ -954,8 +1044,6 @@ void Optimizer::FlattenStaticTransformsVisitor::apply(osg::Transform& transform)
 
     _transformStack.push_back(&transform);
 
-    
-
     // simple traverse the children as if this Transform didn't exist.
     traverse(transform);
 
@@ -1001,7 +1089,7 @@ bool Optimizer::FlattenStaticTransformsVisitor::removeTransforms(osg::Node* node
 }
 
 ////////////////////////////////////////////////////////////////////////////
-// RemoveEmptyNodes.
+// CombineStatucTransforms
 ////////////////////////////////////////////////////////////////////////////
 
 void Optimizer::CombineStaticTransformsVisitor::apply(osg::MatrixTransform& transform)
@@ -1010,7 +1098,8 @@ void Optimizer::CombineStaticTransformsVisitor::apply(osg::MatrixTransform& tran
         transform.getNumChildren()==1 &&
         transform.getChild(0)->asTransform()!=0 &&
         transform.getChild(0)->asTransform()->asMatrixTransform()!=0 &&
-        transform.getChild(0)->asTransform()->getDataVariance()==osg::Object::STATIC)
+        transform.getChild(0)->asTransform()->getDataVariance()==osg::Object::STATIC &&
+        isOperationPermissibleForObject(&transform) && isOperationPermissibleForObject(transform.getChild(0)))
     {
         _transformSet.insert(&transform);
     }
@@ -1027,8 +1116,6 @@ bool Optimizer::CombineStaticTransformsVisitor::removeTransforms(osg::Node* node
         if (itr!=_transformSet.end()) _transformSet.erase(itr);
     }
     
-    osg::notify(osg::INFO)<<"Have found "<<_transformSet.size()<<" static Transform pairs to flatten"<<std::endl;
-    
     bool transformRemoved = false;
 
     while (!_transformSet.empty())
@@ -1073,16 +1160,15 @@ void Optimizer::RemoveEmptyNodesVisitor::apply(osg::Geode& geode)
     for(int i=geode.getNumDrawables()-1;i>=0;--i)
     {
         osg::Geometry* geom = geode.getDrawable(i)->asGeometry();
-        if (geom && geom->empty())
+        if (geom && geom->empty() && isOperationPermissibleForObject(geom))
         {
            geode.removeDrawable(i);
         }
     }
 
-
     if (geode.getNumParents()>0)
     {
-        if (geode.getNumDrawables()==0 && isNodeEmpty(geode)) _redundantNodeList.insert(&geode);
+        if (geode.getNumDrawables()==0 && isOperationPermissibleForObject(&geode)) _redundantNodeList.insert(&geode);
     }
 }
 
@@ -1091,7 +1177,7 @@ void Optimizer::RemoveEmptyNodesVisitor::apply(osg::Group& group)
     if (group.getNumParents()>0)
     {
         // only remove empty groups, but not empty occluders.
-        if (group.getNumChildren()==0 && isNodeEmpty(group) && 
+        if (group.getNumChildren()==0 && isOperationPermissibleForObject(&group) && 
             (typeid(group)==typeid(osg::Group) || dynamic_cast<osg::Transform*>(&group)))
         {
             _redundantNodeList.insert(&group);
@@ -1151,7 +1237,7 @@ void Optimizer::RemoveRedundantNodesVisitor::apply(osg::Group& group)
         {
             if (group.getNumParents()>0 && group.getNumChildren()<=1)
             {
-                if (isNodeEmpty(group) && isOperationPermissibleForObject(&group))
+                if (isOperationPermissibleForObject(&group))
                 {
                     _redundantNodeList.insert(&group);
                 }
@@ -1214,19 +1300,102 @@ void Optimizer::RemoveRedundantNodesVisitor::removeRedundantNodes()
 }
 
 
+////////////////////////////////////////////////////////////////////////////
+// RemoveLoadedProxyNodesVisitor.
+////////////////////////////////////////////////////////////////////////////
+
+void Optimizer::RemoveLoadedProxyNodesVisitor::apply(osg::ProxyNode& proxyNode)
+{
+    if (proxyNode.getNumParents()>0 && proxyNode.getNumFileNames()==proxyNode.getNumChildren())
+    {
+        if (isOperationPermissibleForObject(&proxyNode))
+        {
+            _redundantNodeList.insert(&proxyNode);
+        }
+    }
+    traverse(proxyNode);
+}
+
+void Optimizer::RemoveLoadedProxyNodesVisitor::removeRedundantNodes()
+{
+
+    for(NodeList::iterator itr=_redundantNodeList.begin();
+        itr!=_redundantNodeList.end();
+        ++itr)
+    {
+        osg::ref_ptr<osg::Group> group = dynamic_cast<osg::Group*>(*itr);
+        if (group.valid())
+        {
+            
+            // first check to see if data was attached to the ProxyNode that we need to keep.
+            bool keepData = false;
+            if (!group->getName().empty()) keepData = true;
+            if (!group->getDescriptions().empty()) keepData = true;
+            if (group->getNodeMask()) keepData = true;
+            if (group->getUpdateCallback()) keepData = true;
+            if (group->getEventCallback()) keepData = true;
+            if (group->getCullCallback()) keepData = true;
+
+            if (keepData)
+            {
+                // create a group to store all proxy's children and data.
+                osg::ref_ptr<osg::Group> newGroup = new osg::Group(*group,osg::CopyOp::SHALLOW_COPY);
+                
+
+                // take a copy of parents list since subsequent removes will modify the original one.
+                osg::Node::ParentList parents = group->getParents();
+
+                for(osg::Node::ParentList::iterator pitr=parents.begin();
+                    pitr!=parents.end();
+                    ++pitr)
+                {
+                    (*pitr)->replaceChild(group.get(),newGroup.get());
+                }
+
+            }
+            else
+            {
+                // take a copy of parents list since subsequent removes will modify the original one.
+                osg::Node::ParentList parents = group->getParents();
+
+                for(unsigned int i=0;i<group->getNumChildren();++i)
+                {
+                    osg::Node* child = group->getChild(i);
+                    for(osg::Node::ParentList::iterator pitr=parents.begin();
+                        pitr!=parents.end();
+                        ++pitr)
+                    {
+                        (*pitr)->replaceChild(group.get(),child);
+                    }
+
+                }
+            }
+        }
+        else
+        {
+            osg::notify(osg::WARN)<<"Optimizer::RemoveLoadedProxyNodesVisitor::removeRedundantNodes() - failed dynamic_cast"<<std::endl;
+        }                                
+    }
+    _redundantNodeList.clear();
+}
+
+
 
 ////////////////////////////////////////////////////////////////////////////
 // combine LOD's.
 ////////////////////////////////////////////////////////////////////////////
 void Optimizer::CombineLODsVisitor::apply(osg::LOD& lod)
 {
-    for(unsigned int i=0;i<lod.getNumParents();++i)
-    {
-        if (typeid(*lod.getParent(i))==typeid(osg::Group))
+    if (dynamic_cast<osg::PagedLOD*>(&lod)==0)
+    {    
+        for(unsigned int i=0;i<lod.getNumParents();++i)
         {
-            if (isOperationPermissibleForObject(&lod))
+            if (typeid(*lod.getParent(i))==typeid(osg::Group))
             {
-                _groupList.insert(lod.getParent(i));
+                if (isOperationPermissibleForObject(&lod))
+                {
+                    _groupList.insert(lod.getParent(i));
+                }
             }
         }
     }
@@ -1242,7 +1411,7 @@ void Optimizer::CombineLODsVisitor::combineLODs()
     {
         osg::Group* group = *itr;
 
-        typedef std::set<osg::LOD*>                 LODSet;
+        typedef std::set<osg::LOD*>  LODSet;
 
         LODSet    lodChildren;
 
@@ -1409,9 +1578,11 @@ struct LessGeometry
                 case(osg::Array::UByte4ArrayType):
                     if ((*static_cast<const osg::UByte4Array*>(lhs_colorArray))[0]<(*static_cast<const osg::UByte4Array*>(rhs_colorArray))[0]) return true;
                     if ((*static_cast<const osg::UByte4Array*>(rhs_colorArray))[0]<(*static_cast<const osg::UByte4Array*>(lhs_colorArray))[0]) return false;
+                    break;
                 case(osg::Array::Vec3ArrayType):
                     if ((*static_cast<const osg::Vec3Array*>(lhs_colorArray))[0]<(*static_cast<const osg::Vec3Array*>(rhs_colorArray))[0]) return true;
                     if ((*static_cast<const osg::Vec3Array*>(rhs_colorArray))[0]<(*static_cast<const osg::Vec3Array*>(lhs_colorArray))[0]) return false;
+                    break;
                 case(osg::Array::Vec4ArrayType):
                     if ((*static_cast<const osg::Vec4Array*>(lhs_colorArray))[0]<(*static_cast<const osg::Vec4Array*>(rhs_colorArray))[0]) return true;
                     if ((*static_cast<const osg::Vec4Array*>(rhs_colorArray))[0]<(*static_cast<const osg::Vec4Array*>(lhs_colorArray))[0]) return false;
@@ -1492,8 +1663,6 @@ bool Optimizer::MergeGeometryVisitor::mergeGeode(osg::Geode& geode)
 
 
         // don't merge geometry if its above a maximum number of vertices.
-        unsigned int _maximumNumber = 10000;
-
         for(GeometryDuplicateMap::iterator itr=geometryDuplicateMap.begin();
             itr!=geometryDuplicateMap.end();
             ++itr)
@@ -1509,19 +1678,19 @@ bool Optimizer::MergeGeometryVisitor::mergeGeode(osg::Geode& geode)
                 
                     osg::Geometry* rhs = *dupItr;
                     
-                    if (lhs->getVertexArray() && lhs->getVertexArray()->getNumElements()>=_maximumNumber)
+                    if (lhs->getVertexArray() && lhs->getVertexArray()->getNumElements()>=_targetMaximumNumberOfVertices)
                     {
                         lhs = rhs;
                         continue;
                     }
 
-                    if (rhs->getVertexArray() && rhs->getVertexArray()->getNumElements()>=_maximumNumber)
+                    if (rhs->getVertexArray() && rhs->getVertexArray()->getNumElements()>=_targetMaximumNumberOfVertices)
                     {
                         continue;
                     }
                     
                     if (lhs->getVertexArray() && rhs->getVertexArray() && 
-                        (lhs->getVertexArray()->getNumElements()+rhs->getVertexArray()->getNumElements())>=_maximumNumber)
+                        (lhs->getVertexArray()->getNumElements()+rhs->getVertexArray()->getNumElements())>=_targetMaximumNumberOfVertices)
                     {
                         continue;
                     }
@@ -2024,6 +2193,12 @@ bool Optimizer::SpatializeGroupsVisitor::divide(osg::Group* group, unsigned int
 
     osg::notify(osg::INFO)<<"Dividing "<<group->className()<<"  num children = "<<group->getNumChildren()<<"  xAxis="<<xAxis<<"  yAxis="<<yAxis<<"   zAxis="<<zAxis<<std::endl;
     
+    if (!xAxis && !yAxis && !zAxis)
+    {
+        osg::notify(osg::INFO)<<"  No axis to divide, stopping division."<<std::endl;
+        return false;
+    }
+    
     unsigned int numChildrenOnEntry = group->getNumChildren();
     
     typedef std::pair< osg::BoundingBox, osg::ref_ptr<osg::Group> > BoxGroupPair;
