@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -14,8 +14,8 @@
 #include <osgUtil/RenderStage>
 #include <osgUtil/Statistics>
 
-#include <osg/ImpostorSprite>
 #include <osg/Notify>
+#include <osg/ApplicationUsage>
 
 #include <algorithm>
 
@@ -31,11 +31,11 @@ class RenderBinPrototypeList : public osg::Referenced, public std::map< std::str
 };
 
 // register a RenderStage prototype with the RenderBin prototype list.
-RegisterRenderBinProxy s_registerRenderBinProxy("RenderBin",new RenderBin(RenderBin::SORT_BY_STATE));
+RegisterRenderBinProxy s_registerRenderBinProxy("RenderBin",new RenderBin(RenderBin::getDefaultRenderBinSortMode()));
 RegisterRenderBinProxy s_registerDepthSortedBinProxy("DepthSortedBin",new RenderBin(RenderBin::SORT_BACK_TO_FRONT));
 
 
-RenderBinPrototypeList* renderBinPrototypeList()
+static RenderBinPrototypeList* renderBinPrototypeList()
 {
     static osg::ref_ptr<RenderBinPrototypeList> s_renderBinPrototypeList = new  RenderBinPrototypeList;
     return s_renderBinPrototypeList.get();
@@ -84,6 +84,44 @@ void RenderBin::removeRenderBinPrototype(RenderBin* proto)
     }
 }
 
+static bool s_defaultBinSortModeInitialized = false;
+static RenderBin::SortMode s_defaultBinSortMode = RenderBin::SORT_BY_STATE;
+static osg::ApplicationUsageProxy RenderBin_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_DEFAULT_BIN_SORT_MODE <type>","SORT_BY_STATE | SORT_BY_STATE_THEN_FRONT_TO_BACK | SORT_FRONT_TO_BACK | SORT_BACK_TO_FRONT");
+
+void RenderBin::setDefaultRenderBinSortMode(RenderBin::SortMode mode)
+{
+    s_defaultBinSortModeInitialized = true;
+    s_defaultBinSortMode = mode;
+}
+
+
+RenderBin::SortMode RenderBin::getDefaultRenderBinSortMode()
+{
+    if (!s_defaultBinSortModeInitialized)
+    {
+        s_defaultBinSortModeInitialized = true;
+        
+        const char* str = getenv("OSG_DEFAULT_BIN_SORT_MODE");
+        if (str)
+        {
+            if (strcmp(str,"SORT_BY_STATE")==0) s_defaultBinSortMode = RenderBin::SORT_BY_STATE;
+            else if (strcmp(str,"SORT_BY_STATE_THEN_FRONT_TO_BACK")==0) s_defaultBinSortMode = RenderBin::SORT_BY_STATE_THEN_FRONT_TO_BACK;
+            else if (strcmp(str,"SORT_FRONT_TO_BACK")==0) s_defaultBinSortMode = RenderBin::SORT_FRONT_TO_BACK;
+            else if (strcmp(str,"SORT_BACK_TO_FRONT")==0) s_defaultBinSortMode = RenderBin::SORT_BACK_TO_FRONT;
+        }
+    }
+    
+    return s_defaultBinSortMode;
+}
+
+RenderBin::RenderBin()
+{
+    _binNum = 0;
+    _parent = NULL;
+    _stage = NULL;
+    _sortMode = getDefaultRenderBinSortMode();
+}
+
 RenderBin::RenderBin(SortMode mode)
 {
     _binNum = 0;
@@ -145,6 +183,9 @@ void RenderBin::sortImplementation()
         case(SORT_BY_STATE):
             sortByState();
             break;
+        case(SORT_BY_STATE_THEN_FRONT_TO_BACK):
+            sortByStateThenFrontToBack();
+            break;
         case(SORT_FRONT_TO_BACK):
             sortFrontToBack();
             break;
@@ -166,6 +207,7 @@ struct SortByStateFunctor
 
 void RenderBin::sortByState()
 {
+    //osg::notify(osg::NOTICE)<<"sortByState()"<<std::endl;
     // actually we'll do nothing right now, as fine grained sorting by state
     // appears to cost more to do than it saves in draw.  The contents of
     // the RenderGraph leaves is already coarse grained sorted, this
@@ -173,6 +215,27 @@ void RenderBin::sortByState()
     // cout << "doing sortByState "<<this<<endl;
 }
 
+
+struct RenderGraphFrontToBackSortFunctor
+{
+    bool operator() (const RenderGraph* lhs,const RenderGraph* rhs) const
+    {
+        return (lhs->_minimumDistance<rhs->_minimumDistance);
+    }
+};
+
+void RenderBin::sortByStateThenFrontToBack()
+{
+    for(RenderGraphList::iterator itr=_renderGraphList.begin();
+        itr!=_renderGraphList.end();
+        ++itr)
+    {
+        (*itr)->sortFrontToBack();
+        (*itr)->getMinimumDistance();
+    }
+    std::sort(_renderGraphList.begin(),_renderGraphList.end(),RenderGraphFrontToBackSortFunctor());
+}
+
 struct FrontToBackSortFunctor
 {
     bool operator() (const RenderLeaf* lhs,const RenderLeaf* rhs) const
@@ -351,7 +414,6 @@ bool RenderBin::getStats(Statistics* primStats)
         {
           // then tot up the primtive types and no vertices.
           dw->accept(*primStats); // use sub-class to find the stats for each drawable
-          if (typeid(*dw)==typeid(osg::ImpostorSprite)) primStats->addImpostor(1);
         }
       somestats = true;
 
@@ -374,7 +436,6 @@ bool RenderBin::getStats(Statistics* primStats)
             {
                 // then tot up the primtive types and no vertices.
                 dw->accept(*primStats); // use sub-class to find the stats for each drawable
-                if (typeid(*dw)==typeid(osg::ImpostorSprite)) primStats->addImpostor(1);
             }
         }
         somestats=true;
