@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -24,6 +24,7 @@
 #include <osg/ColorMatrix>
 #include <osg/LightModel>
 #include <osg/CollectOccludersVisitor>
+#include <osg/Shader>
 
 #include <osg/GLU>
 
@@ -52,6 +53,10 @@ SceneView::SceneView(DisplaySettings* ds)
     _drawBufferValue = GL_BACK;
 
     _requiresFlush = true;
+    
+    _activeUniforms = DEFAULT_UNIFORMS;
+    
+    _previousFrameTime = 0;
 }
 
 
@@ -163,11 +168,6 @@ void SceneView::init()
 
 void SceneView::update()
 {
-    // comment out since the init traversal by default with do OpenGL calls,
-    // which you definately don't want to call during update().
-    // if (!_initCalled) init();
-
-
     if (_sceneData.valid() && _updateVisitor.valid())
     { 
         _updateVisitor->reset();
@@ -188,23 +188,54 @@ void SceneView::update()
         // multi-threaded.
         _sceneData->getBound();
     }
+}
+
+void SceneView::updateUniforms()
+{
+    if (!_localStateSet)
+    {
+        _localStateSet = new osg::StateSet;
+    }
+
+    if (!_localStateSet) return;
     
+    if ((_activeUniforms & FRAME_NUMBER_UNIFORM) && _frameStamp.valid())
+    {
+        osg::Uniform* uniform = _localStateSet->getOrCreateUniform("osg_FrameNumber",osg::Uniform::INT);
+        uniform->set(_frameStamp->getFrameNumber());        
+    }
     
+    if ((_activeUniforms & FRAME_TIME_UNIFORM) && _frameStamp.valid())
+    {
+        osg::Uniform* uniform = _localStateSet->getOrCreateUniform("osg_FrameTime",osg::Uniform::FLOAT);
+        uniform->set(static_cast<float>(_frameStamp->getReferenceTime()));
+    }
+    
+    if ((_activeUniforms & DELTA_FRAME_TIME_UNIFORM) && _frameStamp.valid())
+    {
+        float delta_frame_time = (_previousFrameTime != 0.0) ? static_cast<float>(_frameStamp->getReferenceTime()-_previousFrameTime) : 0.0f;
+        _previousFrameTime = _frameStamp->getReferenceTime();
+        
+        osg::Uniform* uniform = _localStateSet->getOrCreateUniform("osg_DeltaFrameTime",osg::Uniform::FLOAT);
+        uniform->set(delta_frame_time);
+    }
+    
+    if (_activeUniforms & VIEW_MATRIX_UNIFORM)
+    {
+        osg::Uniform* uniform = _localStateSet->getOrCreateUniform("osg_ViewMatrix",osg::Uniform::FLOAT_MAT4);
+        uniform->set(_viewMatrix);      
+    }
+
+    if (_activeUniforms & INVERSE_VIEW_MATRIX_UNIFORM)
+    {
+        osg::Uniform* uniform = _localStateSet->getOrCreateUniform("osg_InverseViewMatrix",osg::Uniform::FLOAT_MAT4);
+        uniform->set(osg::Matrix::inverse(_viewMatrix));      
+    }
+
 }
 
 osg::Matrixd SceneView::computeLeftEyeProjectionImplementation(const osg::Matrixd& projection) const
 {
-    double fusionDistance = _displaySettings->getScreenDistance();
-    switch(_fusionDistanceMode)
-    {
-        case(USE_FUSION_DISTANCE_VALUE):
-            fusionDistance = _fusionDistanceValue;
-            break;
-        case(PROPORTIONAL_TO_SCREEN_DISTANCE):
-            fusionDistance *= _fusionDistanceValue;
-            break;
-    }
-
     double iod = _displaySettings->getEyeSeparation();
     double sd = _displaySettings->getScreenDistance();
     double scale_x = 1.0;
@@ -270,17 +301,6 @@ osg::Matrixd SceneView::computeLeftEyeViewImplementation(const osg::Matrixd& vie
 
 osg::Matrixd SceneView::computeRightEyeProjectionImplementation(const osg::Matrixd& projection) const
 {
-    double fusionDistance = _displaySettings->getScreenDistance();
-    switch(_fusionDistanceMode)
-    {
-        case(USE_FUSION_DISTANCE_VALUE):
-            fusionDistance = _fusionDistanceValue;
-            break;
-        case(PROPORTIONAL_TO_SCREEN_DISTANCE):
-            fusionDistance *= _fusionDistanceValue;
-            break;
-    }
-
     double iod = _displaySettings->getEyeSeparation();
     double sd = _displaySettings->getScreenDistance();
     double scale_x = 1.0;
@@ -346,6 +366,8 @@ osg::Matrixd SceneView::computeRightEyeViewImplementation(const osg::Matrixd& vi
 
 void SceneView::cull()
 {
+    // update the active uniforms
+    updateUniforms();
 
     if (!_state)
     {
@@ -593,10 +615,7 @@ void SceneView::cullStage(const osg::Matrixd& projection,const osg::Matrixd& mod
         if (clearNode->getRequiresClear())
         {
             renderStage->setClearColor(clearNode->getClearColor());
-            renderStage->setClearMask(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
-            // really should set clear mask here, but what to? Need
-            // to consider the stencil and accumulation buffers..
-            // will defer to later.  Robert Osfield. October 2001.
+            renderStage->setClearMask(clearNode->getClearMask());
         }
         else
         {
@@ -620,10 +639,7 @@ void SceneView::releaseAllGLObjects()
 {
     if (!_sceneData) return;
    
-    GLObjectsVisitor globjv(GLObjectsVisitor::RELEASE_DISPLAY_LISTS|GLObjectsVisitor::RELEASE_STATE_ATTRIBUTES);
-    globjv.setNodeMaskOverride(0xffffffff);
-    globjv.setState(_state.get());
-    _sceneData->accept(globjv);
+    _sceneData->releaseGLObjects(_state.get());
 }
 
 
@@ -639,7 +655,9 @@ void SceneView::flushAllDeletedGLObjects()
     osg::Drawable::flushDeletedVertexBufferObjects(_state->getContextID(),currentTime,availableTime);
     osg::VertexProgram::flushDeletedVertexProgramObjects(_state->getContextID(),currentTime,availableTime);
     osg::FragmentProgram::flushDeletedFragmentProgramObjects(_state->getContextID(),currentTime,availableTime);
-}
+    osg::Program::flushDeletedGlPrograms(_state->getContextID(),currentTime,availableTime);
+    osg::Shader::flushDeletedGlShaders(_state->getContextID(),currentTime,availableTime);
+ }
 
 void SceneView::flushDeletedGLObjects(double& availableTime)
 {
@@ -651,6 +669,8 @@ void SceneView::flushDeletedGLObjects(double& availableTime)
     osg::Drawable::flushDeletedVertexBufferObjects(_state->getContextID(),currentTime,availableTime);
     osg::VertexProgram::flushDeletedVertexProgramObjects(_state->getContextID(),currentTime,availableTime);
     osg::FragmentProgram::flushDeletedFragmentProgramObjects(_state->getContextID(),currentTime,availableTime);
+    osg::Program::flushDeletedGlPrograms(_state->getContextID(),currentTime,availableTime);
+    osg::Shader::flushDeletedGlShaders(_state->getContextID(),currentTime,availableTime);
 }
 
 void SceneView::draw()
@@ -960,12 +980,17 @@ void SceneView::draw()
     // re apply the defalt OGL state.
     _state->popAllStateSets();
 
-    GLenum errorNo = glGetError();
-    if (errorNo!=GL_NO_ERROR)
+    if (_state->getCheckForGLErrors()!=osg::State::NEVER_CHECK_GL_ERRORS)
     {
-        osg::notify(WARN)<<"Warning: detected OpenGL error '"<<gluErrorString(errorNo)<<"'"<< std::endl;
-        // go into debug mode of OGL errors.
-        _state->setReportGLErrors(true);
+        GLenum errorNo = glGetError();
+        if (errorNo!=GL_NO_ERROR)
+        {
+            osg::notify(WARN)<<"Warning: detected OpenGL error '"<<gluErrorString(errorNo)<<"'"<< std::endl;
+
+            // go into debug mode of OGL error in a fine grained way to help
+            // track down OpenGL errors.
+            _state->setCheckForGLErrors(osg::State::ONCE_PER_ATTRIBUTE);
+        }
     }
 }
 
