@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -35,17 +35,19 @@ Tesselator::~Tesselator()
     reset();
 }
 
+
+
 void Tesselator::beginTesselation()
 {
     reset();
 
     if (!_tobj) _tobj = gluNewTess();
     
-    gluTessCallback(_tobj, GLU_TESS_VERTEX_DATA, (GLvoid (CALLBACK*)()) vertexCallback);
-    gluTessCallback(_tobj, GLU_TESS_BEGIN_DATA,  (GLvoid (CALLBACK*)()) beginCallback);
-    gluTessCallback(_tobj, GLU_TESS_END_DATA,    (GLvoid (CALLBACK*)()) endCallback);
-    gluTessCallback(_tobj, GLU_TESS_COMBINE_DATA,(GLvoid (CALLBACK*)()) combineCallback);
-    gluTessCallback(_tobj, GLU_TESS_ERROR_DATA,  (GLvoid (CALLBACK*)()) errorCallback);
+    gluTessCallback(_tobj, GLU_TESS_VERTEX_DATA, (GLU_TESS_CALLBACK) vertexCallback);
+    gluTessCallback(_tobj, GLU_TESS_BEGIN_DATA,  (GLU_TESS_CALLBACK) beginCallback);
+    gluTessCallback(_tobj, GLU_TESS_END_DATA,    (GLU_TESS_CALLBACK) endCallback);
+    gluTessCallback(_tobj, GLU_TESS_COMBINE_DATA,(GLU_TESS_CALLBACK) combineCallback);
+    gluTessCallback(_tobj, GLU_TESS_ERROR_DATA,  (GLU_TESS_CALLBACK) errorCallback);
 
     gluTessBeginPolygon(_tobj,this);
 }    
@@ -209,7 +211,7 @@ void Tesselator::retesselatePolygons(osg::Geometry &geom)
     {
         osg::ref_ptr<osg::PrimitiveSet> primitive = _Contours[primNo].get();
         if (_ttype==TESS_TYPE_POLYGONS || _ttype==TESS_TYPE_DRAWABLE)
-        {
+        { // this recovers the 'old' tesselation which just retesselates single polygons.
             if (primitive->getMode()==osg::PrimitiveSet::POLYGON || _ttype==TESS_TYPE_DRAWABLE)
             {
 
@@ -231,10 +233,14 @@ void Tesselator::retesselatePolygons(osg::Geometry &geom)
                 }
                 else
                 {
-                    beginTesselation();
+                    if (primitive->getNumIndices()>3) { // April 2005 gwm only retesselate "complex" polygons
+                        beginTesselation();
                         addContour(primitive.get(), vertices);
-                    endTesselation();
-                    collectTesselation(geom);
+                        endTesselation();
+                        collectTesselation(geom);
+                    } else { // April 2005 gwm triangles don't need to be retesselated
+                        geom.addPrimitiveSet(primitive.get());
+                    }
                 }
 
             }
