@@ -16,6 +16,7 @@
 #endif
 
 #include <string.h>
+#include <algorithm>
 //#include <GL/gl.h>
 //#include <GL/glu.h>
 
@@ -44,6 +45,7 @@ Camera::Camera( void )
     memcpy( _viewMatrix, id, sizeof(Matrix::value_type[16]));
     _offset._xshear = _offset._yshear = 0.0f;
     memcpy( _offset._matrix, id, sizeof(Matrix::value_type[16]));
+    _offset._multiplyMethod = Offset::PreMultiply;
 
     _lens = new Lens;
     _lens->setAutoAspect(true);
@@ -262,7 +264,6 @@ void Camera::_frame( bool doSwap )
             int x, y;
             unsigned int width, height;
             _rs->getWindowRectangle(x,y,width,height);
-            _lens->setAspectRatio( float(width)/float(height));
             double ar = (float(width) * (_projrectRight - _projrectLeft))/
                         (float(height) * (_projrectTop - _projrectBottom ));
             _lens->setAspectRatio( ar );
@@ -307,7 +308,11 @@ const Matrix::value_type * Camera::getViewMatrix( void ) const
 
 void Camera::setViewByMatrix( const Matrix &mat )
 {
-    Matrix m = mat * Matrix( _offset._matrix );
+    Matrix m;
+    if ( _offset._multiplyMethod == Offset::PostMultiply )
+        m = Matrix( _offset._matrix ) * mat;
+    else if( _offset._multiplyMethod == Offset::PreMultiply )
+        m = mat * Matrix( _offset._matrix );
     memcpy( _viewMatrix, m.ptr(), sizeof( Matrix::value_type[16] ));
 }
 
@@ -439,9 +444,63 @@ void Camera::Lens::setOrtho( double left, double right,
     setAutoAspect(false);
 }
 
+void Camera::Lens::setMatrix( const Matrix::value_type matrix[16] )
+{
+    memcpy( _matrix, matrix, sizeof(Matrix::value_type[16]) );
+    _projection = Manual;
+    setAutoAspect(false);
+}
+
+bool Camera::Lens::getFrustum( double& left, double& right,
+                double& bottom, double& top,
+                double& zNear, double& zFar ) const
+{
+    //The following code was taken from osg's matrix implementation of getFrustum
+    if (_matrix[3]!=0.0 || _matrix[7]!=0.0 || _matrix[11]!=-1.0 || _matrix[15]!=0.0) return false;
+
+    zNear = _matrix[14] / (_matrix[10]-1.0);
+    zFar = _matrix[14] / (1.0+_matrix[10]);
+    
+    left = zNear * (_matrix[8]-1.0) / _matrix[0];
+    right = zNear * (1.0+_matrix[8]) / _matrix[0];
+
+    top = zNear * (1.0+_matrix[9]) / _matrix[5];
+    bottom = zNear * (_matrix[9]-1.0) / _matrix[5];
+
+    return true;
+}
+
+bool Camera::Lens::getOrtho( double& left, double& right,
+                double& bottom, double& top,
+                double& zNear, double& zFar ) const
+{
+    //The following code was taken from osg's matrix implementation of getOrtho
+    if (_matrix[3]!=0.0 || _matrix[7]!=0.0 || _matrix[11]!=0.0 || _matrix[15]!=1.0) return false;
+
+    zNear = (_matrix[14]+1.0) / _matrix[10];
+    zFar = (_matrix[14]-1.0) / _matrix[10];
+    
+    left = -(1.0+_matrix[12]) / _matrix[0];
+    right = (1.0-_matrix[12]) / _matrix[0];
+
+    bottom = -(1.0+_matrix[13]) / _matrix[5];
+    top = (1.0-_matrix[13]) / _matrix[5];
+
+    return true;
+}
+
 bool Camera::Lens::convertToOrtho( float d )
 {
 
+    if( _projection == Manual )
+    {
+        //Need to extract frustum values from manual matrix
+        if( !getFrustum(_left,_right,_bottom,_top,_nearClip,_farClip) )
+            return false;
+
+        _updateFOV();
+    }
+
     double s = d * tan(_vfov*0.5);
     _ortho_bottom = -s;
     _ortho_top = s;
@@ -453,6 +512,14 @@ bool Camera::Lens::convertToOrtho( float d )
 
 bool Camera::Lens::convertToPerspective( float d )
 {
+
+    if( _projection == Manual )
+    {
+        //Need to extract ortho values from manual matrix
+        if( !getOrtho(_ortho_left,_ortho_right,_ortho_bottom,_ortho_top,_nearClip,_farClip) )
+            return false;
+    }
+
     double hfov = 2 * atan( 0.5 * (_ortho_right - _ortho_left)/d);
     double vfov = 2 * atan( 0.5 * (_ortho_top - _ortho_bottom)/d);
 
@@ -486,13 +553,28 @@ void Camera::Lens::getParams( double &left, double &right, double &bottom, doubl
         bottom = _bottom;
         top    = _top;
     }
-    else
+    else if( _projection == Orthographic )
     {
         left   = _ortho_left;
         right  = _ortho_right;
         bottom = _ortho_bottom;
         top    = _ortho_top;
     }
+    else if( _projection == Manual ) // could only be Manual, but best to make this clear
+    {
+        // Check if Manual matrix is either a valid perspective or orthographic matrix
+        // If neither, then return bogus values -- nothing better we can do
+        if(getFrustum(left,right,bottom,top,nearClip,farClip))
+            return;
+
+        if(getOrtho(left,right,bottom,top,nearClip,farClip))
+            return;
+
+        left   = _left;
+        right  = _right;
+        bottom = _bottom;
+        top    = _top;
+    }
     nearClip   = _nearClip;
     farClip    = _farClip;
 }
@@ -726,6 +808,26 @@ void Camera::Lens::generateMatrix(float xshear, float yshear, Matrix::value_type
             matrix[13] += yshear;
 
         break;
+
+	case Manual:
+
+		memcpy( matrix, _matrix, sizeof(Matrix::value_type[16]));
+
+		if(xshear || yshear)
+		{
+			if (matrix[3]!=0.0 || matrix[7]!=0.0 || matrix[11]!=0.0 || matrix[15]!=1.0)
+			{
+				// It's not an orthographic matrix so just assume a perspective shear
+				matrix[ 8] += -xshear;
+				matrix[ 9] += -yshear;
+			} 
+            else
+			{
+				 matrix[12] += xshear;
+				 matrix[13] += yshear;
+			}
+		}
+		break;
     }
 }
 
@@ -740,12 +842,6 @@ void Camera::Lens::_updateFOV()
 void Camera::setOffset( const Matrix::value_type matrix[16], double xshear, double yshear )
 {
     memcpy( _offset._matrix, matrix, sizeof(Matrix::value_type[16]));
-    //Matrix m = Matrix(_offset._matrix) * Matrix( _viewMatrix );
-    /*
-    Matrix m = Matrix( _viewMatrix ) * Matrix(_offset._matrix);
-    memcpy( _viewMatrix, m.ptr(), sizeof(Matrix::value_type[16]) );
-    */
-
     _offset._xshear = xshear;
     _offset._yshear = yshear;
 }
@@ -822,3 +918,39 @@ const Camera::FrameTimeStampSet &Camera::getFrameStats()
     return _frameStamps; 
 }
 
+bool Camera::removePreCullCallback( Callback *cb )
+{
+    return _removeCallback( preCullCallbacks, cb );
+}
+
+bool Camera::removePostCullCallback( Callback *cb )
+{
+    return _removeCallback( postCullCallbacks, cb );
+}
+
+bool Camera::removePreDrawCallback( Callback *cb ) 
+{
+    return _removeCallback( preDrawCallbacks, cb );
+}
+
+bool Camera::removePostDrawCallback( Callback *cb ) 
+{
+    return _removeCallback( postDrawCallbacks, cb );
+}
+
+bool Camera::removePostSwapCallback( Callback *cb )
+{
+    return _removeCallback( postSwapCallbacks, cb );
+}
+
+
+bool Camera::_removeCallback( std::vector < ref_ptr<Callback> > &callbackList, Callback *callback )
+{
+    std::vector < Producer::ref_ptr< Producer::Camera::Callback> >::iterator p;
+    p = std::find( callbackList.begin(), callbackList.end(), callback );
+    if( p == callbackList.end() )
+        return false;
+
+    callbackList.erase( p );
+    return true;
+}
