@@ -19,6 +19,17 @@ KeyboardProxy proxy;
 
 Keyboard *Keyboard::theKeyboard = 0L;
 
+
+KeyboardProxy::KeyboardProxy() 
+{ 
+    Keyboard::getSingleton(); 
+}
+
+KeyboardProxy::~KeyboardProxy() 
+{ 
+    delete Keyboard::theKeyboard; 
+}
+
 Keyboard::Keyboard()
 { 
     _keyModifier = 0;
@@ -29,14 +40,14 @@ Keyboard::Keyboard()
 // Force singleton
 void *Keyboard::operator new( size_t size )
 {
-    if( theKeyboard == NULL )
-        theKeyboard = (Keyboard *)malloc( size );
+    if( theKeyboard == 0L )
+        theKeyboard = ::new Keyboard;
     return (void *)theKeyboard;
 }
 
 Keyboard *Keyboard::getSingleton()
 {
-    if( theKeyboard == NULL )
+    if( theKeyboard == 0L )
         new Keyboard;
     return theKeyboard;
 }
@@ -60,6 +71,10 @@ KeyCharacter Keyboard::xkeyEventToKeyChar(Display *dpy, XKeyEvent &ev)
     unsigned long mod = 0;
     KeySym ks = XKeycodeToKeysym( dpy, ev.keycode, 0 );
 
+    static XComposeStatus state;
+    unsigned char keybuf[32];
+    XLookupString( &ev, (char *)keybuf, sizeof(keybuf), NULL, &state );
+
     if( ev.state & ShiftMask )
         mod |= KeyMod_Shift;
     if( ev.state & LockMask )
@@ -72,7 +87,18 @@ KeyCharacter Keyboard::xkeyEventToKeyChar(Display *dpy, XKeyEvent &ev)
         mod |= KeyMod_NumLock;
     if( ev.state & Mod4Mask )
         mod |= KeyMod_Super;
-    return _charmap[mod | _keymap[ks]];
+
+    KeyCharacter lookup_key  = KeyCharacter(keybuf[0]);
+    KeyCharacter charmap_key = KeyCharacter( _charmap[mod | _keymap[ks]] );
+
+    if ( charmap_key & 0xFF00 ) 
+    {
+        return charmap_key;
+    } 
+    else 
+    {
+        return lookup_key;
+    }
 }
 #endif
 
@@ -171,17 +197,14 @@ KeyCharacter Keyboard::win32KeyEventToKeyChar( LPARAM lParam, WPARAM wParam )
 }
 #endif
 
-#ifdef _OSX_CGL_IMPLEMENTATION
+#if defined(_OSX_CGL_IMPLEMENTATION) ||  defined(_OSX_AGL_IMPLEMENTATION) 
 KeyCharacter Keyboard::akeyEventToKeyChar(unsigned long key)
 {
-    if( key == kEscapeCharCode )
-            return KeyChar_Escape;
-
+    if (_keymap.count(key)!=0) return KeyCharacter(_charmap[_keymap[key]]);
     return KeyCharacter(key);
 }
 #endif
 
-
 void Keyboard::_initKeyboardKeyMap()
 {
 #ifdef _X11_IMPLEMENTATION
@@ -403,6 +426,44 @@ void Keyboard::_initKeyboardKeyMap()
     _keymap[VK_NUMPAD8      ] = Key_8;
     _keymap[VK_NUMPAD9      ] = Key_9;
 #endif
+
+#if defined(_OSX_CGL_IMPLEMENTATION) ||  defined(_OSX_AGL_IMPLEMENTATION) 
+    _keymap[kEscapeCharCode] = Key_Escape;
+    _keymap[kHomeCharCode] = Key_Home;
+    _keymap[kEnterCharCode] = Key_KP_Enter;
+    _keymap[kEndCharCode] = Key_End;
+    _keymap[kReturnCharCode] = Key_Return;
+    _keymap[kPageUpCharCode] = Key_Page_Up;
+    _keymap[kPageDownCharCode] = Key_Page_Down;
+    _keymap[kLeftArrowCharCode] = Key_Left;
+    _keymap[kRightArrowCharCode] = Key_Right;
+    _keymap[kUpArrowCharCode] = Key_Up;
+    _keymap[kDownArrowCharCode] = Key_Down;
+    _keymap[kBackspaceCharCode] = Key_BackSpace;
+    _keymap[kTabCharCode] = Key_Tab;
+    _keymap[kSpaceCharCode] = Key_space;
+    _keymap[kDeleteCharCode] = Key_Delete;
+
+/*    
+   _keymap[kHelpCharCode] = KeyChar_Help;
+   kLineFeedCharCode = 10,
+   kVerticalTabCharCode = 11,
+   kPageUpCharCode = 11,
+   kFormFeedCharCode = 12,
+   kPageDownCharCode = 12,
+   kReturnCharCode = 13,
+   kFunctionKeyCharCode = 16,
+   kCommandCharCode = 17,
+   kCheckCharCode = 18,
+   kDiamondCharCode = 19,
+   kAppleLogoCharCode = 20,
+   kClearCharCode = 27,
+   kSpaceCharCode = 32,
+   kDeleteCharCode = 127,
+   kBulletCharCode = 165,
+   kNonBreakingSpaceCharCode = 202
+*/   
+   #endif
 }
 
 void Keyboard::_initKeyCharMap()
