@@ -26,6 +26,10 @@
 #include <signal.h>
 #endif
 
+#ifdef _X11_IMPLEMENTATION
+#include <X11/XKBlib.h>
+#endif
+
 #include <iostream>
 #include <stdio.h>
 #include <vector>
@@ -38,7 +42,7 @@
 
 namespace Producer {
 
-class KeyboardMouseImplementationBase //: public Referenced
+class KeyboardMouseImplementationBase : public Referenced
 {
     public :
         KeyboardMouseImplementationBase(RenderSurface *rs): 
@@ -56,6 +60,7 @@ class KeyboardMouseImplementationBase //: public Referenced
             _cb(NULL),
             _canceled(false)
         {}
+
         float mx() { return _mx; }
         float my() { return _my; }
         unsigned int mbutton() { return _mbutton; }
@@ -63,25 +68,11 @@ class KeyboardMouseImplementationBase //: public Referenced
         virtual bool update(KeyboardMouseCallback &, bool ) = 0;
         virtual void startTimer() = 0;
         void setCallback(KeyboardMouseCallback *cb) { _cb = cb; }
+        virtual void setAutoRepeatMode( bool ) {}
+        virtual bool getAutoRepeatMode() {return true;}
 
-#if 0
-        void *run()
+        void cancel()
         {
-            if( _cb == NULL ) return (void *)1L;
-            /*
-            bool use_timer = _cb->idle();
-            if( use_timer )
-                startTimer();
-              */
-            while( !isCanceled() )
-            {
-                update(*_cb,true);
-            }
-            return 0L;
-        }
-#endif
-
-        void cancel(){
             _canceled = true;
         }
 
@@ -91,7 +82,7 @@ class KeyboardMouseImplementationBase //: public Referenced
             {
                 float minL = FLT_MAX;
                 int bestInputRectangle = 0;
-                for(int i=0;i<_inputArea->getNumRenderSurfaces();++i)
+                for(unsigned int i=0;i<_inputArea->getNumRenderSurfaces();++i)
                 {
                     RenderSurface::InputRectangle ir = _inputArea->getRenderSurface(i)->getInputRectangle();
                     float dx = (x-(ir.left()+ir.width()*0.5f));
@@ -111,36 +102,33 @@ class KeyboardMouseImplementationBase //: public Referenced
                 unsigned int w, h;
                 rs->getWindowRectangle( wx, wy, w, h );
                 
-                int ix = int( (float)w*(x - ir.left())/ir.width() );
-                int iy = int( (float)h*(y - ir.bottom())/ir.height() );
+                int ix = int( (float)(w-1)*(x - ir.left())/ir.width() + 0.5f );
+                int iy = int( (float)(h-1)*(y - ir.bottom())/ir.height() + 0.5f );
 
                 rs->positionPointer(ix, iy );
 
             }
             else
             {
-                /*
-                int wx, wy;
-                unsigned int w, h;
-                _rs->getWindowRectangle( wx, wy, w, h );
-                int ix = int((float)w*(x*0.5f+0.5f)); // assume x is in 0.0->1.0 range
-                int iy = int((float)h*(y*0.5f+0.5f)); // assume y is in 0.0->1.0 range
-                */
-
                 RenderSurface::InputRectangle ir = _rs->getInputRectangle();
                 unsigned int ww = _rs->getWindowWidth();
                 unsigned int wh = _rs->getWindowHeight();
-                int ix = int(((x - ir.left())/ir.width()) * float(ww));
-                int iy = int(((y - ir.bottom())/ir.height()) * float(wh));
+                int ix = int(((x - ir.left())/ir.width()) * float(ww-1)+ 0.5f );
+                int iy = int(((y - ir.bottom())/ir.height()) * float(wh-1)+ 0.5f );
                 _rs->positionPointer(ix, iy );
             }
         }
 
-        bool isCanceled(){
+        bool isCanceled()
+        {
             return _canceled;
         }
 
+        virtual void init() = 0;
+        virtual void fini() = 0;
+
     protected:
+
         virtual ~KeyboardMouseImplementationBase() {}
 
         RenderSurface *_rs;
@@ -150,21 +138,75 @@ class KeyboardMouseImplementationBase //: public Referenced
         float _mx, _my;
         unsigned int _mbutton;
 
-        virtual void init() = 0;
-        virtual void fini() = 0;
 
-        void transformMouseMotion( Window win, int wx, int wy )
+        void transformMouseMotion( Window win, int wx, int wy)
         {
-            if( _inputArea == NULL )
+            if( _inputArea )
             {
-                RenderSurface::InputRectangle ir = _rs->getInputRectangle();
-                unsigned int ww = _rs->getWindowWidth();
-                unsigned int wh = _rs->getWindowHeight();
-                _mx = ir.left() + (float(wx)/float(ww))*ir.width();
-                _my = ir.bottom() + (float(int(wh) - wy)/float(wh))*ir.height();
+                _inputArea->transformMouseMotion( win, wx, wy, _mx, _my);
             }
             else
-                _inputArea->transformMouseMotion( win, wx, wy, _mx, _my );
+            {
+            
+#if defined(_OSX_AGL_IMPLEMENTATION) || defined(_OSX_CGL_IMPLEMENTATION)  
+                int positionLeft = 0;
+                int positionTop = 0;
+                int windowWidth = _rs->getWindowWidth();
+                int windowHeight = _rs->getWindowHeight();
+
+                if (win == NULL)
+                {
+                    if (!_rs->isFullScreen())
+                    {
+                        // mouse pointer can't be in this window.
+                        // so don't update _mx, _my.
+                        return;
+                    }
+                
+                    // for full screen render surfaces get the bounds of the complete screen
+                    // and check to see if mouse pointer is within this.
+                    Display* display = _rs->getDisplay();
+                    CGRect theBounds = CGDisplayBounds(*display);
+                    if (wx >= theBounds.origin.x && 
+                        wy >= theBounds.origin.y &&
+                        wx <= (theBounds.origin.x + theBounds.size.width) &&
+                        wy <= (theBounds.origin.y + theBounds.size.height))
+                    {
+                        positionLeft = static_cast<int>(theBounds.origin.x);
+                        positionTop = static_cast<int>(theBounds.origin.y);
+                        windowWidth = static_cast<int>(theBounds.size.width);
+                        windowHeight = static_cast<int>(theBounds.size.height);
+                    }
+                    else
+                    {
+                        // mouse pointer can't be in this window
+                        // so don't update _mx, _my.
+                        return;
+                    }
+                } 
+                else
+                {
+                    Rect theBounds;
+                    GetWindowBounds (win, kWindowContentRgn, &theBounds);
+
+                    positionLeft = theBounds.left;
+                    positionTop = theBounds.top;
+                    windowWidth = theBounds.right-theBounds.left;
+                    windowHeight = theBounds.bottom-theBounds.top;
+
+                }
+#else
+                int positionLeft = 0;
+                int positionTop = 0;
+                int windowWidth = _rs->getWindowWidth();
+                int windowHeight = _rs->getWindowHeight();
+#endif
+
+                RenderSurface::InputRectangle ir = _rs->getInputRectangle();
+                _mx = ir.left() + (float(wx)/float(windowWidth))*ir.width();
+                _my = ir.bottom() + (float(windowHeight-(wy-positionTop+1))/float(windowHeight-1))*ir.height();
+
+            }
         }
 
         bool _canceled;
@@ -186,28 +228,20 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
 {
     public:
         KeyboardMouseImplementation(InputArea *ia):KeyboardMouseImplementationBase(ia) 
-        { init(); }
+        {
+            init();
+        }
 
         KeyboardMouseImplementation(RenderSurface *rs):KeyboardMouseImplementationBase(rs) 
-        { init(); }
+        {
+            init();
+        }
 
 
         bool update( KeyboardMouseCallback &cb, bool block=false)
         {
             if( block )
             {
-#if 0
-                XSync(dpy,0);
-                if( !XPending(dpy) )
-                {
-                    fd_set rfds;
-                    int fd = ConnectionNumber(dpy);
-                    FD_ZERO(&rfds);
-                    FD_SET( fd, &rfds );
-                    if( select(fd+1,&rfds,0,0,0) < 0 )
-                        return false;
-                }
-#endif
                 do{ 
                     process_event(cb);
                 }while( XPending(dpy));
@@ -240,9 +274,26 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
             setitimer(ITIMER_REAL, &itv, (struct itimerval *)0 );
         }
 
-    private:
-        Display *dpy;
-        long _inputMask;
+        virtual void setAutoRepeatMode( bool flag )
+        {
+            int supported;
+            if( flag == false )
+                XkbSetDetectableAutoRepeat( dpy, true, &supported );
+            else
+                XkbSetDetectableAutoRepeat( dpy, false, &supported );
+
+            if( supported == 0 )
+            {
+                std::cerr << "KeyboardMouse::setAutoRepeateMode("<<flag<<"): Warning- not supported." << std::endl;
+            }
+            else
+                _autoRepeatMode = flag;
+        }
+
+        virtual bool getAutoRepeatMode()
+        {
+            return _autoRepeatMode;
+        }
 
         virtual void init()    
         {
@@ -268,7 +319,7 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                     XDisplayName("") << std::endl;
                 return;
             }
-            
+
             _inputMask = StructureNotifyMask |
                          KeyPressMask | KeyReleaseMask |
                          PointerMotionMask  | ButtonPressMask | ButtonReleaseMask ;
@@ -288,8 +339,89 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                 XSelectInput( dpy, win, _inputMask );
             }
             _mbutton = 0;
+
+            //XGetKeyboardControl( dpy, &_originalKBState );
+            //_changeKBStateMask = 0L;
+        
+            // Possible inconsistency if global mode is set to off.
+            // Assumption here is that autoRepeatMode is true if we will 
+            // be testing at the client level for repeated keys.
+            _autoRepeatMode = true;
         }
 
+        virtual void fini() 
+        {
+            /*
+            if( _changeKBStateMask )
+            {
+                XKeyboardControl kc;
+                if( _changeKBStateMask & KBAutoRepeatMode )
+                    kc.auto_repeat_mode = _originalKBState.global_auto_repeat;
+                XChangeKeyboardControl( dpy, _changeKBStateMask, &kc );
+            }
+            */
+
+            // This should only be called from the run() thread if 
+            // we are multi-threaded
+            XSync(dpy,0);
+            XCloseDisplay(dpy);
+            cancel();
+        }
+
+
+    protected:
+
+        virtual ~KeyboardMouseImplementation()
+        {}
+
+    private:
+        Display *dpy;
+        long _inputMask;
+        bool _autoRepeatMode;
+
+        class KeymapState {
+            public:
+                KeymapState()
+                {
+                    memset( _keys,0, sizeof(_keys));
+                }
+
+                KeymapState(char keys[32])
+                {
+                    memcpy( _keys, keys, sizeof( _keys ));
+                }
+
+                KeymapState( const KeymapState &ks )
+                {
+                    memcpy( _keys, ks._keys, sizeof( _keys ));
+                }
+
+                void operator = (char keys[32])
+                {
+                    memcpy( _keys, keys, sizeof( _keys ));
+                }
+
+                bool operator == (char keys[32])
+                {
+                    unsigned int *lhs = (unsigned int *)&_keys[0];
+                    unsigned int *rhs = (unsigned int *)&keys[0];
+
+                    return (lhs[0] == rhs[0] &&
+                            lhs[1] == rhs[1] &&
+                            lhs[2] == rhs[2] &&
+                            lhs[3] == rhs[3] &&
+                            lhs[4] == rhs[4] &&
+                            lhs[5] == rhs[5] &&
+                            lhs[6] == rhs[6] &&
+                            lhs[7] == rhs[7] );
+                }
+            private:
+                char _keys[32];
+        };
+
+        KeymapState _keymapState;
+
+
         void process_event( KeyboardMouseCallback &cb)
         {
             XEvent ev;
@@ -369,6 +501,17 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
     
                 case KeyPress:
                     {
+                        // Check for auto repeated keys and autoRepeat flag;
+                        if( _autoRepeatMode == false )
+                        {
+                            char keys[32];
+                            XQueryKeymap( dpy, keys );
+
+                            if( _keymapState == keys ) // We have an autorepeat.
+                               break;
+                           _keymapState = keys; 
+                        }
+
                         KeyCharacter keychar = Keyboard::getSingleton()->xkeyEventToKeyChar(dpy, ev.xkey);
                         if( keychar != KeyChar_Unknown )
                         {
@@ -382,23 +525,26 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
     
                 case KeyRelease:
                     {
+                        if( _autoRepeatMode == false )
+                        {
+                            char keys[32];
+                            XQueryKeymap( dpy, keys );
+                           _keymapState = keys; 
+                        }
+
                         KeyCharacter keychar = Keyboard::getSingleton()->xkeyEventToKeyChar(dpy, ev.xkey);
                         if( keychar != KeyChar_Unknown )
                         {
-                        if( keychar & 0xFF00 )
-                            cb.specialKeyRelease(keychar);
-                        else
-                            cb.keyRelease(keychar);
+                            if( keychar & 0xFF00 )
+                                cb.specialKeyRelease(keychar);
+                            else
+                                cb.keyRelease(keychar);
                         }
                     }
                     break;
             }
         }
 
-        virtual void fini()
-        {
-            XCloseDisplay(dpy);
-        }
         static void s_trap( int, siginfo_t *, void *) {}
 };
 
@@ -418,7 +564,6 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
         KeyboardMouseImplementation(RenderSurface *rs):KeyboardMouseImplementationBase(rs) 
         { init(); }
 
-        virtual ~KeyboardMouseImplementation() { fini(); }
 
         bool update( KeyboardMouseCallback &cb, bool block=false)
         {/* no blocking for now
@@ -427,11 +572,13 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                 q->waitWhileEmpty();
             }
             */
+            //HACK!!! - This is a work-around for background threads that continuously
+            // poll the input queue as the above "waitWhileEmpty()" is not fully functional.
+            if( block )
+                Sleep(16);
             std::vector< Producer::ref_ptr<EventQueue> >::iterator p;
             for( p = qs.begin(); p != qs.end(); p++ )
             {
-                //HACK!!!
-                Sleep(16);
                 EventQueue *q = (*p).get();
                 while( ! q->empty() )
                 {
@@ -549,9 +696,6 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
             ::SetTimer(_rs->getWindow(),TIMER_ID, 10, NULL );
         }
 
-
-    private:
-
         virtual void init()    
         {
             unsigned int mask = 
@@ -568,7 +712,7 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
             if( _inputArea != NULL )
             {
             
-                for( int i = 0; i < _inputArea->getNumRenderSurfaces(); i++ )
+                for( unsigned int i = 0; i < _inputArea->getNumRenderSurfaces(); i++ )
                 {
                     RenderSurface *rs = _inputArea->getRenderSurface(i);
                     EventQueue *q = rs->selectInput(mask);
@@ -584,11 +728,21 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
             _mbutton = 0;
         }
 
-        virtual void fini()
+        void fini()
         {
             cancel();
         }
 
+
+    protected:
+
+        virtual ~KeyboardMouseImplementation() 
+        {}
+
+    private:
+
+
+
         std::vector< Producer::ref_ptr<EventQueue> > qs;
 };
 
@@ -596,8 +750,7 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
 
 #endif
 
-
-#ifdef _OSX_CGL_IMPLEMENTATION
+#if defined(_OSX_AGL_IMPLEMENTATION) || defined(_OSX_CGL_IMPLEMENTATION) 
 
 namespace Producer {
 
@@ -610,143 +763,176 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
         KeyboardMouseImplementation(RenderSurface *rs):KeyboardMouseImplementationBase(rs) 
         { init(); }
 
-        virtual ~KeyboardMouseImplementation() { fini(); }
+        virtual ~KeyboardMouseImplementation() {}
+
+        bool handleMouseEvent(KeyboardMouseCallback &cb, EventRef theEvent)
+        {
+            
+            // mouse down event                         
+            Point wheresMyMouse;
+            GetEventParameter (theEvent, kEventParamMouseLocation, typeQDPoint, NULL, sizeof(wheresMyMouse), NULL, &wheresMyMouse);
+
+            WindowRef win;
+            int fwres = FindWindow(wheresMyMouse, &win);
+
+            if (fwres == inDrag)
+            {
+                return false;
+            }
+            else
+            {
+
+                EventMouseButton mouseButton;
+                GetEventParameter (theEvent, kEventParamMouseButton, typeMouseButton, NULL, sizeof(mouseButton), NULL, &mouseButton);
+
+                if (mouseButton==1)
+                {
+                    UInt32 modKey = GetCurrentKeyModifiers();
+                    if (modKey & 0x100)
+                    {
+                        mouseButton = 3;
+                    }
+                    else if (modKey & 0x800)
+                    {
+                        mouseButton = 2;
+                    }
+                }
+                else
+                {
+                    if (mouseButton==2) mouseButton=3;
+                    else if (mouseButton==3) mouseButton=2; 
+                }
+                switch(GetEventKind(theEvent))
+                {
+                    case kEventMouseDown:
+                        cb.buttonPress(_mx,_my, mouseButton);
+                        break;
+                    case kEventMouseUp:
+                        cb.buttonRelease(_mx,_my, mouseButton);
+                        break;
+                    case kEventMouseDragged:
+                        break;
+                    case kEventMouseMoved:
+                        break;
+                    default:
+                        return false;
+                }
+            
+                if( _ox != wheresMyMouse.h || _oy != wheresMyMouse.v )
+                {
+                    _ox = wheresMyMouse.h;
+                    _oy = wheresMyMouse.v;
+                    
+                    transformMouseMotion(win, _ox, _oy);
+                    
+                    if( _mbutton )
+                         cb.mouseMotion(_mx, _my);
+                    else
+                         cb.passiveMouseMotion(_mx, _my);
+                }            
+                return true;
+            }
+        }
+        
+        bool handleKeyboardEvent(KeyboardMouseCallback &cb, EventRef theEvent)
+        {
+            UInt32 modifierKeys;
+            GetEventParameter (theEvent,kEventParamKeyModifiers,typeUInt32, NULL,sizeof(modifierKeys), NULL,&modifierKeys);
+            
+            UInt32 dataSize;
+            GetEventParameter( theEvent, kEventParamKeyUnicodes, typeUnicodeText, NULL, 0, &dataSize, NULL );
+            if (dataSize<=1) return false;
+            
+            UniChar* uniChars = new UniChar[dataSize+1];
+            GetEventParameter( theEvent, kEventParamKeyUnicodes, typeUnicodeText, NULL, dataSize, NULL, (void*)uniChars );
+           
+            KeyCharacter keychar = Keyboard::getSingleton()->akeyEventToKeyChar(static_cast<unsigned long>(uniChars[0]));
+            
+            switch(GetEventKind(theEvent))
+            {
+                case kEventRawKeyDown:
+                {
+                     if( keychar & 0xFF00 )
+                        cb.specialKeyPress(keychar);
+                     else
+                        cb.keyPress(keychar);
+                     break;
+                }
+                case kEventRawKeyRepeat:
+                {
+                     if( keychar & 0xFF00 )
+                        cb.specialKeyPress(keychar);
+                     else
+                        cb.keyPress(keychar);
+                     break;
+                }
+                case kEventRawKeyUp:
+                {
+                     if( keychar & 0xFF00 )
+                        cb.specialKeyRelease(keychar);
+                     else
+                        cb.keyRelease(keychar);
+                     break;
+                }
+                case kEventRawKeyModifiersChanged:
+                     break;
+                case kEventHotKeyPressed:
+                     break;
+                case kEventHotKeyReleased:
+                     break;
+                default:
+                    break;
+            
+            }
+           return true;
+        }
 
         bool update( KeyboardMouseCallback &cb, bool block=false)
-		{
-    		EventRecord event;
-			bool res;
-			while( EventAvail( everyEvent, &event ) )
-			{
-    		if( (res = WaitNextEvent( everyEvent, &event, 0, NULL )))                                                 
-    		{   
-        		switch( event.what )
-        		{
-            		case osEvt : 
-                		break;                                                                                
-        
-            		case updateEvt:
-                		break;
-    
-            		case kHighLevelEvent:                                                                     
-                		AEProcessAppleEvent(&event);
-                		break;
-    
-            		case keyDown:
-						{
-							KeyCharacter keychar = Keyboard::getSingleton()->akeyEventToKeyChar(event.message & charCodeMask);
-                            if( keychar & 0xFF00 )
-                                cb.specialKeyPress(keychar);
-                            else
-                                cb.keyPress(keychar);
-						}
-						break;
-
-            		case keyUp:                                                                               
-						{
-							KeyCharacter keychar = Keyboard::getSingleton()->akeyEventToKeyChar(event.message & charCodeMask);
-                            if( keychar & 0xFF00 )
-                                cb.specialKeyRelease(keychar);
-                            else
-                                cb.keyRelease(keychar);
-						}
-                		break;                                                                                
-                                                                                                      
-            		case mouseDown:                                                                           
-						{
-							unsigned int ui = *(unsigned int *)&event.where;
-							short x = (ui&0x0000FFFF);
-							short y = (ui&0xFFFF0000) >> 16;
-							transformMouseMotion( 0L, x,y);
-							unsigned int b = 1;
-							if( event.modifiers & 0xF000 )
-								b = event.modifiers & 0x1000 ? 3 : 1;
-
-							if( event.modifiers & 0xF00 )
-								b = event.modifiers & 0x100 ? 2 :
-									event.modifiers & 0x800 ? 3 : 1;
-
-//printf( "Mouse Down: button %d (modifiers = 0x%x)\n", b, event.modifiers );
-
-							_lastButtonPressed = b;
-							_mbutton |= (1<<(b-1));
-							cb.buttonPress(_mx,_my, b);
-						}
-                		break;                                                                                
-
-            		case mouseUp:                                                                           
-						{
-							unsigned int ui = *(unsigned int *)&event.where;
-							short x = (ui&0x0000FFFF);
-							short y = (ui&0xFFFF0000) >> 16;
-							transformMouseMotion( 0L, x,y);
-
-							/*
-							unsigned int b = 1;
-							if( event.modifiers & 0xF000 )
-								b = event.modifiers & 0x1000 ? 3 : 1;
-
-							if( event.modifiers & 0xF00 )
-								b = event.modifiers & 0x100 ? 2 :
-									event.modifiers & 0x800 ? 3 : 1;
-							
-printf( "Mouse Up: button %d (modifiers = 0x%x)\n", b, event.modifiers );
-
-							_mbutton &= ~(1<<(b-1));
-							cb.buttonRelease(_mx,_my,b);
-							*/
-							// Ugh.... CGL does not provide a hint about
-							// which button was released.  So... we hope
-							// it was the last button pressed
-							cb.buttonRelease(_mx,_my,_lastButtonPressed);
-						}
-						break;
-                                                                                                      
-            		default :                                                                                 
-                		break;                                                                                
-        		}                                                                                             
-    		}                                                                                                 
-			}
-			{
-				Point pt;
-				GetMouse(&pt);
-				unsigned int ui = *(unsigned int *)&event.where;
-				short x = (ui&0x0000FFFF);
-				short y = (ui&0xFFFF0000) >> 16;
-				if( _ox != x || _oy != y )
-				{
-					_ox = x;
-					_oy = y;
-					transformMouseMotion(0, x, y);
-                    if( _mbutton )
-                          cb.mouseMotion(_mx, _my);
-                      else
-                          cb.passiveMouseMotion(_mx, _my);
-				}
-			}
-
-//usleep(16000);
-
-			return true;
-		}
+        {
+            EventRef theEvent;
+            EventTargetRef theTarget = GetEventDispatcherTarget();
+            while (ReceiveNextEvent(0, NULL, 0,true, &theEvent)== noErr)
+            {
+                switch(GetEventClass(theEvent))
+                {
+                    case kEventClassMouse:
+                        if (!handleMouseEvent(cb, theEvent)) {}
+                            SendEventToEventTarget (theEvent, theTarget);
+                        break;
+                        
+                    case kEventClassKeyboard:
+                        handleKeyboardEvent(cb, theEvent);
+                        break;
+                        
+                    default:
+                        SendEventToEventTarget (theEvent, theTarget);
+                        break;
+                }
+                        
+                ReleaseEvent(theEvent);
+            }  
+            
+            return true;
+            
+        }
 
         virtual void startTimer() {}
 
-
-    private:
-
-		short _ox, _oy;
-		int _lastButtonPressed;
-
         virtual void init()    
         {
-			_lastButtonPressed = 0;
+            _lastButtonPressed = 0;
         }
 
         virtual void fini()
         {
         }
 
+
+    private:
+
+        short _ox, _oy;
+        int _lastButtonPressed;
+
 };
 
 }
@@ -787,26 +973,26 @@ KeyboardMouse::~KeyboardMouse()
         //std::cout << "waiting for keyboard mouse to cancel"<<std::endl;
         OpenThreads::Thread::YieldCurrentThread();
     }
-
-    if( _implementation != NULL )
-        delete _implementation;
 }
 
 bool KeyboardMouse::init( void )
 {
-    if( _initialized ) return _initialized;
+    if( _initialized ) 
+        return _initialized;
 
     if( _inputArea != NULL )
     {
-        _inputArea->getRenderSurface(0)->waitForRealize();
-        if( _inputArea->isRealized() == false ) return false;
-            _implementation = new KeyboardMouseImplementation(_inputArea.get() );
+        while( _inputArea->isRealized() == false ) 
+            _inputArea->waitForRealize();
+
+        _implementation = new KeyboardMouseImplementation(_inputArea.get() );
     }
     else
     {
         _rs->waitForRealize();
-        if( _rs->isRealized() == false ) return false;
-            _implementation = new KeyboardMouseImplementation(_rs.get() );
+        if( _rs->isRealized() == false ) 
+            return false;
+        _implementation = new KeyboardMouseImplementation(_rs.get() );
     }
 
     _initialized = true;
@@ -837,7 +1023,7 @@ bool KeyboardMouse::computePixelCoords(float x, float y, RenderSurface* rs, floa
     if( _inputArea != NULL )
     {
         // first locate which InputRectange is appropriate for specified RenderSurface.
-        int i;
+        unsigned int i;
         for(i=0;i<_inputArea->getNumRenderSurfaces();++i)
         {
             if (_inputArea->getRenderSurface(i)==rs) break;
@@ -892,6 +1078,22 @@ bool KeyboardMouse::computePixelCoords(float x, float y, RenderSurface* rs, floa
     }
 }
 
+void KeyboardMouse::setAutoRepeatMode( bool flag )
+{
+    if( !_initialized && !init() ) 
+            return;
+    _implementation->setAutoRepeatMode(flag);
+}
+
+bool KeyboardMouse::getAutoRepeatMode()
+{
+    if( !_initialized && !init() ) 
+            return false;
+    return _implementation->getAutoRepeatMode();
+}
+
+
+
 void KeyboardMouse::run()
 {
     if( _cb == NULL )
@@ -899,44 +1101,29 @@ void KeyboardMouse::run()
         std::cerr << "KeyboardMouse: internal thread cannot start because\n"
                      "no callback has been specified.  Please specify a \n"
                      "callback with KeyboardMouse::setCallback() first.\n";
-        //return (void *)1L;
         return;
     }
 
     if( !_initialized  )
     {
-        // Init() will fail if the render surface
-        // or input Area is not realized.
-        // Check every 5th of a second until it
-        // is realized.
-            
-        // April 1, 2003 DB - This loop should no 
-        // longer happen.  Init should succeed the
-            // first time since the mplementation of
-            // waitForRealize() in RenderSurface
-        while( !init() )
-#ifdef WIN32 
-            Sleep(200);
-#else
-            usleep(200000);
-#endif
+        if( !init() )
+        {
+            std::cerr << "KeyboardMouse::run() - Internal error!" << std::endl;
+            return;
+        }
     }
 
     _implementation->setCallback( _cb.get() );
 
-    //_implementation->run();
-
-
-     /*
-     bool use_timer = _cb->idle();
-     if( use_timer )
-             startTimer();
-    */
     while( !_implementation->isCanceled() )
     {
         _implementation->update(*_cb,true);
         testCancel();
     }
+
+    // We never really get here.  Our thread is canceled before 
+    // this executes.  Thats... OK.
+    _implementation->fini();
 }
 
 
