@@ -25,46 +25,13 @@
 #include <Producer/Keyboard>
 #include "WGLExtensions.h"
 
-// Use the GLUT CURSOR defines for now
-#define  GLUT_CURSOR_RIGHT_ARROW            0x0000
-#define  GLUT_CURSOR_LEFT_ARROW             0x0001
-#define  GLUT_CURSOR_INFO                   0x0002
-#define  GLUT_CURSOR_DESTROY                0x0003
-#define  GLUT_CURSOR_HELP                   0x0004
-#define  GLUT_CURSOR_CYCLE                  0x0005
-#define  GLUT_CURSOR_SPRAY                  0x0006
-#define  GLUT_CURSOR_WAIT                   0x0007
-#define  GLUT_CURSOR_TEXT                   0x0008
-#define  GLUT_CURSOR_CROSSHAIR              0x0009
-#define  GLUT_CURSOR_UP_DOWN                0x000A
-#define  GLUT_CURSOR_LEFT_RIGHT             0x000B
-#define  GLUT_CURSOR_TOP_SIDE               0x000C
-#define  GLUT_CURSOR_BOTTOM_SIDE            0x000D
-#define  GLUT_CURSOR_LEFT_SIDE              0x000E
-#define  GLUT_CURSOR_RIGHT_SIDE             0x000F
-#define  GLUT_CURSOR_TOP_LEFT_CORNER        0x0010
-#define  GLUT_CURSOR_TOP_RIGHT_CORNER       0x0011
-#define  GLUT_CURSOR_BOTTOM_RIGHT_CORNER    0x0012
-#define  GLUT_CURSOR_BOTTOM_LEFT_CORNER     0x0013
-#define  GLUT_CURSOR_INHERIT                0x0064
-#define  GLUT_CURSOR_NONE                   0x0065
-#define  GLUT_CURSOR_FULL_CROSSHAIR         0x0066
-
-    // Set the cursor AND change it for this window class. 
-#define MAP_CURSOR(a,b) case a: SetCursor( LoadCursor(NULL,b) ); \
-SetClassLong(_win,GCL_HCURSOR,LONG(LoadCursor(NULL,b))); \
-break;
-    // Nuke the cursor AND change it for this window class. 
-#define ZAP_CURSOR(a,b) case a: SetCursor(NULL); \
-SetClassLong(_win,GCL_HCURSOR,LONG(NULL)); \
-break;
-
 
 using namespace Producer;
 
 // Class statics
 std::map <Window, RenderSurface *>RenderSurface::registry;
 
+unsigned int RenderSurface::getNumberOfScreens(void) { return 1; }
 
 RenderSurface::Client::Client(unsigned long mask): _mask(mask)
 {
@@ -112,9 +79,9 @@ void RenderSurface::dispatch( ref_ptr<Event> ev )
 #endif
 
         if( (*p)->mask() & (1<<ev->type()) )
-                {                
+        {                
             (*p)->queue( ev ); 
-                }
+        }
     } 
 }
 
@@ -130,17 +97,18 @@ LONG WINAPI RenderSurface::proc( Window hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 #ifndef WM_MOUSEWHEEL
 #define WM_MOUSEWHEEL 522
 #endif
-        case WM_MOUSEWHEEL:
-                {
-                        KeyboardMouseCallback::ScrollingMotion scroll = 
-                                (signed int)(wParam) < 0 ? KeyboardMouseCallback::ScrollDown:
-                                KeyboardMouseCallback::ScrollUp;
-                        ev = new MouseScrollEvent(hWnd,scroll);
-                        dispatch(ev);
-                }
-                if (!_ownWindow)
-                        lRet = CallWindowProc(_oldWndProc, hWnd, uMsg, wParam, lParam);
-                break;
+    case WM_MOUSEWHEEL:
+        {
+            KeyboardMouseCallback::ScrollingMotion scroll = 
+                    (signed int)(wParam) < 0 ? KeyboardMouseCallback::ScrollDown:
+                    KeyboardMouseCallback::ScrollUp;
+            ev = new MouseScrollEvent(hWnd,scroll);
+            dispatch(ev);
+        }
+        if (!_ownWindow)
+            lRet = CallWindowProc(_oldWndProc, hWnd, uMsg, wParam, lParam);
+         break;
+
     case WM_MOUSEMOVE :
         _mx = ((int)(short)LOWORD(lParam));
         _my = ((int)(short)HIWORD(lParam));
@@ -371,7 +339,7 @@ LONG WINAPI RenderSurface::proc( Window hWnd, UINT uMsg, WPARAM wParam, LPARAM l
         {
             if (!_ownWindow)
                 lRet = CallWindowProc(_oldWndProc, hWnd, uMsg, wParam, lParam);
-            _setCursor();
+            _setCursor(_currentCursor);
         }
         else
         {
@@ -502,7 +470,6 @@ bool RenderSurface::_init()
         _screenWidth  = dm.dmPelsWidth;
         _screenHeight = dm.dmPelsHeight;
 
-        _setCursor(GLUT_CURSOR_LEFT_ARROW);
 
         if (_ownWindow )
         {
@@ -510,13 +477,16 @@ bool RenderSurface::_init()
             DWORD dwStyle;
             WNDCLASS wndclass;
 
+            _defaultCursor = LoadCursor( NULL, IDC_ARROW);
+            _currentCursor = _defaultCursor;
+
             wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC; 
             wndclass.lpfnWndProc   = (WNDPROC)s_proc;
             wndclass.cbClsExtra    = 0;
             wndclass.cbWndExtra    = 0;
             wndclass.hInstance     = _hinstance;
             wndclass.hIcon         = LoadIcon( _hinstance, "PRODUCER_ICON" );
-            wndclass.hCursor       = LoadCursor (NULL,IDC_ARROW);
+            wndclass.hCursor       = _defaultCursor;
             wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
             wndclass.lpszMenuName  = 0;
 
@@ -591,10 +561,20 @@ bool RenderSurface::_init()
                 dwExStyle = WS_EX_APPWINDOW;
                 dwStyle   = WS_POPUP | WS_MAXIMIZE;
 
-                x = 0;
-                y = 0;
-                w = _screenWidth;
-                h = _screenHeight;
+                if (_useCustomFullScreen)
+                {
+                    x = _customFullScreenOriginX;
+                    y = _screenHeight - (_customFullScreenOriginY + _customFullScreenHeight);
+                    w = _customFullScreenWidth;
+                    h = _customFullScreenHeight;
+                }
+                else
+                {
+                    x = 0;
+                    y = 0;
+                    w = _screenWidth;
+                    h = _screenHeight;
+                }
             }
             else // own window, no parent, not fullscreen
             {
@@ -672,6 +652,7 @@ bool RenderSurface::_init()
             return FALSE;
         }
     }
+    _setCursor(_defaultCursor);
 
     ref_ptr<TemporaryWindow> tempwnd;
 
@@ -701,7 +682,8 @@ bool RenderSurface::_init()
     else
     {
         lvc = new VisualChooser;
-        lvc->setSimpleConfiguration();
+        bool doublebuffer = (_drawableType == DrawableType_Window);
+        lvc->setSimpleConfiguration(doublebuffer);
     }
 
     if (_drawableType == DrawableType_PBuffer)
@@ -837,7 +819,7 @@ bool RenderSurface::_init()
 
     if (_drawableType == DrawableType_Window)
     {
-        _setCursor();
+        _setCursor(_currentCursor);
         ShowWindow(_win, SW_NORMAL );
         UpdateWindow(_win );
         SetForegroundWindow(_win);
@@ -984,6 +966,7 @@ bool RenderSurface::makeCurrent( void ) const
 
 void RenderSurface::swapBuffers( void )
 {
+    makeCurrent();
     SwapBuffers( _hdc );
 }
 
@@ -1099,56 +1082,90 @@ void RenderSurface::_resizeWindow()
 
 void RenderSurface::_setBorder( bool flag )
 {
-    std::cout << "RenderSurface::_setBorder( "<< flag << " ) : Warning - Not implemented yet\n";
+	//Flags that will be used for SetWindowPos
+	UINT flags = SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOSENDCHANGING |
+                    SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED;
 
+	//Get current style of window
+	DWORD style = GetWindowLong(_win, GWL_STYLE);
+
+	//Get current client dimensions
+	RECT rect;
+	GetClientRect( _win, &rect );
+
+	//Variables that will hold new size of window
+	int cx, cy;
+
+	if(flag)
+	{
+		//Add border style to window
+		style &= ~WS_POPUP;
+        style |= WS_OVERLAPPEDWINDOW;
+
+		//Calculate new size of window with border
+		rect.left = 0;
+		rect.top = 0;
+		rect.right = rect.right - rect.left;
+		rect.bottom = rect.bottom - rect.top;
+		
+		LONG styleEx = GetWindowLong(_win,GWL_EXSTYLE);
+		AdjustWindowRectEx(&rect, style, FALSE, styleEx);
+
+		cx = rect.right - rect.left;
+		cy = rect.bottom - rect.top;
+		flags &= ~(SWP_NOSIZE);
+	}
+	else
+	{
+		//Remove border style from window
+		style &= ~WS_OVERLAPPEDWINDOW;
+		style |= WS_POPUP;
+
+		//Calculate new size of window without border
+		cx = rect.right - rect.left;
+		cy = rect.bottom - rect.top;
+		flags &= ~(SWP_NOSIZE);
+	}
+
+	//Apply new window style
+	SetWindowLong(_win, GWL_STYLE, style);
+	
+	//Need to call this to change window size and update frame style
+	SetWindowPos(_win, 
+					HWND_TOP,
+					0, 0, 
+					cx, cy, 
+					flags);
 }
 
-void RenderSurface::_setCursor()
+void RenderSurface::_setWindowName( const std::string & name )
 {
-    POINT point;
-    GetCursorPos(&point);
-    if (_win == WindowFromPoint(point))
+    SetWindowText(_win, name.c_str());
+}
+
+void RenderSurface::_useCursor(bool flag)
+{
+    _useCursorFlag = flag;
+    if( _realized )
+        _setCursor(_currentCursor);
+}
+
+void RenderSurface::_setCursor(Cursor cursor)
+{
+    if( _useCursorFlag == false )
     {
-        if( _useCursor )
-        {
-            _setCursor(_curCursor);
-        } 
-        else 
-        {
-            _setCursor(GLUT_CURSOR_NONE);
-        }
+        ::SetCursor(_nullCursor);
+    }
+    else
+    {
+        _currentCursor = cursor ? cursor : _nullCursor;
+        ::SetCursor(_currentCursor);
     }
 }
 
-void RenderSurface::_setCursor(int cursorID)
+void RenderSurface::_setCursorToDefault()
 {
-    /*
-     * This is a temporary solution only...
-     */
-    switch( cursorID )
-    {
-        MAP_CURSOR( GLUT_CURSOR_RIGHT_ARROW, IDC_ARROW     );
-        MAP_CURSOR( GLUT_CURSOR_LEFT_ARROW,  IDC_ARROW     );
-        MAP_CURSOR( GLUT_CURSOR_INFO,        IDC_HELP      );
-        MAP_CURSOR( GLUT_CURSOR_DESTROY,     IDC_CROSS     );
-        MAP_CURSOR( GLUT_CURSOR_HELP,        IDC_HELP       );
-        MAP_CURSOR( GLUT_CURSOR_CYCLE,       IDC_SIZEALL   );
-        MAP_CURSOR( GLUT_CURSOR_SPRAY,       IDC_CROSS     );
-        MAP_CURSOR( GLUT_CURSOR_WAIT,         IDC_WAIT      );
-        MAP_CURSOR( GLUT_CURSOR_TEXT,        IDC_UPARROW   );
-        MAP_CURSOR( GLUT_CURSOR_CROSSHAIR,   IDC_CROSS     );
-        ZAP_CURSOR( GLUT_CURSOR_NONE,        NULL       );
-
-        default:
-            MAP_CURSOR( GLUT_CURSOR_UP_DOWN,     IDC_ARROW     );
-    }
-    /*
-     * Remember the currently selected cursor
-     */
-    if (cursorID != GLUT_CURSOR_NONE)
-    {
-        _curCursor = cursorID;
-    }
+    _setCursor(_defaultCursor);
 }
 
 void RenderSurface::_positionPointer(int x, int y)
@@ -1238,5 +1255,15 @@ void RenderSurface::_choosePixelFormat( VisualInfo *)
 	//TODO
 }
 
-   
+void RenderSurface::_useOverrideRedirect( bool )
+{
+    // Ignored on Windows.  Flag value remains false.
+}
+
+void RenderSurface::_initThreads()
+{
+    // Stub
+}
+
+
 #endif  // WIN32
