@@ -12,6 +12,7 @@
  */
 
 
+#include <OpenThreads/ScopedLock>
 #include <Producer/RenderSurface>
 
 #ifdef _X11_IMPLEMENTATION
@@ -21,6 +22,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
+#include <X11/cursorfont.h>
 
 
 using namespace Producer; 
@@ -39,8 +41,13 @@ void (*glXGetProcAddressARB(const GLubyte *procName))(void)
 #define glXGetProcAddressARB glXGetProcAddress
 #endif
 
-#if (defined(__APPLE__) || defined(macintosh)) 
-static GLXFBConfig _glXGetFBConfigFromVisual(Display *dpy, XVisualInfo *visinfo)
+
+#if (defined(__APPLE__) || defined(macintosh)) || defined(_AIX)
+  #define USE_LOCAL_glXGetFBConfigFromVisual 
+#endif
+
+#ifdef USE_LOCAL_glXGetFBConfigFromVisual
+static GLXFBConfig _glXGetFBConfigFromVisual(::Display *dpy, XVisualInfo *visinfo)
 {
     int screen = visinfo->screen;
     int nelements;
@@ -56,20 +63,43 @@ static GLXFBConfig _glXGetFBConfigFromVisual(Display *dpy, XVisualInfo *visinfo)
     }
 	return NULL;
 }
-
 #endif
 
+unsigned int RenderSurface::getNumberOfScreens()
+{
+    if( _numScreens == UnknownAmount )
+    {
+    	Display *dpy = XOpenDisplay("");
+    	if( dpy == NULL )
+    	{
+        	std::cerr << "Unable to open display \"" << XDisplayName( ":0.0") << "\".  Is the DISPLAY environmental variable set?\n";
+        	return 0;
+    	}
+    	_numScreens = ScreenCount(dpy); 
+    	XCloseDisplay(dpy);
+    }
+    return _numScreens;
+}
 
-bool RenderSurface::realize( VisualChooser *vc, GLContext shared_context )
+static OpenThreads::Mutex realize_mutex;
+
+bool RenderSurface::realize( VisualChooser *vc, GLContext sharedGLContext )
 {
     if( _realized ) return _realized;
 
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(realize_mutex);
 
     if( vc != NULL )
         _visualChooser = vc;
-    _sharedGlcontext = shared_context;
 
-    _init();
+
+    if( sharedGLContext != 0L )
+        _sharedGLContext = sharedGLContext;
+    else
+        _sharedGLContext = _globallySharedGLContext;
+
+    if( _init() == false )
+        return false;
 
     if( _drawableType == DrawableType_Window )
     {
@@ -98,15 +128,15 @@ bool RenderSurface::realize( VisualChooser *vc, GLContext shared_context )
 #endif
     testVSync();
 
-#ifdef USE_OVERRIDE_REDIRECT
+    if( _overrideRedirectFlag == true )
     {
         XFlush(_dpy);
         XSync(_dpy,0);
+        usleep(500000); // Hack to allow window manager to catch up 
         XSetInputFocus(  _dpy, _win, RevertToParent, CurrentTime );
         XFlush(_dpy);
         XSync(_dpy,0);
     }
-#endif
 
     _realized = true;
  
@@ -188,7 +218,7 @@ bool RenderSurface::_init()
             }
         }
     }
-    
+
     if( _visualInfo == NULL )
     {
         if( _dpy_created_locally )
@@ -198,8 +228,9 @@ bool RenderSurface::_init()
         }
         return false;
     }
-        
-    if((_glcontext = glXCreateContext( _dpy, _visualInfo, _sharedGlcontext, GL_TRUE ))==(GLXContext) 0L)
+
+
+    if((_glcontext = glXCreateContext( _dpy, _visualInfo, _sharedGLContext, GL_TRUE ))==(GLXContext) 0L)
     {
         std::cerr << "RenderSurface:realize() - Can't create GLX Context." << std::endl;
         if( _dpy_created_locally )
@@ -210,6 +241,15 @@ bool RenderSurface::_init()
         return false;
     }
 
+    // Initialize globallySharedGLContext if global sharing is requested and 
+    // the global variable is still NULL.
+    // Caveat is if the user attempted to share GL contexts already through realize()
+    if( _shareAllGLContexts == true && 
+            _globallySharedGLContext == 0L &&
+            _sharedGLContext == 0L )
+    {
+        _globallySharedGLContext = _sharedGLContext = _glcontext;
+    }
 
     if( _win == 0 )
     {
@@ -267,10 +307,11 @@ bool RenderSurface::_init()
             swatt.border_pixel = 0;
             swatt.event_mask =  0;
             unsigned long mask = CWBackPixel | CWBorderPixel | CWEventMask | CWColormap;
-#ifdef USE_OVERRIDE_REDIRECT
-            swatt.override_redirect = true;
-            mask |= CWOverrideRedirect;
-#endif
+            if( _overrideRedirectFlag == true )
+            {
+                swatt.override_redirect = true;
+                mask |= CWOverrideRedirect;
+            }
             _win = XCreateWindow( _dpy, _parent,
                                   _windowX,
                                   _parentWindowHeight - (_windowHeight + _windowY),
@@ -294,6 +335,11 @@ bool RenderSurface::_init()
             // This turns off border decorations
             if( _decorations == false )
                 _setBorder(_decorations);
+
+            // Create  default Cursor
+            {
+                _defaultCursor = XCreateFontCursor( _dpy, XC_left_ptr );
+            }
     
             // Create Null Cursor
             {
@@ -304,12 +350,12 @@ bool RenderSurface::_init()
                 pixmap = XCreateBitmapFromData( _dpy, DefaultRootWindow(_dpy), buff, 1, 1);
                _nullCursor = XCreatePixmapCursor( _dpy, pixmap, pixmap, &ncol, &ncol, 0, 0 );
             }
-            _setCursor();
+            _setCursorToDefault();
         }
         else if( _drawableType == DrawableType_PBuffer )
         {
-
-#if (defined(__APPLE__) || defined(macintosh)) 
+#ifdef GLX_VERSION_1_3
+#ifdef USE_LOCAL_glXGetFBConfigFromVisual 
             GLXFBConfig fbconfig = _glXGetFBConfigFromVisual( _dpy, _visualInfo );
 #else
             GLXFBConfig fbconfig = glXGetFBConfigFromVisualSGIX( _dpy, _visualInfo );
@@ -328,6 +374,10 @@ bool RenderSurface::_init()
             attributeList.push_back( 0L );
 
             _win = glXCreatePbuffer(_dpy, fbconfig, &attributeList.front() );
+#else
+            _win = 0L;
+            return false;
+#endif
         }
     }
     else
@@ -372,7 +422,7 @@ bool RenderSurface::_init()
     else
         _isFullScreen = false;
 
-    if( !_useConfigEventThread )
+    if( !_useConfigEventThread && _drawableType == DrawableType_Window )
         XSelectInput( _dpy, _win, ExposureMask | StructureNotifyMask );
 
     XFlush( _dpy );
@@ -385,7 +435,11 @@ bool RenderSurface::makeCurrent( void ) const
 
     if( _readDrawableRenderSurface != 0L && _readDrawableRenderSurface->isRealized())
     {
+#ifdef GLX_VERSION_1_3
         glXMakeContextCurrent(_dpy, _win, _readDrawableRenderSurface->getWindow(), _glcontext );
+#else
+       glXMakeCurrent( _dpy, _win, _glcontext );
+#endif
     }
     else
         glXMakeCurrent( _dpy, _win, _glcontext );
@@ -442,6 +496,15 @@ void RenderSurface::testVSync( void )
     if( __glxGetRefreshRateSGI != NULL )
     __glxGetRefreshRateSGI(&_refreshRate);
 
+    std::string emsg(
+        "Warning:  Producer::RenderSurface, under the X11 environment\n"
+        "          has tested the glXWaitVideoSyncSGI extension and found it\n"
+        "          to not be working properly.  This does not necessarily mean\n"
+        "          that glXSwapBuffers will not block on VSYNC, but it does mean\n"
+        "          that the CPU bound application will not be able to synchronize\n"
+        "          with the graphics video retrace signal.  Most operations will work\n"
+        "          fine.");
+        
     // Prior reference to a bug here in the nVidia drivers is moot.
     // Bug now seems to be fixed.
     if( __glxGetVideoSyncSGI != NULL && __glxWaitVideoSyncSGI != NULL )
@@ -455,13 +518,13 @@ void RenderSurface::testVSync( void )
             __glxWaitVideoSyncSGI( 1, 0, &frame );
 
             if( frame != (_frameCount + 2) )
-                std::cerr << "Warning... Video Sync not working correctly\n";
+                std::cerr << emsg << std::endl;
         }
         else
-            std::cerr << "Warning... Video Sync not working correctly (glXGetVideoSyncSGI() failed)\n";
+            std::cerr << emsg << std::endl;
     }
     else
-        std::cerr << "Warning... Video Sync capability not available\n";
+        std::cerr << emsg << std::endl;
 
     _frameCount = frame;
 }
@@ -471,6 +534,7 @@ bool RenderSurface::_checkEvents( Display *dpy )
     bool ret = true;
     XEvent ev;
     XNextEvent( dpy, &ev );
+
 // Window manager will reparent and reset the x,y location relative
 // to the window manager's border window.  On a reparent, we set 
 // this flag.  Seems to work but .....
@@ -492,7 +556,7 @@ static bool _ignore_resize_flag_hack = false;
                     _inputRectangle.set( 0.0, float(_windowWidth), 0.0, float (_windowHeight));
 
            }
-           _ignore_resize_flag_hack = false;
+_ignore_resize_flag_hack = false;
         break;
 
         case GravityNotify :
@@ -514,7 +578,7 @@ static bool _ignore_resize_flag_hack = false;
             break;
 
         case ReparentNotify:
-_ignore_resize_flag_hack = true;    
+            _ignore_resize_flag_hack = true;    
             break;
 
         case DestroyNotify :
@@ -572,7 +636,11 @@ void RenderSurface::run( void )
 
 void RenderSurface::fullScreen( bool flag )
 {
-    if( flag == _isFullScreen ) return;
+    if( _drawableType != DrawableType_Window )
+        return;
+
+    if( flag == _isFullScreen ) 
+        return;
 
     if( flag )
     {
@@ -585,10 +653,7 @@ void RenderSurface::fullScreen( bool flag )
 
         _isFullScreen = true;
         _setBorder(false);
-        XFlush(_dpy); XSync(_dpy,0);
-        XUnmapWindow(_dpy, _win );
-        XFlush(_dpy); XSync(_dpy,0);
-        usleep(100000); // sleep to try and give the window manager a chance to resize properly - accounting for bugggy window manager..
+
         if( _useCustomFullScreen )
         {
             XMoveResizeWindow( _dpy, _win, 
@@ -603,10 +668,7 @@ void RenderSurface::fullScreen( bool flag )
         }
 
         XFlush(_dpy); XSync(_dpy,0);
-        XMapWindow(_dpy, _win );
-        XFlush(_dpy); XSync(_dpy,0);
-        XSetInputFocus(_dpy, _win, RevertToNone, CurrentTime );
-//    XFlush(_dpy); XSync(_dpy,0);
+        usleep(100000); // Hack to allow window manager to catch up 
     }
     else
     {
@@ -614,32 +676,33 @@ void RenderSurface::fullScreen( bool flag )
         _resizeWindow();
         _setBorder(_decorations);
         XFlush(_dpy); XSync(_dpy,0);
+        usleep(100000); // Hack to allow window manager to catch up 
     }
 }
 
 void RenderSurface::_resizeWindow()
 {
-    XFlush(_dpy); XSync(_dpy,0);
-    XUnmapWindow(_dpy,_win);
-    XFlush(_dpy); XSync(_dpy,0);
-    usleep(100000); // sleep to try and give the window manager a chance to resize properly - accounting for bugggy window manager..
+    if( _drawableType != DrawableType_Window )
+        return;
 
     XMoveResizeWindow( _dpy, _win, 
                     _windowX, 
                     _screenHeight - (_windowY + _windowHeight), 
                     _windowWidth, _windowHeight );
     XFlush(_dpy); XSync(_dpy,0);
-    XMapWindow(_dpy, _win);
-    XFlush(_dpy); XSync(_dpy,0);
+    usleep(100000); // Hack to allow window manager to do its thing.
 }
 
 void RenderSurface::_setBorder( bool flag )
 {
+    if( _drawableType != DrawableType_Window )
+        return;
+
     Atom atom;
     if( (atom = XInternAtom( _dpy, "_MOTIF_WM_HINTS", 0 )) != None )
     {
 // Hack for sending 64 bit atom to Xserver
-#if defined( _MIPS_SIM) && (_MIPS_SIM == _MIPS_SIM_ABI64) || defined ( __ia64 )
+#if defined( _MIPS_SIM) && (_MIPS_SIM == _MIPS_SIM_ABI64) || defined ( __ia64 ) || defined (__amd64 )
         struct {
             CARD32 flags0;
             CARD32 flags1;
@@ -653,7 +716,7 @@ void RenderSurface::_setBorder( bool flag )
             CARD32 status1;
         } wmHints;
 
-#if defined( __ia64 )
+#if defined( __ia64 ) || defined (__amd64)
         wmHints.flags0       = (1L << 1);
         wmHints.functions0   = 0;
         wmHints.decorations0 = flag;
@@ -681,8 +744,10 @@ void RenderSurface::_setBorder( bool flag )
         wmHints.decorations = flag;
         wmHints.input_mode  = 0;
 #endif
+        XUnmapWindow(_dpy, _win );
         XChangeProperty( _dpy, _win, atom, atom, 32, PropModeReplace,
                 (unsigned char *)&wmHints,  5 );
+        XMapWindow(_dpy, _win );
 
         XFlush(_dpy);
         XSync(_dpy,0);
@@ -692,13 +757,55 @@ void RenderSurface::_setBorder( bool flag )
          "RenderSurface::setBorder(" << flag << ") - Warning : Couldn't change decorations." << std::endl;
 }
 
-void RenderSurface::_setCursor()
+void RenderSurface::_setWindowName( const std::string &name )
 {
-    if( _useCursor == false )
+    if( !_realized )
+        return;
+
+    if( _drawableType != DrawableType_Window )
+        return;
+
+
+    char *slist[] = { (char *)name.c_str(), 0L };
+    XTextProperty xtp;
+
+    XStringListToTextProperty( slist, 1, &xtp );
+    XSetWMName( _dpy, _win, &xtp );
+    XFlush(_dpy); XSync(_dpy,0);
+}
+
+void RenderSurface::_useCursor(bool flag)
+{
+    if( _drawableType != DrawableType_Window )
+        return;
+
+    _useCursorFlag = flag;
+    if( _realized )
+        _setCursor(_currentCursor);
+}
+
+void RenderSurface::_setCursor(Cursor cursor)
+{
+    if( _drawableType != DrawableType_Window )
+        return;
+
+    if( _useCursorFlag == false )
+    {
         XDefineCursor( _dpy, _win, _nullCursor );
+    }
     else
-        XUndefineCursor( _dpy, _win );
+    {
+        _currentCursor = cursor ? cursor : _nullCursor;
+            XDefineCursor( _dpy, _win, _currentCursor );
+    }
     XFlush(_dpy);
+    XSync(_dpy,0);
+}
+
+void RenderSurface::_setCursorToDefault()
+{
+    _setCursor(_defaultCursor);
+    //XUndefineCursor( _dpy, _win );
 }
 
 void RenderSurface::_positionPointer( int x, int y )
@@ -706,6 +813,9 @@ void RenderSurface::_positionPointer( int x, int y )
     if( !_realized )
         return;
 
+    if( _drawableType != DrawableType_Window )
+        return;
+
     if( _isFullScreen )
         y = _screenHeight - y;
     else
@@ -730,6 +840,9 @@ void RenderSurface::_computeScreenSize( unsigned int &width, unsigned int &heigh
 
 void RenderSurface::mapWindow()
 {
+    if( _drawableType != DrawableType_Window )
+        return;
+
     if( _realized && _parent == RootWindow( _dpy, _screen ) )
     {
         XMapWindow( _dpy, _win );
@@ -740,6 +853,9 @@ void RenderSurface::mapWindow()
 
 void RenderSurface::unmapWindow()
 {
+    if( _drawableType != DrawableType_Window )
+        return;
+
     if( _parent == RootWindow( _dpy, _screen ) )
     {
         XUnmapWindow( _dpy, _win );
@@ -760,7 +876,9 @@ void RenderSurface::_fini()
         }
         else if( _drawableType == DrawableType_PBuffer )
         {
+#ifdef GLX_VERSION_1_3
             glXDestroyPbuffer( _dpy, _win );
+#endif
         }
         XFlush( _dpy );
         XSync(_dpy,0);
@@ -780,5 +898,78 @@ void RenderSurface::_choosePixelFormat( VisualInfo *)
 	//TODO
 }
 
+void RenderSurface::_useOverrideRedirect( bool flag)
+{
+    if( _drawableType != DrawableType_Window )
+        return;
+
+    if( _realized )
+    {
+        std::cerr << "RenderSurface::_useOverrideRedirect(" << flag << "): WARNING - Has no effect after realize() has been called" << std::endl;
+        return;
+    }
+    else
+        _overrideRedirectFlag = flag;
+}
+
+void RenderSurface::_initThreads()
+{
+	// For multi-threaded X environment we must initialize the
+	// multi-threaded support before we do anything else -- LJE
+	static bool initialized = false;    // We shouldn't do this multiple times...
+
+    if( initialized )
+        return;
+
+	initialized = true;
+
+#ifdef GLX_SUN_init_threads
+    Display *dpy = XOpenDisplay(":0.0");
+    if( dpy == NULL )
+    {
+        std::cerr << "Unable to open display \"" << XDisplayName( ":0.0") << "\".  Is the DISPLAY environmental variable set?\n";
+        exit(1);
+    }
+    if ( strstr ( glXGetClientString (dpy, GLX_EXTENSIONS ), "GLX_SUN_init_threads" ) )
+    {
+        if ( ! glXInitThreadsSUN ( ) )
+        {
+            std::cerr <<  "Unable to init GL for multithreaded rendering\n" << std::endl;
+            exit(1);
+        }
+        else
+        {
+            std::cout << "Initialized Threads...\n";
+        }
+    }
+    XCloseDisplay(dpy);
+#else
+     // InitThreads is only required for Sun (for some unknown reason).  
+     // XInitThreads man page reads:
+     //
+     //   It is only necessary to call this function if multiple threads might use Xlib concurrently.  If all
+     //   calls to Xlib functions are protected by some other access mechanism (for example, a mutual exclusion
+     //   lock in a toolkit or through explicit client programming), Xlib thread initialization is not required.
+     //   It is recommended that single-threaded programs not call this function.
+     //
+     // Since OSG enforces a policy of one-thread per client, this is not necessary and, potentially, detrimental
+     // Don Burns - May 3, 2005
+
+#if 0
+    if (XInitThreads() == 0)
+    {
+        std::cerr << "Error in RenderSurface::InitThreads(): XInitThreads() failed. Aborting." << std::endl;
+        exit(1);
+    }
+    else
+    {
+        std::cout << "RenderSurface::InitThreads(): multi-threaded X support initialized.\n";
+    }
+#endif
+
+#endif
+
+}
+
 #endif
 
